<?php

namespace App\Http\Controllers\Ads;

use App\Http\Controllers\Controller;

//use Elastic\Elasticsearch\ClientBuilder;
use Elastic\Elasticsearch\ClientBuilder;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use function GuzzleHttp\Promise\all;
use App\Traits\ElasticFunctions;


class CategoryController extends Controller
{    use ElasticFunctions;


    public function __construct()
    {

        $this->client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'),env('ELASTIC_PASSWORD'))
            ->build();
        //$this->middleware('check_role:CATEGORIES_LIST')->only(['index']);
        $this->middleware('check_role:CATEGORIES_CREATE')->only(['store,store2']);
        //$this->middleware('check_role:CATEGORIES_LIST')->only(['show','showCatById','showCatArchive']);
//        $this->middleware('check_role:CATEGORIES_UPDATE')->only(['update']);
//        $this->middleware('check_role:CATEGORIES_DELETE')->only(['destroy']);


    }



    public function show(Request $request)
    {
        $client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'),env('ELASTIC_PASSWORD'))
            ->build();

        $must = [];

        if(isset($request->language))
        {
            $m = [
                "match" => [
                    "language" => $request->language,
                ]
            ];
            array_push($must, $m);
        }

        $size = $request->size ?? 100;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'categories',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "must" => $must,
                    ]
                ],
                "sort" => [
                    ['priority' => 'asc' ]
                ]
            ],
        ];
        $response = $client->search($params);

        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' =>$size,
                'currentpage' =>$page,
                'totalpage' =>ceil($response['hits']['total']['value']/$size),
                'totalrows' =>$response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];
    }



    public function showCatById(Request $request)
    {
        $params = [
            'index' => 'categories',
            'body' => [
                'query' => [
                    "match" => [
                        "_id" => $request->cat_id,
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        return $response['hits']['hits'];
    }




    public function showCatArchive(Request $request)
    {
        $params = [
            'index' => 'categories',
            'body' => [
                'query' => [
                    "match_all" => [
                        "boost" => 1.0,
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        $cats = $response['hits']['hits'];
        $result = [];
        foreach ($cats as $cat) {
            $params = [
                'index' => 'products',
                'body' => [
                    'query' => [
                        "match" => [
                            "cat_name.keyword" => $cat["_source"]["slug"]
                        ],
                    ]
                ]
            ];
            $response = $this->client->search($params);
            $pcount = $response['hits']['total']['value'];
            $total = [$cat,$pcount];
            array_push($result,$total);
        }
        return $result;
    }


    public function store(Request $request)
    {
        $client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'),env('ELASTIC_PASSWORD'))
            ->build();

        $params = [
            'index' => 'categories',
            'body' => [
                'query' => [
                    'match' => [
                        'title' => $request[0]['title']
                    ]
                ]
            ]
        ];
        $response = $client->search($params);
        if ($response['hits']['hits']) {
            if($response['hits']['hits'][0]['_source']['title'] == $request[0]['title'])
            {
                abort(400, trans('messages.cat_title_fail'));
            }
        }

        $params = [
            'index' => 'categories',
            'body' => [
                'query' => [
                    'match' => [
                        'cat_name' => $request[0]['cat_name']
                    ]
                ]
            ]
        ];
        $response = $client->search($params);
        if ($response['hits']['hits']) {
            if($response['hits']['hits'][0]['_source']['cat_name'] == $request[0]['cat_name'])
            {
                abort(400, trans('messages.cat_title_fail'));
            }
        }
        $params = [];
        $temp = $request->all();
//        $temp[0]['language'] = ($request->hasHeader('X-localization')) ? $request->header('X-localization') : env('DEFAULT_LANG');
        unset($temp['admin']);
        unset($temp['id']);
        foreach ($temp as $item){
            if(isset($item['cat_name'])){
                $item['slug'] = str_replace(" ", '-', $item['cat_name']);
            }
            if(isset($item['sub'])){
                for($i = 0 ; $i < count($item['sub']) ; $i++){
                    $item['sub'][$i]['slug'] = str_replace(" ", '-', $item['sub'][$i]['cat_name']);
                    if( ! empty($item['sub'][$i]['sub']) )
                    {
                        for ($j = 0 ; $j < count($item['sub'][$i]['sub']) ; $j++)
                        {
                            $item['sub'][$i]['sub'][$j]['slug'] = str_replace(" ", '-', $item['sub'][$i]['sub'][$j]['cat_name']);
                        }
                    }
                }
            }
            $params['body'][] = [
                'index' => [
                    '_index' => 'categories'
            ]
            ];
            $params['body'][] = $item;
        }
        $response = $client->bulk($params);
    }

    public function update(Request $request)
    {
        $req = $request->all();
//        $req['object']['language'] = ($request->hasHeader('X-localization')) ? $request->header('X-localization') : env('DEFAULT_LANG');

        if(isset($request->input('object')['title']))
        {
            $params = [
                'index' => 'categories',
                'body' => [
//                    'query' => [
//                        'match' => [
//                            'title' => $request->input('object')['title']
//                        ]
//                    ],
                    'query' => [
                        'bool' => [
                            'must' => [
                                [
                                    'match' => [
                                        'title' => $request->input('object')['title']
                                    ],
                                ]
                            ],
                            'must_not' =>
                                [
                                    'match' => [
                                        '_id' => $request->input('id')
                                    ],
                                ]
                        ]
                    ],
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {
                if($response['hits']['hits'][0]['_source']['title'] == $request->input('object')['title'])
                {
                    abort(400, trans('messages.cat_title_fail'));
                }
            }
        }

        if(isset($request->input('object')['cat_name']))
        {
            $params = [
                'index' => 'categories',
                'body' => [
//                    'query' => [
//                        'match' => [
//                            'cat_name' => $request->input('object')['cat_name']
//                        ]
//                    ],
                    'query' => [
                        'bool' => [
                            'must' => [
                                [
                                    'match' => [
                                        'cat_name' => $request->input('object')['cat_name']
                                    ],
                                ]
                            ],
                            'must_not' =>
                                [
                                    'match' => [
                                        '_id' => $request->input('id')
                                    ],
                                ]
                        ]
                    ],
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {

                if($response['hits']['hits'][0]['_source']['cat_name'] == $request->input('object')['cat_name'])
                {
                    abort(400, trans('messages.cat_title_fail'));
                }
            }
            $req['object']['slug'] = str_replace(" ", '-', $request->all()['object']['cat_name']);
        }
        if(isset($req['object']['sub'])){
            for($i = 0 ; $i < count($req['object']['sub']) ; $i++){
                $req['object']['sub'][$i]['slug'] = str_replace(" ", '-', $req['object']['sub'][$i]['cat_name']);
                if( ! empty($req['object']['sub'][$i]['sub']) )
                {
                    for ($j = 0 ; $j < count($req['object']['sub'][$i]['sub']) ; $j++)
                    {
                        $req['object']['sub'][$i]['sub'][$j]['slug'] = str_replace(" ", '-', $req['object']['sub'][$i]['sub'][$j]['cat_name']);
                    }
                }
            }
        }

        $params = [
            'index' => 'categories',
            'id' => $request->input('id'),
            'body' => [
                'doc'=> $req['object'],
            ]
        ];

        $response = $this->client->update($params);


    }


    public function destroy(Request $request)
    {
        $params = [
            'index' => 'categories',
            'id'    => $request->input('id')
        ];

        $response = $this->client->delete($params);

    }


}
