<?php

namespace App\Http\Controllers\Ads;

use App\Http\Controllers\Controller;
use Elastic\Elasticsearch\ClientBuilder;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use function GuzzleHttp\Promise\all;
use App\Traits\ElasticFunctions;


class  FormController extends Controller
{

   use ElasticFunctions;


    public function showAll(Request $request)
    {
        $size = $request->size ?? 100;
        $page = $request->page ?? 1;
        $must = [];
        if(isset($request->language))
        {
            $m = [
                "match" => [
                    "language" => $request->language,
                ]
            ];
            array_push($must, $m);
        }
        if(isset($request->cat_name) && !empty($request->cat_name))
        {
            $m =[
                'match' => [
                    'cat_name' => $request->cat_name,
                ]
            ];
            array_push($must, $m);

        }

        $params = [
            'index' => 'forms',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    'bool' => [
                        'must' => $must,
                    ]
                ],
            ],
        ];
        $response = $this->client->search($params);

        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' =>$size,
                'currentpage' =>$page,
                'totalpage' =>ceil($response['hits']['total']['value']/$size),
                'totalrows' =>$response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];
    }

    public function show(Request $request)
    {
        $params = [];
        if($request->cat_name){
            $params = [
                'index' => 'forms',
                'body' => [
                    'query' => [
                        'match' => [
                            'cat_name.keyword' => $request->input('cat_name')
                        ]
                    ]
                ]
            ];
        }elseif($request->cat_slug){
            $params = [
                'index' => 'forms',
                'body' => [
                    'query' => [
                        'match' => [
                            'cat_slug.keyword' => $request->input('cat_slug')
                        ]
                    ]
                ]
            ];
        }

        $response = $this->client->search($params);


        if($response['hits']['hits']){
            return $response['hits']['hits'][0]; // documents
        } else{
            return trans('messages.cat_notfound');
        }


    }


    public function showByCategory(Request $request)
    {

        $params = [
            'index' => 'forms',
            'body' => [
                'query' => [
                    'match' => [
                        'cat_name' => $request->input('cat_name')
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        return $response['hits']; // documents
    }


    public function store(Request $request)
    {
        $data = [
            'language' => ($request->hasHeader('X-localization')) ? $request->header('X-localization') : env('DEFAULT_LANG'),
            'cat_name' => $request->input('cat_name'),
            'items' => $request->input('items'),
            'cat_slug' => null
        ];
//        array_walk_recursive($data['items']['form']['options'], function(&$string){
//            $string = (string) $string;
//        });
//        Log::info($data['items']['form']['options']);
        if(isset($data['items']['category']))
        {
            $data['cat_name'] = $data['items']['category'];
            $params = [
                'index' => 'categories',
                'body' => [
                    'query' => [
                        'bool' => [
                            "must" => [
                                [
                                    'match' => [
                                        'cat_name' => $data['items']['category'],
                                    ]
                                ],
                            ],
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {
                $data['cat_slug'] = $response['hits']['hits'][0]['_source']['slug'];
            }
        }

        if(isset($data['items']['subCategory']))
        {
            $data['cat_name'] = $data['items']['subCategory'];
            $params = [
                'index' => 'categories',
                'body' => [
                    'query' => [
                        'bool' => [
                            "must" => [
                                [
                                    'match' => [
                                        'sub.cat_name' => $data['items']['subCategory'],
                                    ]
                                ],
                            ],
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {
                foreach ($response['hits']['hits'][0]['_source']['sub'] As $sub)
                {
                    if($sub['cat_name'] == $data['items']['subCategory'])
                    {
                        $data['cat_slug'] = $sub['slug'];
                    }
                }

            }
        }

        if(isset($data['items']['innerCategory']))
        {
            $data['cat_name'] = $data['items']['innerCategory'];
            $params = [
                'index' => 'categories',
                'body' => [
                    'query' => [
                        'bool' => [
                            "must" => [
                                [
                                    'match' => [
                                        'sub.sub.cat_name' => $data['items']['innerCategory'],
                                    ]
                                ],
                            ],
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {
                foreach ($response['hits']['hits'][0]['_source']['sub'] As $subs)
                {
                    if(isset($subs['sub']))
                    {
                        foreach ($subs['sub'] as $sub)
                        {
                            if($sub['cat_name'] == $data['items']['innerCategory'])
                            {
                                $data['cat_slug'] = $sub['slug'];
                            }
                        }
                    }
                }

            }
        }
        $params = [];
        $params = [
            'index' => 'forms',
            'mappings' => [
                'properties' => [
                    'items.form.options.options.value' => [
                        'type' => 'text'
                    ]
                ]
            ],
            'body' => $data
        ];

            $response = $this->client->index($params);

    }


    public function update(Request $request)
    {
        $data = [
            'cat_name' => null,
            'language' => ($request->hasHeader('X-localization')) ? $request->header('X-localization') : env('DEFAULT_LANG'),
            'items' => $request->input('items'),
            'cat_slug' => null
        ];
        if(isset($data['items']['category']))
        {
            $data['cat_name'] = $data['items']['category'];
            $params = [
                'index' => 'categories',
                'body' => [
                    'query' => [
                        'bool' => [
                            "must" => [
                                [
                                    'match' => [
                                        'cat_name' => $data['items']['category'],
                                    ]
                                ],
                            ],
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {
                $data['cat_slug'] = $response['hits']['hits'][0]['_source']['slug'];
            }
        }

        if(isset($data['items']['subCategory']))
        {
            $data['cat_name'] = $data['items']['subCategory'];
            $params = [
                'index' => 'categories',
                'body' => [
                    'query' => [
                        'bool' => [
                            "must" => [
                                [
                                    'match' => [
                                        'sub.cat_name' => $data['items']['subCategory'],
                                    ]
                                ],
                            ],
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {
                foreach ($response['hits']['hits'][0]['_source']['sub'] As $sub)
                {
                    if($sub['cat_name'] == $data['items']['subCategory'])
                    {
                        $data['cat_slug'] = $sub['slug'];
                    }
                }

            }
        }

        if(isset($data['items']['innerCategory']))
        {
            $data['cat_name'] = $data['items']['innerCategory'];
            $params = [
                'index' => 'categories',
                'body' => [
                    'query' => [
                        'bool' => [
                            "must" => [
                                [
                                    'match' => [
                                        'sub.sub.cat_name' => $data['items']['innerCategory'],
                                    ]
                                ],
                            ],
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);
            if ($response['hits']['hits']) {
                foreach ($response['hits']['hits'][0]['_source']['sub'] As $subs)
                {
                    if(isset($subs['sub']))
                    {
                        foreach ($subs['sub'] as $sub)
                        {
                            if($sub['cat_name'] == $data['items']['innerCategory'])
                            {
                                $data['cat_slug'] = $sub['slug'];
                            }
                        }
                    }
                }

            }
        }
        $params = [
            'index' => 'forms',
            'id' => $request->input('id'),
            'body' => [
                'doc' => $data
            ]
        ];


            $response = $this->client->update($params);


    }


    public function destroy(Request $request)
    {
        $params = [
            'index' => 'forms',
            'id' => $request->input('id')
        ];


            $response = $this->client->delete($params);
    }


}
