<?php

namespace App\Http\Controllers\Ads;

use App\Http\Controllers\ClientResponseException;
use App\Http\Controllers\Controller;
use App\Http\Controllers\Exception;
use App\Http\Controllers\ServerResponseException;
use App\Models\Business;
use App\Models\Favorite;
use App\Models\LastVisit;
use App\Models\Note;
use App\Models\Feedback;
use App\Models\Support;
use App\Models\Token;
use App\Models\Chat;
use App\Models\User;
use Carbon\Carbon;
use Elastic\Elasticsearch\ClientBuilder;
use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use App\Traits\ElasticFunctions;


class ProductController extends Controller
{

    public function __construct()
    {
        $this->client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'),env('ELASTIC_PASSWORD'))
            ->build();

//        $this->middleware('check_role:PRODUCTS_LIST')->only(['index']);
//        $this->middleware('check_role:CATEGORIES_CREATE')->only(['store,store2']);
//        $this->middleware('check_role:CATEGORIES_LIST')->only(['show','showCatById','showCatArchive']);
//        $this->middleware('check_role:PRODUCTS_UPDATE')->only(['update']);
//        $this->middleware('check_role:PRODUCTS_DELETE')->only(['destroy']);
    }
    use ElasticFunctions;


    public function show3(Request $request)
    {
        $params = [
            'index' => 'products',
            'body'  => [
                'query' => [
                    'match' => [
                        "object.mains." . "city" => '117'
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);

        printf("Total docs: %d\n", $response['hits']['total']['value']);
        printf("Max score : %.4f\n", $response['hits']['max_score']);
        printf("Took      : %d ms\n", $response['took']);

        print_r($response['hits']['hits']); // documents
    }
    public function show(Request $request)
    {
        $m = [];
        $must = [];
        $must_not = [];

        if(isset($request->ads_type) && $request->ads_type == "shop")
        {
            $type = [
                'range' => [
                    'object.shop' => [
                        'gt' => 0
                    ]
                ]
            ];
            array_push($must, $type);
        }
        if(isset($request->ads_type) && $request->ads_type == "personal")
        {
            $type = [
                'range' => [
                    'object.shop' => [
                        'gt' => 0
                    ]
                ]
            ];
            array_push($must_not, $type);
        }
        if(isset($request->title))
        {
            $m = [
                "match" => [
                    "object.mains." . "title" => $request->title,
                ]
            ];
            array_push($must, $m);
        }
        if(isset($request->language))
        {
            $m = [
                "match" => [
                    "lang" => $request->language,
                ]
            ];
            array_push($must, $m);
        }
        if(isset($request->city_id))
        {
            $m = [
                "match" => [
                    "object.mains." . "city_id" => $request->city_id,
                ]
            ];
            array_push($must, $m);
        }

        if(isset($request->location))
        {
            $loc = json_decode($request->location, true);
            $m = [
                "bool" => [
                    "must" => [
                        [
                            "range" => [
                                "object.mains.location.latitude" => [
                                    "gt" => $loc['latitude'] - 5,
                                    "lt" => $loc['latitude'] + 5
                                ]
                            ]
                        ], [
                            "range" => [
                                "object.mains.location.longitude" => [
                                    "gt" => $loc['longitude'] - 5,
                                    "lt" => $loc['longitude'] + 5
                                ]
                            ]
                        ],
                    ],
                ]
            ];
            array_push($must, $m);
        }

        $s = [
            'match' => [
                'object.' . 'status' => 'accepted',
            ]
        ];
        array_push($must, $s);
        foreach ($request->header() as $key => $value) {
            if (($key !='city_id') && ($key !='location') && ($key !='title')) {
                continue;
            }
            if (($key == 'city_id')) {
                $m = [
                    "match" => [
                        "object.mains." . "city_id" => $value[0],
                    ]
                ];
                array_push($must, $m);

            } if (($key === 'location')) {
                $loc = json_decode($value, true);
                $m = [
                    "bool" => [
                        "must" => [
                            [
                                "range" => [
                                    "object.mains.location.latitude" => [
                                        "gt" => $loc['latitude'] - 5,
                                        "lt" => $loc['latitude'] + 5
                                    ]
                                ]
                            ], [
                                "range" => [
                                    "object.mains.location.longitude" => [
                                        "gt" => $loc['longitude'] - 5,
                                        "lt" => $loc['longitude'] + 5
                                    ]
                                ]
                            ],
                        ],
                    ]
                ];
                array_push($must, $m);

            }
            if (($key === 'title')) {
                $m = [
                    "match" => [
                        "object.mains." . "title" => $value[0],
                    ]
                ];
                array_push($must, $m);
            }

//            $m = [
//                "range" => [
//                    "object.renewal_at" => [
//                        "gte" => Carbon::now()
//                    ],
//                ]
//            ];
        }
        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "filter" => [
                            [
                                "range" => [
                                    "object.renewal_at" => [
                                        "gte" => Carbon::now()->subDay(30)
                                    ]
                                ]
                            ]
                        ],
                        "must" => $must,
                        "must_not" => $must_not,
                    ]
                ],
                "sort" => [
                    ['object.updated_at' => 'desc' ]
                ]
            ],
        ];
        $response = $this->client->search($params);
//        $c = count($response['hits']['hits']);
//        for($i=0;$i<$c;$i++)
//        {
//            $renewDate = $response['hits']['hits'][$i]['_source']['object']['renewal_at'];
//            if(strtotime(date('Y-m-d')) > strtotime(date('Y-m-d', strtotime($renewDate))))
//            {
//                $response['hits']['hits'][$i] = 0;
//            }
//        }
        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];


    }

    public function showPending(Request $request)
    {
        $m = [];
        $must = [];
        $must_not = [];

        if(isset($request->ads_type) && $request->ads_type == "shop")
        {
            $type = [
                'range' => [
                    'object.shop' => [
                        'gt' => 0
                    ]
                ]
            ];
            array_push($must, $type);
        }
        if(isset($request->ads_type) && $request->ads_type == "personal")
        {
            $type = [
                'range' => [
                    'object.shop' => [
                        'gt' => 0
                    ]
                ]
            ];
            array_push($must_not, $type);
        }
        if(isset($request->title))
        {
            $m = [
                "match" => [
                    "object.mains." . "title" => $request->title,
                ]
            ];
            array_push($must, $m);
        }
        if(isset($request->city_id))
        {
            $m = [
                "match" => [
                    "object.mains." . "city_id" => $request->city_id,
                ]
            ];
            array_push($must, $m);
        }

        if(isset($request->location))
        {
            $loc = json_decode($request->location, true);
            $m = [
                "bool" => [
                    "must" => [
                        [
                            "range" => [
                                "object.mains.location.latitude" => [
                                    "gt" => $loc['latitude'] - 5,
                                    "lt" => $loc['latitude'] + 5
                                ]
                            ]
                        ], [
                            "range" => [
                                "object.mains.location.longitude" => [
                                    "gt" => $loc['longitude'] - 5,
                                    "lt" => $loc['longitude'] + 5
                                ]
                            ]
                        ],
                    ],
                ]
            ];
            array_push($must, $m);
        }

        $s = [
            'match' => [
                'object.' . 'status' => 'pending',
            ]
        ];
        array_push($must, $s);
        foreach ($request->header() as $key => $value) {
            if (($key !='city_id') && ($key !='location') && ($key !='title')) {
                continue;
            }
            if (($key == 'city_id')) {
                $m = [
                    "match" => [
                        "object.mains." . "city_id" => $value[0],
                    ]
                ];
                array_push($must, $m);

            } if (($key === 'location')) {
                $loc = json_decode($value, true);
                $m = [
                    "bool" => [
                        "must" => [
                            [
                                "range" => [
                                    "object.mains.location.latitude" => [
                                        "gt" => $loc['latitude'] - 5,
                                        "lt" => $loc['latitude'] + 5
                                    ]
                                ]
                            ], [
                                "range" => [
                                    "object.mains.location.longitude" => [
                                        "gt" => $loc['longitude'] - 5,
                                        "lt" => $loc['longitude'] + 5
                                    ]
                                ]
                            ],
                        ],
                    ]
                ];
                array_push($must, $m);

            }
            if (($key === 'title')) {
                $m = [
                    "match" => [
                        "object.mains." . "title" => $value[0],
                    ]
                ];
                array_push($must, $m);
            }

//            $m = [
//                "range" => [
//                    "object.renewal_at" => [
//                        "gte" => Carbon::now()
//                    ],
//                ]
//            ];
        }
        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "filter" => [
                            [
                                "range" => [
                                    "object.renewal_at" => [
                                        "gte" => Carbon::now()->subDay(30)
                                    ]
                                ]
                            ]
                        ],
                        "must" => $must,
                        "must_not" => $must_not,
                    ]
                ]
            ],
        ];
        $response = $this->client->search($params);
//        $c = count($response['hits']['hits']);
//        for($i=0;$i<$c;$i++)
//        {
//            $renewDate = $response['hits']['hits'][$i]['_source']['object']['renewal_at'];
//            if(strtotime(date('Y-m-d')) > strtotime(date('Y-m-d', strtotime($renewDate))))
//            {
//                $response['hits']['hits'][$i] = 0;
//            }
//        }
        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];


    }

    public function showRejected(Request $request)
    {
        $m = [];
        $must = [];
        $must_not = [];

        if(isset($request->ads_type) && $request->ads_type == "shop")
        {
            $type = [
                'range' => [
                    'object.shop' => [
                        'gt' => 0
                    ]
                ]
            ];
            array_push($must, $type);
        }
        if(isset($request->ads_type) && $request->ads_type == "personal")
        {
            $type = [
                'range' => [
                    'object.shop' => [
                        'gt' => 0
                    ]
                ]
            ];
            array_push($must_not, $type);
        }
        if(isset($request->title))
        {
            $m = [
                "match" => [
                    "object.mains." . "title" => $request->title,
                ]
            ];
            array_push($must, $m);
        }
        if(isset($request->city_id))
        {
            $m = [
                "match" => [
                    "object.mains." . "city_id" => $request->city_id,
                ]
            ];
            array_push($must, $m);
        }

        if(isset($request->location))
        {
            $loc = json_decode($request->location, true);
            $m = [
                "bool" => [
                    "must" => [
                        [
                            "range" => [
                                "object.mains.location.latitude" => [
                                    "gt" => $loc['latitude'] - 5,
                                    "lt" => $loc['latitude'] + 5
                                ]
                            ]
                        ], [
                            "range" => [
                                "object.mains.location.longitude" => [
                                    "gt" => $loc['longitude'] - 5,
                                    "lt" => $loc['longitude'] + 5
                                ]
                            ]
                        ],
                    ],
                ]
            ];
            array_push($must, $m);
        }

//        $s = [
//            'match' => [
//                'object.' . 'status' => 'rejected',
//            ]
//        ];
//        array_push($must, $s);
//        $s = [
//            'match' => [
//                'object.' . 'status' => 'blocked',
//            ]
//        ];
//        array_push($must, $s);
//

        $s = [
            "bool" => [          //here its a nested boolean query
                "should" => [  //should is for **OR**
                    [
                        "match" => [
                            'object.' . 'status' => 'blocked',
                        ]
                    ],
                    [
                        "match" => [
                            'object.' . 'status' => 'rejected',
                        ]
                    ]
                ]
            ]
        ];

        array_push($must, $s);
        foreach ($request->header() as $key => $value) {
            if (($key !='city_id') && ($key !='location') && ($key !='title')) {
                continue;
            }
            if (($key == 'city_id')) {
                $m = [
                    "match" => [
                        "object.mains." . "city_id" => $value[0],
                    ]
                ];
                array_push($must, $m);

            } if (($key === 'location')) {
                $loc = json_decode($value, true);
                $m = [
                    "bool" => [
                        "must" => [
                            [
                                "range" => [
                                    "object.mains.location.latitude" => [
                                        "gt" => $loc['latitude'] - 5,
                                        "lt" => $loc['latitude'] + 5
                                    ]
                                ]
                            ], [
                                "range" => [
                                    "object.mains.location.longitude" => [
                                        "gt" => $loc['longitude'] - 5,
                                        "lt" => $loc['longitude'] + 5
                                    ]
                                ]
                            ],
                        ],
                    ]
                ];
                array_push($must, $m);

            }
            if (($key === 'title')) {
                $m = [
                    "match" => [
                        "object.mains." . "title" => $value[0],
                    ]
                ];
                array_push($must, $m);
            }

        }
        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "filter" => [
                            [
                                "range" => [
                                    "object.renewal_at" => [
                                        "gte" => Carbon::now()->subDay(30)
                                    ]
                                ]
                            ]
                        ],
                        "must" => $must,
                        "must_not" => $must_not,
                    ]
                ]
            ],
        ];
        $response = $this->client->search($params);
        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];


    }

    public function showProductbyBusiness(Request $request)
    {
        if ($request->input('b_id')) {
            $b = Business::where('user_id', $request->input('b_id'))->first();
        } else {
            $b = Business::where('user_id', $request->id)->first();
        }
        if ($b) {
            $size = $request->size ?? 10;
            $page = $request->page ?? 1;
            $params = [
                'index' => 'products',
                'body' => [
                    "size" => $size,
                    "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                    'query' => [
                        'bool' => [
                            'must' => [
                                [
                                    'match' => [
                                        'id' => $request->id
                                    ]
                                ]
                                ,
                                [
                                    'range' => [
                                        'object.shop' => [
                                            'gt' => 0
                                        ]
                                    ]
                                ]
                            ],
                        ]
                    ]
                ]];
            $response = $this->client->search($params);
            return [
                'total' => $response['hits']['total'],
                'meta' => [
                    'perpage' => $size,
                    'currentpage' => $page,
                    'totalpage' => ceil($response['hits']['total']['value'] / $size),
                    'totalrows' => $response['hits']['total']['value'],
                ],
                'max_score' => $response['hits']['max_score'],
                'hits' => $response['hits']['hits'],
            ];

        } else {
            abort(403, trans('messages.user_business_fail'));
        }

    }


    public function showBusinessProductCount(Request $request)
    {
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'match' => [
                        'id' => $request->id,
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        return ($response['hits']['total']['value']);
    }


    public function showSingleProduct(Request $request)
    {

        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'bool' => [
                        "must" => [
                            [
                                "term" => ['_id' => $request->product_id]
                            ]
                        ],
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        if ($response['hits']['hits']) {
            $res = $response['hits']['hits'][0];
            $token = Token::query()->where('token', $request->header('token'))->first();
            if ($token) {
                LastVisit::query()->create([
                    'product_id' => $request->product_id,
                    'user_id' => $token->user_id
                ]);
                $note = Note::where('product_id', $request->product_id)->where('user_id', $token->user_id)->first();
                $user = User::find($token->user_id);
                $request->merge(['id' => $token->user_id]);
                $favorite = Favorite::where('product_id', $request->product_id)->where('user_id', $token->user_id)->first();
                $views = LastVisit::where('product_id', $request->product_id)->get();


                $monthly_views = LastVisit::whereBetween('created_at',
                    [Carbon::now()->startOfMonth(), Carbon::now()->endOfMonth()])->where('product_id', $request->product_id)->get();

                $weekly_views = LastVisit::whereBetween('created_at',
                    [Carbon::now()->startOfWeek(), Carbon::now()->endOfWeek()])->where('product_id', $request->product_id)->get();

                $daily_views = LastVisit::whereBetween('created_at',
                    [Carbon::now()->startOfDay(), Carbon::now()->endOfDay()])->where('product_id', $request->product_id)->get();

                if ($res['_source']['id'] == $request->id) {
                    $res['_source']['object']['owned'] = true;
                } else {
                    $res['_source']['object']['owned'] = false;
                }

                if ($res['_source']['object']['owned'] == true) {
                    $res['_source']['object']['views'] = [
                        'total_views' => count($views),
                        'monthly_views' => count($monthly_views),
                        'weekly_views' => count($weekly_views),
                        'daily_views' => count($daily_views)
                    ];
                }else{
                    $chat =  Chat::where('user_id',$token->user_id)->where('product_id',$request->product_id)->first();
                    if($chat){
                        $res['_source']['chat_id'] = $chat->id;
                        $res['_source']['chat_status'] = $chat->status;
                    }else{
                        $res['_source']['chat_id'] = null;
                    }
                    if ($favorite){
                        $res['_source']['bookmarked'] = true;
                    }
                    if ($note) {
                        $res['_source']['object']['note'] = $note->text;
                    }
                }

                //the date of creation
                $startDate = Carbon::parse($res['_source']['object']['created_at']);
                $endDate = Carbon::now();
                $diffInDays = $startDate->diffInDays($endDate);
                $res['_source']['object']['creation_date'] = $diffInDays;

                //days left
                $startDateLeft = Carbon::parse($res['_source']['object']['renewal_at']);
                $diffInDays = $startDateLeft->diffInDays($endDate);
                $res['_source']['object']['days_left'] = 29 - $diffInDays;
                if ($res['_source']['object']['days_left'] < 0) {
                    $res['_source']['object']['status'] = 'expired';
                }
            }
            if(isset($res['_source']['object']['shop']) && $res['_source']['object']['shop'] != null && $res['_source']['object']['shop'] != 'null' )
            {
                $shop = Business::query()->where('id', $res['_source']['object']['shop'])->first();
                $shopInfo = [
                    'address' => $shop->address,
                    'logo' => $shop->logo,
                    'title' => $shop->title
                ];
                $res['_source']['shop_details'] = $shopInfo;
                return $res;
            }
            return $res;
        } else {
            abort(403, trans('messages.product_notfound'));
        }


    }

    public function store(Request $request)
    {
        $temp = $request->all();
        $temp['lang'] = ($request->hasHeader('X-localization')) ? $request->header('X-localization') : env('DEFAULT_LANG');

        $params = [
            'index' => 'products',
            'body' => $temp,
        ];
        array_walk_recursive($params['body']['object']['others'], function(&$string){
            if($string == false)
            {
                $string = "0";
            }
            else
            {
                $string = (string) $string;
            }
        });
        Log::info($params['body']['object']['others']);
        $time = Carbon::now();
        $params["body"]["object"]["created_at"] = $time;
        $params["body"]["object"]["updated_at"] = $time;
        $params["body"]["object"]["renewal_at"] = $time;
        $params["body"]["object"]["status"] = 'pending';
        $params["body"]["object"]["fori"] = false;
        $string = $request['object']['mains']['title'] . '-' . $request['object']['mains']['innerCategory'];
        str_replace(['/','|','\\'], ['','',''], $string);
        $params["body"]["object"]["slug"] =
            str_replace(' ', '-', $string);
        $response = $this->client->index($params);


    }


    public function update(Request $request)
    {
        $time = Carbon::now();
        $find = [
            'index' => 'products',
            'body' => [
                'query' => [
                    "match" => [

                        "_id" => $request->input('pid')

                    ],
                ]
            ]
        ];

        $response = $this->client->search($find);

        $temp = $request->all();
        $temp['lang'] = ($request->hasHeader('X-localization')) ? $request->header('X-localization') : env('DEFAULT_LANG');
        if(isset($temp['object']['mains']['title']))
        {
            $temp['object']['mains']['title'] = str_replace(['/','|','\\'], ['','',''],  $temp['object']['mains']['title']);

            $temp['object']['slug'] = $temp['object']['mains']['title'] . '-' .$response['hits']['hits'][0]['_source']['object']['mains']['innerCategory'];
            $temp['object']['slug'] = str_replace(' ', '-', $temp['object']['slug']);
        }

        $params = [
            'index' => 'products',
            'id' => $request->input('pid'),
            'body' => [
                'doc' => $temp,
            ]
        ];
        $params['body']['doc']['id'] = $response['hits']['hits'][0]['_source']['id'];

        $params['body']['doc']['object']['updated_at'] = $time;
        $params["body"]['doc']["object"]["status"] = 'pending';

        $response = $this->client->update($params);


    }







    public function search(Request $request)
    {
        $must = [];


        $s = [
            'match' => [
                'object.' . 'status' => 'accepted',
            ]
        ];
        array_push($must, $s);


        $m = [];
        $mustn = [];
        $should = [];


        //get sayyar shop
        $bus = Business::where('pay_at_home', 1)->get();
        $b_id = [];
        foreach ($bus as $b) {
            array_push($b_id, $b->user_id);
        }


        foreach ($request->all() as $key => $value) {
            if ($request->category || $request->subCategory || $request->innerCategory) {
                if ($request->category) {
                    $item_type = $this->getItemType($key, $request['category']); //range
                }
                if ($request->subCategory) {
                    $item_type = $this->getItemType($key, $request['subCategory']); //range
                }
                if ($request->innerCategory) {
                    $item_type = $this->getItemType($key, $request['innerCategory']); //range
                }


                if ($key === "cat_name") {
                    $m = [
                        "match" => [
                            "cat_name.keyword" => $value,
                        ]
                    ];


                } elseif (($item_type == 'switch') || ($item_type == 'text')) {
                    $m = [
                        'match' => [
                            'object.others.' . $key => $value,
                        ]
                    ];


                } elseif (($item_type === 'range')) {


                    $q = [];
                    if (isset($value['priceFrom'])) {
                        $q["gt"] = $value['priceFrom'];
                    }
                    if (isset($value['priceTo'])) {
                        $q["lt"] = $value['priceTo'];
                    }


                    $m = [
                        "range" => [
                            'object.mains.' . $key =>
                                $q,
                        ]
                    ];




                } elseif (($item_type === 'select') || ($item_type === 'limited_select')) {
                    $m = [
                        "terms" => [
                            'object.others.' . $key => $value,
                        ]
                    ];
                }
            }


            if ($key === 'page') {


                continue;


            } elseif ($key === 'size') {


                continue;


            } elseif (($key == 'q')) {
                $m = [
                    'prefix' => [
                        'object.' . 'mains.' . 'title' => $value,
                    ]
                ];


            } elseif (($key === 'category')) {
                $m = [
                    "match" => [
                        'object.mains.category.keyword' => $value,
                    ]
                ];
            } elseif (($key === 'subCategory')) {
                $m = [
                    "match" => [
                        'object.mains.subCategory.keyword' => $value,
                    ]
                ];
            } elseif (($key === 'innerCategory')) {
                $m = [
                    "match" => [
                        'object.mains.innerCategory.keyword' => $value,
                    ]
                ];
            } elseif (($key === 'withImage') && ($value == true)) {
                $m = [
                    "exists" => [
                        "field" => 'object.mains.images'
                    ]
                ];


            } elseif (($key === 'mobileShop') && ($value == true)) {
                $m = [
                    "terms" => [
                        'id' => $b_id,
                    ]
                ];
            } elseif (($key === 'city_id')) {
                $m = [
                    "match" => [
                        "object.mains." . "city_id" => $value,
                    ]
                ];


            } elseif (($key === 'location')) {
                $loc = json_decode($value, true);
                $m = [
                    "bool" => [
                        "must" => [
                            [
                                "range" => [
                                    "object.mains.location.latitude" => [
                                        "gt" => $loc['latitude'] - 5,
                                        "lt" => $loc['latitude'] + 5
                                    ]
                                ]
                            ], [
                                "range" => [
                                    "object.mains.location.longitude" => [
                                        "gt" => $loc['longitude'] - 5,
                                        "lt" => $loc['longitude'] + 5
                                    ]
                                ]
                            ],
                        ],
                    ]
                ];


            } elseif (($key === 'adType')) {
                $m = [
                    "bool" => [
                        "should" => []
                    ]
                ];
                if (is_array($value)) {
                    foreach ($value as $ad) {
                        if ($ad == 1) {
                            $term = ['object.mains.fori' => true];
                        } elseif ($ad == 2) {
                            $term = ['object.mains.agreement' => true];
                        } elseif ($ad == 3) {
                        } elseif ($ad == 3) {
                            $term = ['object.mains.exchange' => true];
                        }
                        array_push($m["bool"]["should"], ['term' => $term]);
                    }
                }
            } elseif (($key === 'advertiserType')) {
                if (is_array($value)) {


                    foreach ($value as $ad) {
                        if ($ad == "shop") {
                            $m = [
                                "exists" => [
                                    "field" => 'object.shop'
                                ]
                            ];
                        } elseif ($ad == "personal") {
                            $mustn = [
                                "exists" => [
                                    "field" => 'object.shop'
                                ]
                            ];
                        }
                    }
                } else {
                    if ($value == "shop") {
                        $m = [
                            "exists" => [
                                "field" => 'object.shop'
                            ]
                        ];
                    } elseif ($value == "personal") {
                        $mustn = [
                            "exists" => [
                                "field" => 'object.shop'
                            ]
                        ];
                    }
                }
            }


            if (!empty($m)) {
                array_push($must, $m);
            }
        }


        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "filter" => [
                            [
                                "range" => [
                                    "object.renewal_at" => [
                                        "gte" => Carbon::now()->subDay(30)
                                    ]
                                ]
                            ]
                        ],
                        "must" => $must,
                        "must_not" => $mustn,
                    ]
                ],
                "sort" => [
                    ['object.updated_at' => 'desc' ]
                ]
            ],
        ];


        $response = $this->client->search($params);
        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];


    }




    public function getItemType($item, $cat_name)
    {
        $category = $this->showItemType($cat_name);
        //amlak form
        if (count($category['hits']) > 0) {
            $infos = $category['hits'][0]['_source']['items']['form']['options'];
            foreach ($infos as $key => $value) {
                if ($item === $value['name']) {
                    return $value['type'];
                    //"range"
                    break;
                }
            }
        } else {
            return trans('messages.cat_notfound');
        }
    }


    public function showItemType($category)
    {
        $params = [
            'index' => 'forms',
            'body' => [
                'query' => [
                    'match' => [
                        'cat_name' => $category
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);

        return $response['hits']; // documents

    }



    public function changeProductStatus(Request $request)
    {
        $data = $request->validate([
            'status' => 'required|in:accepted,rejected,blocked,pending',
            'product_id' => 'required|string|max:100'
        ]);
        $params = [
            'index' => 'products',
            'id' => $request->product_id,
            'body' => [
                'doc' => [
                    'object' => [
                        'status' => $request->status
                    ]
                ],
            ]
        ];

        $response = $this->client->update($params);
    }



    public function addToFavorite(Request $request)
    {

        $userid = $request->id;
        $productid = $request->product_id;
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    "match" => [
                        '_id' => $request->product_id,
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        if ($response['hits']['hits'][0]['_source']['id'] == $userid) {
            abort(400, 'access denied');
        } else {
            $favo = Favorite::where('user_id', $userid)->where('product_id', $productid)->first();
            if (!$favo) {
                $fav = Favorite::query()
                    ->create([
                        'user_id' => $userid,
                        'product_id' => $productid
                    ]);
            } else {
                $favo->delete();
            }
        }


    }


    public function addNote(Request $request)
    {

        $userid = $request->id;
        $productid = $request->product_id;
        $note = Note::where('user_id', $userid)->where('product_id', $productid)->first();
        if (!$note) {
            if ($request->text !== null) {
                Note::query()
                    ->create([
                        'user_id' => $userid,
                        'product_id' => $productid,
                        'text' => $request->text
                    ]);
            }
        } else {
            if ($request->text === null) {
                $note->delete();
            } else {
                Note::where('user_id', $userid)
                    ->where('product_id', $productid)
                    ->update(['text' => $request->text]);
            }
        }

    }


    public function showProductFeedbacks(Request $request)
    {
        if($request->product_id){
            $feedbacks = Feedback::where('product_id',$request->product_id)->get()->sortByDesc("created_at");
        }else{
            $feedbacks = Feedback::all()->sortByDesc("created_at");
        }
        return $feedbacks;

    }


    public function storeFeedback(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'email' => 'required|email',
            'product_id' => 'required',
            'message' => 'required',
            'lower_price' => 'nullable|boolean',
            'Insufficient_data' => 'nullable|boolean',
            'low_image_quality' => 'nullable|boolean',
            'rate' => 'nullable',
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $feedback = new Feedback();
            $feedback->fill([
                'email' => $request->email,
                'product_id' => $request->product_id,
                'message' => $request->message,
                'lower_price' => $request->lower_price,
                'Insufficient_data' => $request->Insufficient_data,
                'low_image_quality' => $request->low_image_quality,
                'rate' => $request->rate,
            ]);
            $feedback->user_id = $request->id;
            $feedback->save();
        }

    }


    public function destroy(Request $request)
    {

        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    "match" => [

                        "_id" => $request->product_id

                    ],
                ]
            ]
        ];

        $response = $this->client->search($params);
        $user_id = $response['hits']['hits'][0]['_source']['id'];
        $token = Token::query()
            ->where('token' , $request->header()['token'])
            ->first();
        if(($request->id == $user_id) || ($token->role == "admin")){
            $response = $this->client->delete([
                'index' => 'products',
                'id' => $request->product_id
            ]);
        }else{
            abort(403, trans('messages.auth_fail'));
        }

    }

}
