<?php
namespace App\Http\Controllers\Auth;
use App\Actions\UserAction;
use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\Hash;
class UserController extends Controller
{
    public function __construct()
    {
        $this->middleware('check_role:USERS_LIST')->only(['get','getById']);
        $this->middleware('check_role:USERS_DELETE')->only(['deleteById']);
    }
    /**
     * @throws ValidationException
     */
    public function login(Request $request)
    {
        $request->validate([
            'phone' => 'required|phone',
            'password' => 'required'
        ]);
        $user = User::where('phone', $request->phone)->first();
        if (! $user || ! Hash::check($request->password, $user->password)) {
            throw ValidationException::withMessages([
                'phone' => ['The provided credentials are incorrect.'],
            ]);
        }
        return $user->createToken('token_base_name')->plainTextToken;
    }


    public function users(Request $request)
    {
        $size = $request->input('size');
        $page = $request->input('page');
        $user = User::paginate($size);
        return $user;

    }

    public function get(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Users List : ',
            'data' => (new UserAction())
                ->setRequest($request)
                ->setValidationRule('get_query')
                ->makeEloquentViaRequest()
                ->getByRequestAndEloquent()
        ]);
    }

    public function getById(string $id) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'User : ',
            'data' => (new UserAction())
                ->getById($id)
        ]);
    }

    public function deleteById (string $id): JsonResponse
    {
        (new UserAction())->deleteById($id);

        return response()->json([
            'success' => true,
            'message' => 'deleted'
        ]);
    }

    public function creditCardUpdate(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Updated : ',
            'data' => (new UserAction())
                ->setRequest($request)
                ->setValidationRule('cc_update')
                ->makeEloquentViaRequest()
                ->ccUpdate($request->header('token'))
        ]);
    }

    public function getCreditCard(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Credit Card : ',
            'data' => (new UserAction())
                ->setRequest($request)
                ->makeEloquentViaRequest()
                ->getCC($request->header('token'))
        ]);
    }

    public function changeStatus(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'User : ',
            'data' => (new UserAction())
                ->setRequest($request)
                ->setValidationRule('status_update')
                ->makeEloquentViaRequest()
                ->changeUserStatus()
        ]);
    }



}
