<?php

namespace App\Http\Controllers;

use App\Models\Admin;
use App\Models\Business;
use App\Models\DiscountCode;
use App\Models\PersonalAccessToken;
use App\Models\Token;
use App\Models\User;
use App\Models\VerificationCode;
use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use function PHPUnit\Framework\isNull;
use Kavenegar;

class AuthController extends Controller
{

//    public function test()
//    {
//        return Kavenegar::VerifyLookup('09357382069', '13233', null, null, 'verify', $type = null);
//    }

    public function doRegister(Request $request)
    {
        $this->validate($request, [
            'phone' => ['required','regex:/^(?:[0-9]\x20?){1,3}[0-9]{1,14}$/'],
            'country_code' => 'max:4',
        ]);
        $country_code = 'IR';
        if(isset($request->country_code))
        {
            $country_code = $request->country_code;
        }
        $user = User::firstOrCreate(['phone' => request('phone')]);
        $veriy_code = VerificationCode::where('user_id', $user->id)->first();
        if ($veriy_code) {
            if ($veriy_code->isValid()) {
                if(! empty(env('SMS_API_KEY')))
                {
                    Kavenegar::VerifyLookup($request->phone, $veriy_code->code, null, null, 'verify', $type = null);
                }
                else
                {
                    return $veriy_code->code;
                }
            } else {
                $veriy_code->code = $this->newCode();
                $veriy_code->save();
                if(! empty(env('SMS_API_KEY')))
                {
                    Kavenegar::VerifyLookup($request->phone, $veriy_code->code, null, null, 'verify', $type = null);
                }
                else
                {
                    return $veriy_code->code;
                }
            }
        } else {
            $veriy_code = VerificationCode::create([
                'code' => $this->newCode(),
                'user_id' => $user->id,
            ]);
            $veriy_code->sendCode();
            if(! empty(env('SMS_API_KEY')))
            {
                Kavenegar::VerifyLookup($request->phone, $veriy_code->code, null, null, 'verify', $type = null);
            }
            else
            {
                return $veriy_code->code;
            }
        }
        return trans('messages.sms');

    }


//    Generate new code
    public function newCode()
    {
        $max = pow(10, 4);
        $min = $max / 10 - 1;
        $code = mt_rand($min, $max);
        return $code;
    }


//    verify the user phone and code
    public function doVerify(Request $request)
    {
        $this->validate($request, [
            'code' => 'required|numeric',
            'phone' => ['required','regex:/^(?:[0-9]\x20?){1,3}[0-9]{1,14}$/'],
            'country_code' => 'max:4'
        ]);
        $country_code = 'IR';
        if(isset($request->country_code))
        {
            $country_code = $request->country_code;
        }
        $user = User::query()->where('phone', request('phone'))->first();

        if ($user) {
            if($user->status == false)
            {
                abort(403,'user is blocked');
            }
            $veriy_code = VerificationCode::where('user_id', $user->id)->first();
            if ($veriy_code->isValid()) {
                if ($veriy_code->code == request('code')) {
//                 create token for verified user
                    return $this->generate_token($user->id);
                } else {
                    abort(403, trans('messages.code_fail'));
                }
            } else {
                abort(403, trans('messages.code_fail'));
            }


        } else {
            abort(403, trans('messages.user_notfound'));
        }


//end of doVerify
    }


    protected function generate_token($id)
    {
        $user = User::query()->find($id);
        $token = Token::query()
            ->create([
                'token' => Str::random(60),
                'user_id' => $user->id,
                'role' =>'user'
            ]);
        return $token->token;
    }


    protected function generate_admin_token($id)
    {
        $admin = Admin::query()->find($id);
        $token = Token::query()
            ->create([
                'token' => Str::random(60),
                'user_id' => $admin->id,
                'role' => 'admin'
            ]);
        return $token->token;
    }


    public function addAdmin(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'fullname' => 'string',
            'email' => 'email',
            'password' =>'string',
            'role_id' => 'int'
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $admin = Admin::create([
                'fullname' => $request->fullname,
                'email' => $request->email,
                'role_id' => $request->role_id,
                'password' => Hash::make($request->password),
            ]);
            $admin->save();
        }
    }


    public function adminLogin(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'email' => 'email',
            'password' =>'string'
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $admin = Admin::where('email',$request->email)->with('role.permissions')->first();
            if($admin){
                if((Hash::check($request->password,$admin->password))){
                    return [
                        $this->generate_admin_token($admin->id),
                        $admin
                    ];
                }else{
                    abort(403, trans('messages.auth_fail'));
                }
            }else{
                abort(403, trans('messages.auth_fail'));
            }

        }
    }


    public function adminChangePassword(Request $request)
    {
        $admin = Admin::where('id' , $request->id)->first();
        $validate = Validator::make($request->all(), [
            'password' => 'string|required'
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $admin = Admin::where('id' , $request->id)->first();
            $admin->fill([
                'password' => Hash::make($request->password),
            ]);
            $admin->save();
        }

    }

    public function updateAdminRole(Request $request,$id)
    {
        $admin = Admin::query()->where('id', $id)->first();
        $validate = Validator::make($request->all(), [
            'role_id' => 'int|required'
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $admin->fill([
                'role_id' => $request->role_id,
            ]);
            $admin->save();
        }
    }

    public function allAdmins()
    {
        return Admin::with('role')->get();
    }

    public function deleteAdmin($id)
    {
        return Admin::query()->where('id', $id)->first()->delete();
    }

    public function getById($id)
    {
        return Admin::query()->where('id', $id)->with('role')->first();
    }

}
