<?php

namespace App\Http\Controllers\Dashboard;

use App\Actions\BusinessAction;
use App\Actions\UserAction;
use App\Models\BusinessSocial;
use App\Models\BusinessValue;
use App\Models\Video;
use Carbon\Carbon;
use App\Http\Controllers\Controller;
use App\Models\Business;
use App\Models\User;
use App\Models\Photo;
use App\Traits\ImageUploader;
use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class BusinessController extends Controller
{
    use ImageUploader;

    public function showAllBusinesses(Request $request)
    {
        $size = $request->size ?? 2;
        $page = $request->page ?? 1;
        $business = Business::query()->where('status',$request->status)->skip(($page - 1) * $size)->limit($size)
            ->with(['socials'])
            ->with(['values'])
            ->get();
        return [
            'meta' => [
                'perpage' =>$size,
                'currentpage' =>$page,
                'totalpage' =>ceil(count(Business::all())/$size),
                'totalrows' =>count($business),
            ],
            'hits' => $business,
        ];
    }

    public function show(Request $request)
    {
        $user = User::find($request->id);
        if($user){
            $business = Business::query()->where('user_id', $user->id)
                ->with(['socials'])
                ->with(['values'])
                ->first();
            if ($business) {
                //the date of creation
                $startDate = Carbon::parse($business['created_at']);
                $endDate = Carbon::now();
                $diffInDays = $startDate->diffInDays($endDate);
                $business['creation_date'] = $diffInDays;
                $business = $business->toArray();
                $business['socials'] = $business['socials']['socials'] ?? null;
                $business['values'] = $business['values']['values'] ?? null;
                return $business;
            } else {
                abort(404, trans('messages.business_notfound'));
            }
        }else{
            abort(403, trans('messages.auth_fail'));
        }

    }

        public function showByID(Request $request)
    {
        $business = Business::query()->where('id',$request->id)->with(['socials'])                ->with(['values'])

            ->first();
        if($business->status == 0)
        {
            throw new CustomException(trans('messages.auth_fail'),403,403);
        }
        if ($business) {
            //the date of creation
        $startDate = Carbon::parse($business['created_at']);
        $endDate = Carbon::now();
        $diffInDays = $startDate->diffInDays($endDate);
        $business['creation_date'] = $diffInDays;
            $business = $business->toArray();
            $business['socials'] = $business['socials']['socials'] ?? null;
            $business['values'] = $business['values']['values'] ?? null;
            return $business;
        } else {
            abort(404, trans('messages.business_notfound'));
        }

    }


    public function store(Request $request)
        {
        $validate = Validator::make($request->all(), [
            'user_id' => 'nullable',
            'status' => 'boolean',
            'name' => 'string|required',
            'last_name' => 'string|required',
            'city_id' => 'int|required',
            'chat' => 'boolean|required|required',
            'national_code' => 'size:10',
            'title' => 'string',
            'description' => 'string',
            'category' => 'string|required',
            'subCategory' => 'string|nullable',
            'innerCategory' => 'string|nullable',
            'address' => 'string',
            'location' => 'required',
            'exactLocation' => 'required',
            'phone' => 'required',
            'country_code' => 'nullable',
            'country_code_tell' => 'nullable',
            'telephone' => 'required',
            'postal_code' => 'size:10',
            'logo' => 'nullable',
            'business_license' => 'nullable',
            'personal_photo' => 'nullable',
            'store_photo' => 'nullable',
            'special_post' => 'nullable|boolean',
            'delivery' => 'nullable|boolean',
            'home_delivery' => 'nullable|boolean',
            'pay_at_home' => 'nullable|boolean',
            'shop_at_store' => 'nullable|boolean',
            'email_notification' => 'nullable|boolean',
            'phone_notification' => 'nullable|boolean',
            'socials' => ['array'], //todo INJA
            'values' => ['array'], //todo INJA
            'socials.*' => ['string', 'max:2500'],
            'values.*' => ['string', 'max:2500'],
        ]);
        $payload = $request->all();
        if(isset($payload['socials']))
        {
            unset($payload['socials']);
        }
        if(isset($payload['values']))
        {
            unset($payload['values']);
        }
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $user = User::find($request->id);
            $business = Business::where('user_id', $user->id)->first();
            $payload['location'] = json_encode($payload['location']);
            $payload['store_photo'] = json_encode($payload['store_photo']);
            if (!$business){
                $business = new Business();
                $business->fill($payload);
                $business->user_id = $user->id;
                $business->status = 0;
                $business->save();

                if(isset($request->socials))
                {
                    BusinessSocial::query()->create([
                        'business_id' => $business->id,
                        'socials' => $request->socials
                    ]);
                }
                if(isset($request->values))
                {
                    BusinessValue::query()->create([
                        'business_id' => $business->id,
                        'values' => $request->values
                    ]);
                }

            }else{
                abort(403, trans('messages.business_found'));
            }

        }

    }


    public function update(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'status' => 'boolean',
            'national_code' => 'size:10',
            'title' => 'string',
            'description' => 'string',
            'address' => 'string',
            'location' => 'required',
            'phone' => 'required',
            'country_code' => 'nullable',
            'country_code_tell' => 'nullable',
            'telephone' => 'required',
            'postal_code' => 'size:10',
            'logo' => 'nullable',
            'business_license' => 'nullable',
            'personal_photo' => 'nullable',
            'special_post' => 'nullable|boolean',
            'delivery' => 'nullable|boolean',
            'home_delivery' => 'nullable|boolean',
            'pay_at_home' => 'nullable|boolean',
            'shop_at_store' => 'nullable|boolean',
            'emeil_notification' => 'nullable|boolean',
            'phone_notification' => 'nullable|boolean',
            'socials' => ['array'], //todo INJA
            'values' => ['array'], //todo INJA
            'socials.*' => ['required','string', 'max:2500'],
            'values.*' => ['required','string', 'max:2500'],
        ]);
        $payload = $request->all();
        if(isset($payload['socials']))
        {
            unset($payload['socials']);
        }
        if(isset($payload['values']))
        {
            unset($payload['values']);
        }
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $user = User::find($request->id);
            $business = Business::where('user_id', $user->id)->first();
            if($business){
            $business->fill($payload);
            $business->user_id = $user->id;
            $business->save();
            if(isset($request->socials))
            {
                $businessSocials = BusinessSocial::query()->where('business_id',$business->id)->first();
                if(!empty($businessSocials))
                {
                    $businessSocials->socials = $request->socials;
                    $businessSocials->save();
                }
                else
                {
                    BusinessSocial::query()->create([
                        'business_id' => $business->id,
                        'socials' => $request->socials
                    ]);
                }
            }
            if(isset($request->values))
            {
                $businessValues = BusinessValue::query()->where('business_id',$business->id)->first();
                if(!empty($businessValues))
                {
                    $businessValues->values = $request->values;
                    $businessValues->save();
                }
                else
                {
                    BusinessValue::query()->create([
                        'business_id' => $business->id,
                        'values' => $request->values
                    ]);
                }
            }
            }else{
                abort(404, trans('messages.business_notfound'));
            }

        }
    }



    public function changeBusinessStatus(Request $request)
    {
        $business = Business::where('user_id',$request->id)->first();
        $business->status = $request->status;
        $business->save();

    }


    public function convertImageToUrl(Request $request)
    {
        $photo = new Photo();
        $photo->store_photo = $request->file('store_photo')->store('store_photo');
        $photo->save();
        $res = [env('PUBLIC_URL')."/storage/app/".$photo->store_photo,$photo->id];
        return [
            'url' => $res[0],
            'id' => $res[1],
        ];

    }

    public function convertVideoToUrl(Request $request)
    {
        $video = new Video();
        $video->store_video = $request->file('store_video')->store('store_video');
        $video->save();
        $res = [env('PUBLIC_URL')."/storage/app/".$video->store_video,$video->id];
        return [
            'url' => $res[0],
            'id' => $res[1],
        ];

    }


    public function get(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Businesses List : ',
            'data' => (new BusinessAction())
                ->setRequest($request)
                ->setValidationRule('get_query')
                ->makeEloquentViaRequest()
                ->getByRequestAndEloquent()
        ]);
    }

    public function changeStatus(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Chnage Status : ',
            'data' => (new BusinessAction())
                ->setRequest($request)
                ->setValidationRule('change_status')
                ->changeStatus()
        ]);
    }

    public function getById(string $id) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Business : ',
            'data' => (new BusinessAction())
                ->getById($id)
        ]);
    }

    public function deleteById (string $id): JsonResponse
    {
        (new BusinessAction())->deleteById($id);

        return response()->json([
            'success' => true,
            'message' => 'deleted'
        ]);
    }

}
