<?php

namespace App\Actions;

use App\Models\Business;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class BusinessAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(Business::class)
            ->setValidationRules([
                'get_query' => [
                    'first_name' => 'string|max:100',
                    'last_name' => 'string|max:100',
                    'store_name' => 'string|max:100',
                    'city_id' => 'int|max:99999',
                    'mobile' => 'string|max:100',
                    'telephone' => 'string|max:100',
                    'status' => 'boolean',
                    'is_special' => 'boolean',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ],
                'change_status' => [
                    'store_id' => ['required', 'int', 'max:999999']
                ],
                'set_special' => [
                    'store_id' => ['required', 'int', 'max:999999'],
                    'is_special' => ['required', 'boolean'],
                ]
            ])
            ->setQueryToEloquentClosures([
                'first_name' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('name', 'LIKE', "%{$query['first_name']}%");
                    });
                },
                'last_name' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('lastname', 'LIKE', "%{$query['last_name']}%");
                    });
                },
                'city_id' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('city_id', 'LIKE', $query['city_id']);
                    });
                },
                'mobile' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('phone', 'LIKE', "%{$query['mobile']}%");
                    });
                },
                'status' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('status', $query['status']);
                    });
                },
                'is_special' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('is_special', $query['is_special']);
                    });
                },
                'store_name' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('title', 'LIKE', "%{$query['store_name']}%");
                    });
                },
                'telephone' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('telephone', 'LIKE', "%{$query['telephone']}%");
                    });
                },
                'from_created_at' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['from_created_at']);
                },
                'is_special' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('is_special', $query['is_special']);
                    });
                },
            ]);
        parent::__construct();
    }

    public function changeStatus()
    {
        $data = $this->getDataFromRequest();
        $business = Business::query()->where('id', $data['store_id'])->first();
        $status = $business->status;
        $status = !$status;

        return $business->update([
            'status' => $status
        ]);
    }

    public function setSpecial()
    {
        $data = $this->getDataFromRequest();
        $business = Business::query()->where('id', $data['store_id'])->first();

        $business->is_special = $data['is_special'];
        $business->save();
        return $business;
    }
}
