<?php

namespace App\Actions;

use App\Http\Resources\ChatResource;
use App\Http\Resources\TransactionResource;
use App\Models\Chat;
use App\Models\Message;
use App\Models\Transaction;
use App\Models\User;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class ChatAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(Message::class)
            ->setResource(ChatResource::class)
            ->setValidationRules([
                'user_get_query' => [
                    'chat_id' => 'int|nullable',
                    'user_id' => 'int',
                    'sender' => 'string',
                    'text' => 'string',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ],
                'get_query' => [
                    'chat_id' => '',
                    'user_id' => 'int',
                    'sender' => 'string',
                    'text' => 'string',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ]
            ])
            ->setQueryToEloquentClosures([
                'chat_id' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('chat_id', $query['chat_id']);
                    });
                },
                'user_id' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('users', function ($q) use ($query){
                        $q->where('users.id', $query['user_id']);
                    });
                },
                'first_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('user', function ($q) use ($query){
                        $q->where('users.name', $query['first_name']);
                    });
                },
                'last_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('user', function ($q) use ($query){
                        $q->where('users.lastname', $query['last_name']);
                    });
                },
                'sender' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('sender', $query['sender']);
                    });
                },
                'text' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('text', 'LIKE', "%{$query['text']}%");
                    });
                },
                'from_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['from_created_at']);
                },
            ]);
        parent::__construct();
    }

    public function getByUserRequest()
    {
        if(empty($this->query['chat_id']))
        {
            return [
                'count' => 0,
                'data' => []
            ];
        }
        return parent::getByRequestAndEloquent();
    }
}
