<?php

namespace App\Actions;

use App\Models\City;
use Genocide\Radiocrud\Helpers;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class CityAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(City::class)
            ->setValidationRules([
                'store' => [
                    'name' => ['required', 'string', 'max:150'],
                    'province_id' => ['required', 'int', 'max:15000'],
                    'big' => ['boolean', 'max:150'],
                ],
                'get_query' => [
                    'search' => ['string', 'max:10'],
                    'province_id' => ['string', 'max:10'],
                    'country_id' => ['string', 'max:10'],
                ],

            ])
            ->setQueryToEloquentClosures([
                'country_id' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('province', function ($q) use ($query){
                        $q->where('country_id' ,$query['country_id']);
                    });
                },
                'province_id' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where('province_id', $query['province_id']);
                },
                'search' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where('name', $query['search']);
                }
            ]);
        parent::__construct();
    }
//        public function insert()
//    {
//        $data = '[
//  {
//    "big": true,
//    "id": 1,
//    "province_id": 1,
//    "name": "تبریز"
//  },
//  {
//    "id": 2,
//    "province_id": 1,
//    "name": "كندوان"
//  },
//  {
//    "id": 3,
//    "province_id": 1,
//    "name": "بندر شرفخانه"
//  },
//  {
//    "id": 4,
//    "province_id": 1,
//    "name": "مراغه"
//  },
//  {
//    "id": 5,
//    "province_id": 1,
//    "name": "ميانه"
//  },
//  {
//    "id": 6,
//    "province_id": 1,
//    "name": "شبستر"
//  },
//  {
//    "id": 7,
//    "province_id": 1,
//    "name": "مرند"
//  },
//  {
//    "id": 8,
//    "province_id": 1,
//    "name": "جلفا"
//  },
//  {
//    "id": 9,
//    "province_id": 1,
//    "name": "سراب"
//  },
//  {
//    "id": 10,
//    "province_id": 1,
//    "name": "هاديشهر"
//  },
//  {
//    "id": 11,
//    "province_id": 1,
//    "name": "بناب"
//  },
//  {
//    "id": 12,
//    "province_id": 1,
//    "name": "كليبر"
//  },
//  {
//    "id": 13,
//    "province_id": 1,
//    "name": "تسوج"
//  },
//  {
//    "id": 14,
//    "province_id": 1,
//    "name": "اهر"
//  },
//  {
//    "id": 15,
//    "province_id": 1,
//    "name": "هريس"
//  },
//  {
//    "id": 16,
//    "province_id": 1,
//    "name": "عجبشير"
//  },
//  {
//    "id": 17,
//    "province_id": 1,
//    "name": "هشترود"
//  },
//  {
//    "id": 18,
//    "province_id": 1,
//    "name": "ملكان"
//  },
//  {
//    "id": 19,
//    "province_id": 1,
//    "name": "بستان آباد"
//  },
//  {
//    "id": 20,
//    "province_id": 1,
//    "name": "ورزقان"
//  },
//  {
//    "id": 21,
//    "province_id": 1,
//    "name": "اسكو"
//  },
//  {
//    "id": 22,
//    "province_id": 1,
//    "name": "آذر شهر"
//  },
//  {
//    "id": 23,
//    "province_id": 1,
//    "name": "قره آغاج"
//  },
//  {
//    "id": 24,
//    "province_id": 1,
//    "name": "ممقان"
//  },
//  {
//    "id": 25,
//    "province_id": 1,
//    "name": "صوفیان"
//  },
//  {
//    "id": 26,
//    "province_id": 1,
//    "name": "ایلخچی"
//  },
//  {
//    "id": 27,
//    "province_id": 1,
//    "name": "خسروشهر"
//  },
//  {
//    "id": 28,
//    "province_id": 1,
//    "name": "باسمنج"
//  },
//  {
//    "id": 29,
//    "province_id": 1,
//    "name": "سهند"
//  },
//  {
//    "id": 30,
//    "province_id": 2,
//    "name": "اروميه"
//  },
//  {
//    "id": 31,
//    "province_id": 2,
//    "name": "نقده"
//  },
//  {
//    "id": 32,
//    "province_id": 2,
//    "name": "ماكو"
//  },
//  {
//    "id": 33,
//    "province_id": 2,
//    "name": "تكاب"
//  },
//  {
//    "id": 34,
//    "province_id": 2,
//    "name": "خوي"
//  },
//  {
//    "id": 35,
//    "province_id": 2,
//    "name": "مهاباد"
//  },
//  {
//    "id": 36,
//    "province_id": 2,
//    "name": "سر دشت"
//  },
//  {
//    "id": 37,
//    "province_id": 2,
//    "name": "چالدران"
//  },
//  {
//    "id": 38,
//    "province_id": 2,
//    "name": "بوكان"
//  },
//  {
//    "id": 39,
//    "province_id": 2,
//    "name": "مياندوآب"
//  },
//  {
//    "id": 40,
//    "province_id": 2,
//    "name": "سلماس"
//  },
//  {
//    "id": 41,
//    "province_id": 2,
//    "name": "شاهين دژ"
//  },
//  {
//    "id": 42,
//    "province_id": 2,
//    "name": "پيرانشهر"
//  },
//  {
//    "id": 43,
//    "province_id": 2,
//    "name": "سيه چشمه"
//  },
//  {
//    "id": 44,
//    "province_id": 2,
//    "name": "اشنويه"
//  },
//  {
//    "id": 45,
//    "province_id": 2,
//    "name": "چایپاره"
//  },
//  {
//    "id": 46,
//    "province_id": 2,
//    "name": "پلدشت"
//  },
//  {
//    "id": 47,
//    "province_id": 2,
//    "name": "شوط"
//  },
//  {
//    "id": 48,
//    "province_id": 3,
//    "name": "اردبيل"
//  },
//  {
//    "id": 49,
//    "province_id": 3,
//    "name": "سرعين"
//  },
//  {
//    "id": 50,
//    "province_id": 3,
//    "name": "بيله سوار"
//  },
//  {
//    "id": 51,
//    "province_id": 3,
//    "name": "پارس آباد"
//  },
//  {
//    "id": 52,
//    "province_id": 3,
//    "name": "خلخال"
//  },
//  {
//    "id": 53,
//    "province_id": 3,
//    "name": "مشگين شهر"
//  },
//  {
//    "id": 54,
//    "province_id": 3,
//    "name": "مغان"
//  },
//  {
//    "id": 55,
//    "province_id": 3,
//    "name": "نمين"
//  },
//  {
//    "id": 56,
//    "province_id": 3,
//    "name": "نير"
//  },
//  {
//    "id": 57,
//    "province_id": 3,
//    "name": "كوثر"
//  },
//  {
//    "id": 58,
//    "province_id": 3,
//    "name": "كيوي"
//  },
//  {
//    "id": 59,
//    "province_id": 3,
//    "name": "گرمي"
//  },
//  {
//    "big": true,
//    "id": 60,
//    "province_id": 4,
//    "name": "اصفهان"
//  },
//  {
//    "id": 61,
//    "province_id": 4,
//    "name": "فريدن"
//  },
//  {
//    "id": 62,
//    "province_id": 4,
//    "name": "فريدون شهر"
//  },
//  {
//    "id": 63,
//    "province_id": 4,
//    "name": "فلاورجان"
//  },
//  {
//    "id": 64,
//    "province_id": 4,
//    "name": "گلپايگان"
//  },
//  {
//    "id": 65,
//    "province_id": 4,
//    "name": "دهاقان"
//  },
//  {
//    "id": 66,
//    "province_id": 4,
//    "name": "نطنز"
//  },
//  {
//    "id": 67,
//    "province_id": 4,
//    "name": "نايين"
//  },
//  {
//    "id": 68,
//    "province_id": 4,
//    "name": "تيران"
//  },
//  {
//    "id": 69,
//    "province_id": 4,
//    "name": "كاشان"
//  },
//  {
//    "id": 70,
//    "province_id": 4,
//    "name": "فولاد شهر"
//  },
//  {
//    "id": 71,
//    "province_id": 4,
//    "name": "اردستان"
//  },
//  {
//    "id": 72,
//    "province_id": 4,
//    "name": "سميرم"
//  },
//  {
//    "id": 73,
//    "province_id": 4,
//    "name": "درچه"
//  },
//  {
//    "id": 74,
//    "province_id": 4,
//    "name": "کوهپایه"
//  },
//  {
//    "id": 75,
//    "province_id": 4,
//    "name": "مباركه"
//  },
//  {
//    "id": 76,
//    "province_id": 4,
//    "name": "شهرضا"
//  },
//  {
//    "id": 77,
//    "province_id": 4,
//    "name": "خميني شهر"
//  },
//  {
//    "id": 78,
//    "province_id": 4,
//    "name": "شاهين شهر"
//  },
//  {
//    "id": 79,
//    "province_id": 4,
//    "name": "نجف آباد"
//  },
//  {
//    "id": 80,
//    "province_id": 4,
//    "name": "دولت آباد"
//  },
//  {
//    "id": 81,
//    "province_id": 4,
//    "name": "زرين شهر"
//  },
//  {
//    "id": 82,
//    "province_id": 4,
//    "name": "آران و بيدگل"
//  },
//  {
//    "id": 83,
//    "province_id": 4,
//    "name": "باغ بهادران"
//  },
//  {
//    "id": 84,
//    "province_id": 4,
//    "name": "خوانسار"
//  },
//  {
//    "id": 85,
//    "province_id": 4,
//    "name": "مهردشت"
//  },
//  {
//    "id": 86,
//    "province_id": 4,
//    "name": "علويجه"
//  },
//  {
//    "id": 87,
//    "province_id": 4,
//    "name": "عسگران"
//  },
//  {
//    "id": 88,
//    "province_id": 4,
//    "name": "نهضت آباد"
//  },
//  {
//    "id": 89,
//    "province_id": 4,
//    "name": "حاجي آباد"
//  },
//  {
//    "id": 90,
//    "province_id": 4,
//    "name": "تودشک"
//  },
//  {
//    "id": 91,
//    "province_id": 4,
//    "name": "ورزنه"
//  },
//  {
//    "id": 92,
//    "province_id": 6,
//    "name": "ايلام"
//  },
//  {
//    "id": 93,
//    "province_id": 6,
//    "name": "مهران"
//  },
//  {
//    "id": 94,
//    "province_id": 6,
//    "name": "دهلران"
//  },
//  {
//    "id": 95,
//    "province_id": 6,
//    "name": "آبدانان"
//  },
//  {
//    "id": 96,
//    "province_id": 6,
//    "name": "شيروان چرداول"
//  },
//  {
//    "id": 97,
//    "province_id": 6,
//    "name": "دره شهر"
//  },
//  {
//    "id": 98,
//    "province_id": 6,
//    "name": "ايوان"
//  },
//  {
//    "id": 99,
//    "province_id": 6,
//    "name": "سرابله"
//  },
//  {
//    "id": 100,
//    "province_id": 7,
//    "name": "بوشهر"
//  },
//  {
//    "id": 101,
//    "province_id": 7,
//    "name": "تنگستان"
//  },
//  {
//    "id": 102,
//    "province_id": 7,
//    "name": "دشتستان"
//  },
//  {
//    "id": 103,
//    "province_id": 7,
//    "name": "دير"
//  },
//  {
//    "id": 104,
//    "province_id": 7,
//    "name": "ديلم"
//  },
//  {
//    "id": 105,
//    "province_id": 7,
//    "name": "كنگان"
//  },
//  {
//    "id": 106,
//    "province_id": 7,
//    "name": "گناوه"
//  },
//  {
//    "id": 107,
//    "province_id": 7,
//    "name": "ريشهر"
//  },
//  {
//    "id": 108,
//    "province_id": 7,
//    "name": "دشتي"
//  },
//  {
//    "id": 109,
//    "province_id": 7,
//    "name": "خورموج"
//  },
//  {
//    "id": 110,
//    "province_id": 7,
//    "name": "اهرم"
//  },
//  {
//    "id": 111,
//    "province_id": 7,
//    "name": "برازجان"
//  },
//  {
//    "id": 112,
//    "province_id": 7,
//    "name": "خارك"
//  },
//  {
//    "id": 113,
//    "province_id": 7,
//    "name": "جم"
//  },
//  {
//    "id": 114,
//    "province_id": 7,
//    "name": "کاکی"
//  },
//  {
//    "id": 115,
//    "province_id": 7,
//    "name": "عسلویه"
//  },
//  {
//    "id": 116,
//    "province_id": 7,
//    "name": "بردخون"
//  },
//  {
//    "big": true,
//    "id": 117,
//    "province_id": 8,
//    "name": "تهران"
//  },
//  {
//    "id": 118,
//    "province_id": 8,
//    "name": "ورامين"
//  },
//  {
//    "id": 119,
//    "province_id": 8,
//    "name": "فيروزكوه"
//  },
//  {
//    "id": 120,
//    "province_id": 8,
//    "name": "ري"
//  },
//  {
//    "id": 121,
//    "province_id": 8,
//    "name": "دماوند"
//  },
//  {
//    "id": 122,
//    "province_id": 8,
//    "name": "اسلامشهر"
//  },
//  {
//    "id": 123,
//    "province_id": 8,
//    "name": "رودهن"
//  },
//  {
//    "id": 124,
//    "province_id": 8,
//    "name": "لواسان"
//  },
//  {
//    "id": 125,
//    "province_id": 8,
//    "name": "بومهن"
//  },
//  {
//    "id": 126,
//    "province_id": 8,
//    "name": "تجريش"
//  },
//  {
//    "id": 127,
//    "province_id": 8,
//    "name": "فشم"
//  },
//  {
//    "id": 128,
//    "province_id": 8,
//    "name": "كهريزك"
//  },
//  {
//    "id": 129,
//    "province_id": 8,
//    "name": "پاكدشت"
//  },
//  {
//    "id": 130,
//    "province_id": 8,
//    "name": "چهاردانگه"
//  },
//  {
//    "id": 131,
//    "province_id": 8,
//    "name": "شريف آباد"
//  },
//  {
//    "id": 132,
//    "province_id": 8,
//    "name": "قرچك"
//  },
//  {
//    "id": 133,
//    "province_id": 8,
//    "name": "باقرشهر"
//  },
//  {
//    "id": 134,
//    "province_id": 8,
//    "name": "شهريار"
//  },
//  {
//    "id": 135,
//    "province_id": 8,
//    "name": "رباط كريم"
//  },
//  {
//    "id": 136,
//    "province_id": 8,
//    "name": "قدس"
//  },
//  {
//    "id": 137,
//    "province_id": 8,
//    "name": "ملارد"
//  },
//  {
//    "id": 138,
//    "province_id": 9,
//    "name": "شهركرد"
//  },
//  {
//    "id": 139,
//    "province_id": 9,
//    "name": "فارسان"
//  },
//  {
//    "id": 140,
//    "province_id": 9,
//    "name": "بروجن"
//  },
//  {
//    "id": 141,
//    "province_id": 9,
//    "name": "چلگرد"
//  },
//  {
//    "id": 142,
//    "province_id": 9,
//    "name": "اردل"
//  },
//  {
//    "id": 143,
//    "province_id": 9,
//    "name": "لردگان"
//  },
//  {
//    "id": 144,
//    "province_id": 9,
//    "name": "سامان"
//  },
//  {
//    "id": 145,
//    "province_id": 10,
//    "name": "قائن"
//  },
//  {
//    "id": 146,
//    "province_id": 10,
//    "name": "فردوس"
//  },
//  {
//    "id": 147,
//    "province_id": 10,
//    "name": "بيرجند"
//  },
//  {
//    "id": 148,
//    "province_id": 10,
//    "name": "نهبندان"
//  },
//  {
//    "id": 149,
//    "province_id": 10,
//    "name": "سربيشه"
//  },
//  {
//    "id": 150,
//    "province_id": 10,
//    "name": "طبس مسینا"
//  },
//  {
//    "id": 151,
//    "province_id": 10,
//    "name": "قهستان"
//  },
//  {
//    "id": 152,
//    "province_id": 10,
//    "name": "درمیان"
//  },
//  {
//    "big": true,
//    "id": 153,
//    "province_id": 11,
//    "name": "مشهد"
//  },
//  {
//    "id": 154,
//    "province_id": 11,
//    "name": "نيشابور"
//  },
//  {
//    "id": 155,
//    "province_id": 11,
//    "name": "سبزوار"
//  },
//  {
//    "id": 156,
//    "province_id": 11,
//    "name": "كاشمر"
//  },
//  {
//    "id": 157,
//    "province_id": 11,
//    "name": "گناباد"
//  },
//  {
//    "id": 158,
//    "province_id": 11,
//    "name": "طبس"
//  },
//  {
//    "id": 159,
//    "province_id": 11,
//    "name": "تربت حيدريه"
//  },
//  {
//    "id": 160,
//    "province_id": 11,
//    "name": "خواف"
//  },
//  {
//    "id": 161,
//    "province_id": 11,
//    "name": "تربت جام"
//  },
//  {
//    "id": 162,
//    "province_id": 11,
//    "name": "تايباد"
//  },
//  {
//    "id": 163,
//    "province_id": 11,
//    "name": "قوچان"
//  },
//  {
//    "id": 164,
//    "province_id": 11,
//    "name": "سرخس"
//  },
//  {
//    "id": 165,
//    "province_id": 11,
//    "name": "بردسكن"
//  },
//  {
//    "id": 166,
//    "province_id": 11,
//    "name": "فريمان"
//  },
//  {
//    "id": 167,
//    "province_id": 11,
//    "name": "چناران"
//  },
//  {
//    "id": 168,
//    "province_id": 11,
//    "name": "درگز"
//  },
//  {
//    "id": 169,
//    "province_id": 11,
//    "name": "كلات"
//  },
//  {
//    "id": 170,
//    "province_id": 11,
//    "name": "طرقبه"
//  },
//  {
//    "id": 171,
//    "province_id": 11,
//    "name": "سر ولایت"
//  },
//  {
//    "id": 172,
//    "province_id": 12,
//    "name": "بجنورد"
//  },
//  {
//    "id": 173,
//    "province_id": 12,
//    "name": "اسفراين"
//  },
//  {
//    "id": 174,
//    "province_id": 12,
//    "name": "جاجرم"
//  },
//  {
//    "id": 175,
//    "province_id": 12,
//    "name": "شيروان"
//  },
//  {
//    "id": 176,
//    "province_id": 12,
//    "name": "آشخانه"
//  },
//  {
//    "id": 177,
//    "province_id": 12,
//    "name": "گرمه"
//  },
//  {
//    "id": 178,
//    "province_id": 12,
//    "name": "ساروج"
//  },
//  {
//    "big": true,
//    "id": 179,
//    "province_id": 13,
//    "name": "اهواز"
//  },
//  {
//    "id": 180,
//    "province_id": 13,
//    "name": "ايرانشهر"
//  },
//  {
//    "id": 181,
//    "province_id": 13,
//    "name": "شوش"
//  },
//  {
//    "id": 182,
//    "province_id": 13,
//    "name": "آبادان"
//  },
//  {
//    "id": 183,
//    "province_id": 13,
//    "name": "خرمشهر"
//  },
//  {
//    "id": 184,
//    "province_id": 13,
//    "name": "مسجد سليمان"
//  },
//  {
//    "id": 185,
//    "province_id": 13,
//    "name": "ايذه"
//  },
//  {
//    "id": 186,
//    "province_id": 13,
//    "name": "شوشتر"
//  },
//  {
//    "id": 187,
//    "province_id": 13,
//    "name": "انديمشك"
//  },
//  {
//    "id": 188,
//    "province_id": 13,
//    "name": "سوسنگرد"
//  },
//  {
//    "id": 189,
//    "province_id": 13,
//    "name": "هويزه"
//  },
//  {
//    "id": 190,
//    "province_id": 13,
//    "name": "دزفول"
//  },
//  {
//    "id": 191,
//    "province_id": 13,
//    "name": "شادگان"
//  },
//  {
//    "id": 192,
//    "province_id": 13,
//    "name": "بندر ماهشهر"
//  },
//  {
//    "id": 193,
//    "province_id": 13,
//    "name": "بندر امام خميني"
//  },
//  {
//    "id": 194,
//    "province_id": 13,
//    "name": "اميديه"
//  },
//  {
//    "id": 195,
//    "province_id": 13,
//    "name": "بهبهان"
//  },
//  {
//    "id": 196,
//    "province_id": 13,
//    "name": "رامهرمز"
//  },
//  {
//    "id": 197,
//    "province_id": 13,
//    "name": "باغ ملك"
//  },
//  {
//    "id": 198,
//    "province_id": 13,
//    "name": "هنديجان"
//  },
//  {
//    "id": 199,
//    "province_id": 13,
//    "name": "لالي"
//  },
//  {
//    "id": 200,
//    "province_id": 13,
//    "name": "رامشیر"
//  },
//  {
//    "id": 201,
//    "province_id": 13,
//    "name": "حمیدیه"
//  },
//  {
//    "id": 202,
//    "province_id": 13,
//    "name": "دغاغله"
//  },
//  {
//    "id": 203,
//    "province_id": 13,
//    "name": "ملاثانی"
//  },
//  {
//    "id": 204,
//    "province_id": 13,
//    "name": "شادگان"
//  },
//  {
//    "id": 205,
//    "province_id": 13,
//    "name": "ویسی"
//  },
//  {
//    "id": 206,
//    "province_id": 14,
//    "name": "زنجان"
//  },
//  {
//    "id": 207,
//    "province_id": 14,
//    "name": "ابهر"
//  },
//  {
//    "id": 208,
//    "province_id": 14,
//    "name": "خدابنده"
//  },
//  {
//    "id": 209,
//    "province_id": 14,
//    "name": "كارم"
//  },
//  {
//    "id": 210,
//    "province_id": 14,
//    "name": "ماهنشان"
//  },
//  {
//    "id": 211,
//    "province_id": 14,
//    "name": "خرمدره"
//  },
//  {
//    "id": 212,
//    "province_id": 14,
//    "name": "ايجرود"
//  },
//  {
//    "id": 213,
//    "province_id": 14,
//    "name": "زرين آباد"
//  },
//  {
//    "id": 214,
//    "province_id": 14,
//    "name": "آب بر"
//  },
//  {
//    "id": 215,
//    "province_id": 14,
//    "name": "قيدار"
//  },
//  {
//    "id": 216,
//    "province_id": 15,
//    "name": "سمنان"
//  },
//  {
//    "id": 217,
//    "province_id": 15,
//    "name": "شاهرود"
//  },
//  {
//    "id": 218,
//    "province_id": 15,
//    "name": "گرمسار"
//  },
//  {
//    "id": 219,
//    "province_id": 15,
//    "name": "ايوانكي"
//  },
//  {
//    "id": 220,
//    "province_id": 15,
//    "name": "دامغان"
//  },
//  {
//    "id": 221,
//    "province_id": 15,
//    "name": "بسطام"
//  },
//  {
//    "id": 222,
//    "province_id": 16,
//    "name": "زاهدان"
//  },
//  {
//    "id": 223,
//    "province_id": 16,
//    "name": "چابهار"
//  },
//  {
//    "id": 224,
//    "province_id": 16,
//    "name": "خاش"
//  },
//  {
//    "id": 225,
//    "province_id": 16,
//    "name": "سراوان"
//  },
//  {
//    "id": 226,
//    "province_id": 16,
//    "name": "زابل"
//  },
//  {
//    "id": 227,
//    "province_id": 16,
//    "name": "سرباز"
//  },
//  {
//    "id": 228,
//    "province_id": 16,
//    "name": "نيكشهر"
//  },
//  {
//    "id": 229,
//    "province_id": 16,
//    "name": "ايرانشهر"
//  },
//  {
//    "id": 230,
//    "province_id": 16,
//    "name": "راسك"
//  },
//  {
//    "id": 231,
//    "province_id": 16,
//    "name": "ميرجاوه"
//  },
//  {
//    "big": true,
//    "id": 232,
//    "province_id": 17,
//    "name": "شيراز"
//  },
//  {
//    "id": 233,
//    "province_id": 17,
//    "name": "اقليد"
//  },
//  {
//    "id": 234,
//    "province_id": 17,
//    "name": "داراب"
//  },
//  {
//    "id": 235,
//    "province_id": 17,
//    "name": "فسا"
//  },
//  {
//    "id": 236,
//    "province_id": 17,
//    "name": "مرودشت"
//  },
//  {
//    "id": 237,
//    "province_id": 17,
//    "name": "خرم بيد"
//  },
//  {
//    "id": 238,
//    "province_id": 17,
//    "name": "آباده"
//  },
//  {
//    "id": 239,
//    "province_id": 17,
//    "name": "كازرون"
//  },
//  {
//    "id": 240,
//    "province_id": 17,
//    "name": "ممسني"
//  },
//  {
//    "id": 241,
//    "province_id": 17,
//    "name": "سپيدان"
//  },
//  {
//    "id": 242,
//    "province_id": 17,
//    "name": "لار"
//  },
//  {
//    "id": 243,
//    "province_id": 17,
//    "name": "فيروز آباد"
//  },
//  {
//    "id": 244,
//    "province_id": 17,
//    "name": "جهرم"
//  },
//  {
//    "id": 245,
//    "province_id": 17,
//    "name": "ني ريز"
//  },
//  {
//    "id": 246,
//    "province_id": 17,
//    "name": "استهبان"
//  },
//  {
//    "id": 247,
//    "province_id": 17,
//    "name": "لامرد"
//  },
//  {
//    "id": 248,
//    "province_id": 17,
//    "name": "مهر"
//  },
//  {
//    "id": 249,
//    "province_id": 17,
//    "name": "حاجي آباد"
//  },
//  {
//    "id": 250,
//    "province_id": 17,
//    "name": "نورآباد"
//  },
//  {
//    "id": 251,
//    "province_id": 17,
//    "name": "اردكان"
//  },
//  {
//    "id": 252,
//    "province_id": 17,
//    "name": "صفاشهر"
//  },
//  {
//    "id": 253,
//    "province_id": 17,
//    "name": "ارسنجان"
//  },
//  {
//    "id": 254,
//    "province_id": 17,
//    "name": "قيروكارزين"
//  },
//  {
//    "id": 255,
//    "province_id": 17,
//    "name": "سوريان"
//  },
//  {
//    "id": 256,
//    "province_id": 17,
//    "name": "فراشبند"
//  },
//  {
//    "id": 257,
//    "province_id": 17,
//    "name": "سروستان"
//  },
//  {
//    "id": 258,
//    "province_id": 17,
//    "name": "ارژن"
//  },
//  {
//    "id": 259,
//    "province_id": 17,
//    "name": "گويم"
//  },
//  {
//    "id": 260,
//    "province_id": 17,
//    "name": "داريون"
//  },
//  {
//    "id": 261,
//    "province_id": 17,
//    "name": "زرقان"
//  },
//  {
//    "id": 262,
//    "province_id": 17,
//    "name": "خان زنیان"
//  },
//  {
//    "id": 263,
//    "province_id": 17,
//    "name": "کوار"
//  },
//  {
//    "id": 264,
//    "province_id": 17,
//    "name": "ده بید"
//  },
//  {
//    "id": 265,
//    "province_id": 17,
//    "name": "باب انار/خفر"
//  },
//  {
//    "id": 266,
//    "province_id": 17,
//    "name": "بوانات"
//  },
//  {
//    "id": 267,
//    "province_id": 17,
//    "name": "خرامه"
//  },
//  {
//    "id": 268,
//    "province_id": 17,
//    "name": "خنج"
//  },
//  {
//    "id": 269,
//    "province_id": 17,
//    "name": "سیاخ دارنگون"
//  },
//  {
//    "id": 270,
//    "province_id": 18,
//    "name": "قزوين"
//  },
//  {
//    "id": 271,
//    "province_id": 18,
//    "name": "تاكستان"
//  },
//  {
//    "id": 272,
//    "province_id": 18,
//    "name": "آبيك"
//  },
//  {
//    "id": 273,
//    "province_id": 18,
//    "name": "بوئين زهرا"
//  },
//  {
//    "id": 274,
//    "province_id": 19,
//    "name": "قم"
//  },
//  {
//    "id": 275,
//    "province_id": 5,
//    "name": "طالقان"
//  },
//  {
//    "id": 276,
//    "province_id": 5,
//    "name": "نظرآباد"
//  },
//  {
//    "id": 277,
//    "province_id": 5,
//    "name": "اشتهارد"
//  },
//  {
//    "id": 278,
//    "province_id": 5,
//    "name": "هشتگرد"
//  },
//  {
//    "id": 279,
//    "province_id": 5,
//    "name": "كن"
//  },
//  {
//    "id": 280,
//    "province_id": 5,
//    "name": "آسارا"
//  },
//  {
//    "id": 281,
//    "province_id": 5,
//    "name": "شهرک گلستان"
//  },
//  {
//    "id": 282,
//    "province_id": 5,
//    "name": "اندیشه"
//  },
//  {
//    "big": true,
//    "id": 283,
//    "province_id": 5,
//    "name": "كرج"
//  },
//  {
//    "id": 284,
//    "province_id": 5,
//    "name": "نظر آباد"
//  },
//  {
//    "id": 285,
//    "province_id": 5,
//    "name": "گوهردشت"
//  },
//  {
//    "id": 286,
//    "province_id": 5,
//    "name": "ماهدشت"
//  },
//  {
//    "id": 287,
//    "province_id": 5,
//    "name": "مشکین دشت"
//  },
//  {
//    "id": 288,
//    "province_id": 20,
//    "name": "سنندج"
//  },
//  {
//    "id": 289,
//    "province_id": 20,
//    "name": "ديواندره"
//  },
//  {
//    "id": 290,
//    "province_id": 20,
//    "name": "بانه"
//  },
//  {
//    "id": 291,
//    "province_id": 20,
//    "name": "بيجار"
//  },
//  {
//    "id": 292,
//    "province_id": 20,
//    "name": "سقز"
//  },
//  {
//    "id": 293,
//    "province_id": 20,
//    "name": "كامياران"
//  },
//  {
//    "id": 294,
//    "province_id": 20,
//    "name": "قروه"
//  },
//  {
//    "id": 295,
//    "province_id": 20,
//    "name": "مريوان"
//  },
//  {
//    "id": 296,
//    "province_id": 20,
//    "name": "صلوات آباد"
//  },
//  {
//    "id": 297,
//    "province_id": 20,
//    "name": "حسن آباد"
//  },
//  {
//    "big": true,
//    "id": 298,
//    "province_id": 21,
//    "name": "كرمان"
//  },
//  {
//    "id": 299,
//    "province_id": 21,
//    "name": "راور"
//  },
//  {
//    "id": 300,
//    "province_id": 21,
//    "name": "بابك"
//  },
//  {
//    "id": 301,
//    "province_id": 21,
//    "name": "انار"
//  },
//  {
//    "id": 302,
//    "province_id": 21,
//    "name": "کوهبنان"
//  },
//  {
//    "id": 303,
//    "province_id": 21,
//    "name": "رفسنجان"
//  },
//  {
//    "id": 304,
//    "province_id": 21,
//    "name": "بافت"
//  },
//  {
//    "id": 305,
//    "province_id": 21,
//    "name": "سيرجان"
//  },
//  {
//    "id": 306,
//    "province_id": 21,
//    "name": "كهنوج"
//  },
//  {
//    "id": 307,
//    "province_id": 21,
//    "name": "زرند"
//  },
//  {
//    "id": 308,
//    "province_id": 21,
//    "name": "بم"
//  },
//  {
//    "id": 309,
//    "province_id": 21,
//    "name": "جيرفت"
//  },
//  {
//    "id": 310,
//    "province_id": 21,
//    "name": "بردسير"
//  },
//  {
//    "id": 311,
//    "province_id": 22,
//    "name": "كرمانشاه"
//  },
//  {
//    "id": 312,
//    "province_id": 22,
//    "name": "اسلام آباد غرب"
//  },
//  {
//    "id": 313,
//    "province_id": 22,
//    "name": "سر پل ذهاب"
//  },
//  {
//    "id": 314,
//    "province_id": 22,
//    "name": "كنگاور"
//  },
//  {
//    "id": 315,
//    "province_id": 22,
//    "name": "سنقر"
//  },
//  {
//    "id": 316,
//    "province_id": 22,
//    "name": "قصر شيرين"
//  },
//  {
//    "id": 317,
//    "province_id": 22,
//    "name": "گيلان غرب"
//  },
//  {
//    "id": 318,
//    "province_id": 22,
//    "name": "هرسين"
//  },
//  {
//    "id": 319,
//    "province_id": 22,
//    "name": "صحنه"
//  },
//  {
//    "id": 320,
//    "province_id": 22,
//    "name": "پاوه"
//  },
//  {
//    "id": 321,
//    "province_id": 22,
//    "name": "جوانرود"
//  },
//  {
//    "id": 322,
//    "province_id": 22,
//    "name": "شاهو"
//  },
//  {
//    "id": 323,
//    "province_id": 23,
//    "name": "ياسوج"
//  },
//  {
//    "id": 324,
//    "province_id": 23,
//    "name": "گچساران"
//  },
//  {
//    "id": 325,
//    "province_id": 23,
//    "name": "دنا"
//  },
//  {
//    "id": 326,
//    "province_id": 23,
//    "name": "دوگنبدان"
//  },
//  {
//    "id": 327,
//    "province_id": 23,
//    "name": "سي سخت"
//  },
//  {
//    "id": 328,
//    "province_id": 23,
//    "name": "دهدشت"
//  },
//  {
//    "id": 329,
//    "province_id": 23,
//    "name": "ليكك"
//  },
//  {
//    "id": 330,
//    "province_id": 24,
//    "name": "گرگان"
//  },
//  {
//    "id": 331,
//    "province_id": 24,
//    "name": "آق قلا"
//  },
//  {
//    "id": 332,
//    "province_id": 24,
//    "name": "گنبد كاووس"
//  },
//  {
//    "id": 333,
//    "province_id": 24,
//    "name": "علي آباد كتول"
//  },
//  {
//    "id": 334,
//    "province_id": 24,
//    "name": "مينو دشت"
//  },
//  {
//    "id": 335,
//    "province_id": 24,
//    "name": "تركمن"
//  },
//  {
//    "id": 336,
//    "province_id": 24,
//    "name": "كردكوي"
//  },
//  {
//    "id": 337,
//    "province_id": 24,
//    "name": "بندر گز"
//  },
//  {
//    "id": 338,
//    "province_id": 24,
//    "name": "كلاله"
//  },
//  {
//    "id": 339,
//    "province_id": 24,
//    "name": "آزاد شهر"
//  },
//  {
//    "id": 340,
//    "province_id": 24,
//    "name": "راميان"
//  },
//  {
//    "id": 341,
//    "province_id": 25,
//    "name": "رشت"
//  },
//  {
//    "id": 342,
//    "province_id": 25,
//    "name": "منجيل"
//  },
//  {
//    "id": 343,
//    "province_id": 25,
//    "name": "لنگرود"
//  },
//  {
//    "id": 344,
//    "province_id": 25,
//    "name": "رود سر"
//  },
//  {
//    "id": 345,
//    "province_id": 25,
//    "name": "تالش"
//  },
//  {
//    "id": 346,
//    "province_id": 25,
//    "name": "آستارا"
//  },
//  {
//    "id": 347,
//    "province_id": 25,
//    "name": "ماسوله"
//  },
//  {
//    "id": 348,
//    "province_id": 25,
//    "name": "آستانه اشرفيه"
//  },
//  {
//    "id": 349,
//    "province_id": 25,
//    "name": "رودبار"
//  },
//  {
//    "id": 350,
//    "province_id": 25,
//    "name": "فومن"
//  },
//  {
//    "id": 351,
//    "province_id": 25,
//    "name": "صومعه سرا"
//  },
//  {
//    "id": 352,
//    "province_id": 25,
//    "name": "بندرانزلي"
//  },
//  {
//    "id": 353,
//    "province_id": 25,
//    "name": "كلاچاي"
//  },
//  {
//    "id": 354,
//    "province_id": 25,
//    "name": "هشتپر"
//  },
//  {
//    "id": 355,
//    "province_id": 25,
//    "name": "رضوان شهر"
//  },
//  {
//    "id": 356,
//    "province_id": 25,
//    "name": "ماسال"
//  },
//  {
//    "id": 357,
//    "province_id": 25,
//    "name": "شفت"
//  },
//  {
//    "id": 358,
//    "province_id": 25,
//    "name": "سياهكل"
//  },
//  {
//    "id": 359,
//    "province_id": 25,
//    "name": "املش"
//  },
//  {
//    "id": 360,
//    "province_id": 25,
//    "name": "لاهیجان"
//  },
//  {
//    "id": 361,
//    "province_id": 25,
//    "name": "خشک بيجار"
//  },
//  {
//    "id": 362,
//    "province_id": 25,
//    "name": "خمام"
//  },
//  {
//    "id": 363,
//    "province_id": 25,
//    "name": "لشت نشا"
//  },
//  {
//    "id": 364,
//    "province_id": 25,
//    "name": "بندر کياشهر"
//  },
//  {
//    "id": 365,
//    "province_id": 26,
//    "name": "خرم آباد"
//  },
//  {
//    "id": 366,
//    "province_id": 26,
//    "name": "ماهشهر"
//  },
//  {
//    "id": 367,
//    "province_id": 26,
//    "name": "دزفول"
//  },
//  {
//    "id": 368,
//    "province_id": 26,
//    "name": "بروجرد"
//  },
//  {
//    "id": 369,
//    "province_id": 26,
//    "name": "دورود"
//  },
//  {
//    "id": 370,
//    "province_id": 26,
//    "name": "اليگودرز"
//  },
//  {
//    "id": 371,
//    "province_id": 26,
//    "name": "ازنا"
//  },
//  {
//    "id": 372,
//    "province_id": 26,
//    "name": "نور آباد"
//  },
//  {
//    "id": 373,
//    "province_id": 26,
//    "name": "كوهدشت"
//  },
//  {
//    "id": 374,
//    "province_id": 26,
//    "name": "الشتر"
//  },
//  {
//    "id": 375,
//    "province_id": 26,
//    "name": "پلدختر"
//  },
//  {
//    "id": 376,
//    "province_id": 27,
//    "name": "ساري"
//  },
//  {
//    "id": 377,
//    "province_id": 27,
//    "name": "آمل"
//  },
//  {
//    "id": 378,
//    "province_id": 27,
//    "name": "بابل"
//  },
//  {
//    "id": 379,
//    "province_id": 27,
//    "name": "بابلسر"
//  },
//  {
//    "id": 380,
//    "province_id": 27,
//    "name": "بهشهر"
//  },
//  {
//    "id": 381,
//    "province_id": 27,
//    "name": "تنكابن"
//  },
//  {
//    "id": 382,
//    "province_id": 27,
//    "name": "جويبار"
//  },
//  {
//    "id": 383,
//    "province_id": 27,
//    "name": "چالوس"
//  },
//  {
//    "id": 384,
//    "province_id": 27,
//    "name": "رامسر"
//  },
//  {
//    "id": 385,
//    "province_id": 27,
//    "name": "سواد كوه"
//  },
//  {
//    "id": 386,
//    "province_id": 27,
//    "name": "قائم شهر"
//  },
//  {
//    "id": 387,
//    "province_id": 27,
//    "name": "نكا"
//  },
//  {
//    "id": 388,
//    "province_id": 27,
//    "name": "نور"
//  },
//  {
//    "id": 389,
//    "province_id": 27,
//    "name": "بلده"
//  },
//  {
//    "id": 390,
//    "province_id": 27,
//    "name": "نوشهر"
//  },
//  {
//    "id": 391,
//    "province_id": 27,
//    "name": "پل سفيد"
//  },
//  {
//    "id": 392,
//    "province_id": 27,
//    "name": "محمود آباد"
//  },
//  {
//    "id": 393,
//    "province_id": 27,
//    "name": "فريدون كنار"
//  },
//  {
//    "id": 394,
//    "province_id": 28,
//    "name": "اراك"
//  },
//  {
//    "id": 395,
//    "province_id": 28,
//    "name": "آشتيان"
//  },
//  {
//    "id": 396,
//    "province_id": 28,
//    "name": "تفرش"
//  },
//  {
//    "id": 397,
//    "province_id": 28,
//    "name": "خمين"
//  },
//  {
//    "id": 398,
//    "province_id": 28,
//    "name": "دليجان"
//  },
//  {
//    "id": 399,
//    "province_id": 28,
//    "name": "ساوه"
//  },
//  {
//    "id": 400,
//    "province_id": 28,
//    "name": "سربند"
//  },
//  {
//    "id": 401,
//    "province_id": 28,
//    "name": "محلات"
//  },
//  {
//    "id": 402,
//    "province_id": 28,
//    "name": "شازند"
//  },
//  {
//    "id": 403,
//    "province_id": 29,
//    "name": "بندرعباس"
//  },
//  {
//    "id": 404,
//    "province_id": 29,
//    "name": "قشم"
//  },
//  {
//    "id": 405,
//    "province_id": 29,
//    "name": "كيش"
//  },
//  {
//    "id": 406,
//    "province_id": 29,
//    "name": "بندر لنگه"
//  },
//  {
//    "id": 407,
//    "province_id": 29,
//    "name": "بستك"
//  },
//  {
//    "id": 408,
//    "province_id": 29,
//    "name": "حاجي آباد"
//  },
//  {
//    "id": 409,
//    "province_id": 29,
//    "name": "دهبارز"
//  },
//  {
//    "id": 410,
//    "province_id": 29,
//    "name": "انگهران"
//  },
//  {
//    "id": 411,
//    "province_id": 29,
//    "name": "ميناب"
//  },
//  {
//    "id": 412,
//    "province_id": 29,
//    "name": "ابوموسي"
//  },
//  {
//    "id": 413,
//    "province_id": 29,
//    "name": "بندر جاسك"
//  },
//  {
//    "id": 414,
//    "province_id": 29,
//    "name": "تنب بزرگ"
//  },
//  {
//    "id": 415,
//    "province_id": 29,
//    "name": "بندر خمیر"
//  },
//  {
//    "id": 416,
//    "province_id": 29,
//    "name": "پارسیان"
//  },
//  {
//    "id": 417,
//    "province_id": 29,
//    "name": "قشم"
//  },
//  {
//    "big": true,
//    "id": 418,
//    "province_id": 30,
//    "name": "همدان"
//  },
//  {
//    "id": 419,
//    "province_id": 30,
//    "name": "ملاير"
//  },
//  {
//    "id": 420,
//    "province_id": 30,
//    "name": "تويسركان"
//  },
//  {
//    "id": 421,
//    "province_id": 30,
//    "name": "نهاوند"
//  },
//  {
//    "id": 422,
//    "province_id": 30,
//    "name": "كبودر اهنگ"
//  },
//  {
//    "id": 423,
//    "province_id": 30,
//    "name": "رزن"
//  },
//  {
//    "id": 424,
//    "province_id": 30,
//    "name": "اسدآباد"
//  },
//  {
//    "id": 425,
//    "province_id": 30,
//    "name": "بهار"
//  },
//  {
//    "big": true,
//    "id": 426,
//    "province_id": 31,
//    "name": "يزد"
//  },
//  {
//    "id": 427,
//    "province_id": 31,
//    "name": "تفت"
//  },
//  {
//    "id": 428,
//    "province_id": 31,
//    "name": "اردكان"
//  },
//  {
//    "id": 429,
//    "province_id": 31,
//    "name": "ابركوه"
//  },
//  {
//    "id": 430,
//    "province_id": 31,
//    "name": "ميبد"
//  },
//  {
//    "id": 431,
//    "province_id": 31,
//    "name": "طبس"
//  },
//  {
//    "id": 432,
//    "province_id": 31,
//    "name": "بافق"
//  },
//  {
//    "id": 433,
//    "province_id": 31,
//    "name": "مهريز"
//  },
//  {
//    "id": 434,
//    "province_id": 31,
//    "name": "اشكذر"
//  },
//  {
//    "id": 435,
//    "province_id": 31,
//    "name": "هرات"
//  },
//  {
//    "id": 436,
//    "province_id": 31,
//    "name": "خضرآباد"
//  },
//  {
//    "id": 437,
//    "province_id": 31,
//    "name": "شاهديه"
//  },
//  {
//    "id": 438,
//    "province_id": 31,
//    "name": "حمیدیه شهر"
//  },
//  {
//    "id": 439,
//    "province_id": 31,
//    "name": "سید میرزا"
//  },
//  {
//    "id": 440,
//    "province_id": 31,
//    "name": "زارچ"
//  }
//]' ;
//        $data = json_decode($data, true);
//        foreach ($data As $value)
//        {
//            City::query()->create([
//                'id' => $value['id'],
//                'province_id' => $value['province_id'],
//                'big' => $value['big'] ?? false,
//                'name' => Helpers::sanitize($value['name'])
//            ]);
//        }
////        Province::query()->insert($data);
//        return $this;
//    }
}
