<?php

namespace App\Actions;

use App\Models\DiscountCode;
use App\Models\DiscountCodeUser;
use App\Models\Token;
use App\Models\User;
use Genocide\Radiocrud\Services\ActionService\ActionService;
use Illuminate\Support\Facades\App;

class DiscountCodeAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(DiscountCode::class)
            ->setValidationRules([
                'storeByAdmin' => [
                    'code' => ['required', 'string', 'max:150', 'unique:discount_codes'],
                    'type' => ['required', 'string', 'max:150', 'in:tamdid,nardeban,fori'],
                    'percent' => ['required', 'int', 'min:1', 'max:100'],
                    'max_price' => ['int', 'digits_between:0,20'],
                    'expires_at' => ['date_format:Y-m-d H:i:s'],
                    'users' => ['array'],
                    'users.*' => ['int'],
                ],
                'update' => [
                    'code' => ['string', 'max:150', 'unique:discount_codes'],
                    'type' => ['string', 'max:150', 'in:renewal,nardeban,fori'],
                    'percent' => ['int', 'between:1,100'],
                    'max_price' => ['int', 'digits_between:0,20'],
                    'expires_at' => ['date_format:Y-m-d H:i:s'],
                    'users' => ['array'],
                    'users.*' => ['int'],
                ],
                'get_query' => [
                    'search' => ['string', 'max:255'],
                ]
            ]);

        parent::__construct();
    }
    public function store(array $data, callable $storing = null): mixed
    {
        $discountCodeUsers = [];
        $discount = parent::store($data, $storing);
        if(! isset($data['users']))
        {
            $data['users'] = User::query()->get()->pluck('id');
        }

        foreach ($data['users'] AS $userId)
        {
            $discountCodeUsers[] = [
                'discount_id' => $discount->id,
                'user_id' => $userId
            ];
        }
        DiscountCodeUser::query()->insert($discountCodeUsers);
        return $discount;
    }

    public function updateDiscount($id): bool|int
    {
        $data = $this->getDataFromRequest();
        $discount = DiscountCode::query()->where('id', $id)->first();

        if(! isset($data['users']))
        {
            $data['users'] = User::query()->get()->pluck('id');
        }

        foreach ($data['users'] AS $userId)
        {
            $discountCodeUsers[] = [
                'discount_id' => $discount->id,
                'user_id' => $userId
            ];
        }
        DiscountCodeUser::query()->where('discount_id', $discount->id)->delete();
        DiscountCodeUser::query()->insert($discountCodeUsers);

        if(isset($data['code']))
        {
            $discount->code = $data['code'];
        }
        if(isset($data['percent']))
        {
            $discount->percent = $data['percent'];

        }
        if(isset($data['max_price']))
        {
            $discount->max_price = $data['max_price'];

        }
        if(isset($data['type']))
        {
            $discount->type = $data['type'];

        }
        if(isset($data['expires_at']))
        {
            $discount->expires_at = $data['expires_at'];

        }
        $discount->save();

        return 1;
    }

    public function getById2($id)
    {
        $discount = DiscountCode::query()->where('id', $id)->first();
        $discountUsers_id = DiscountCodeUser::query()->where('discount_id', $id)->get()->pluck('user_id');
        $users = User::query()->whereIn('id',$discountUsers_id)->get();
        if($users == User::all())
        {
            $users = "همه کاربران";
        }
        return [
            'discount_code' => $discount,
            'users' => $users
        ];
    }

    public function getByCode($code,$token)
    {
        $discount = DiscountCode::query()->where('code', $code)->firstOrFail();
        $discountUsers_id = DiscountCodeUser::query()->where('discount_id', $discount->id)->get()->pluck('user_id');
        $user_id = Token::query()->where('token', $token)->first()->user_id;

        foreach ($discountUsers_id AS $userId)
        {
            if($userId == $user_id)
            {
                return $discount;

            }
        }
        abort(403, trans('messages.auth_fail'));
    }
}
