<?php

namespace App\Actions;

use App\Http\Resources\OrderResource;
use App\Models\Order;
use App\Models\Token;
use App\Models\User;
use Genocide\Radiocrud\Exceptions\CustomException;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class OrderAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(Order::class)
            ->setResource(OrderResource::class)
            ->setValidationRules([
                'get_query' => [
                    'status' => 'string',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ],
                'change_status' => [
                    'order_id' => 'required|int'
                ]
            ])
            ->setQueryToEloquentClosures([
                'status' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('status', $query['status']);
                    });
                },
                'from_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['from_created_at']);
                },
            ]);
        parent::__construct();
    }

    public function purchaseConfirmation($token)
    {
        $data = $this->getDataFromRequest();
        $user_id = Token::query()->where('token', $token)->first()->user_id;

        $order = Order::query()->where('id', $data['order_id'])->first();
        $order->has_reached_destination = true;
        $order->save();

        $advertiser = User::query()->where('id', $order->advertiser_id)->first();
        $advertiser->withdrawable_amount += $order->price;
        $advertiser->save();

        throw new CustomException('رسيدن محصول به دست كاربر تاييد شد',200,200);

    }

    public function getUserOrder($token)
    {
        $user_id = Token::query()->where('token', $token)->first()->user_id;
        return OrderResource::collection(Order::query()->where('user_id', $user_id)->get());
    }
}
