<?php

namespace App\Actions;

use App\Models\Chat;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class ProductChatAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(Chat::class)
            ->setValidationRules([
                'user_get_query' => [
                    'product_id' => 'int|nullable',
                    'user_id' => 'int',
                    'sender' => 'string',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ],
                'get_query' => [
                    'product_id' => 'string|nullable',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ]
            ])
            ->setQueryToEloquentClosures([
                'product_id' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('product_id', $query['product_id']);
                    });
                },
                'from_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['from_created_at']);
                },
            ]);
        parent::__construct();
    }
}
