<?php

namespace App\Actions;

use App\Http\Resources\ChatResource;
use App\Http\Resources\TransactionResource;
use App\Models\Chat;
use App\Models\Message;
use App\Models\Report;
use App\Models\Transaction;
use App\Models\User;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class ReportAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(Report::class)
            ->setValidationRules([
                'get_query' => [
                    'chat_id' => 'int',
                    'reason' => 'string',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ]
            ])
            ->setQueryToEloquentClosures([
                'chat_id' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('chat_id', $query['chat_id']);
                    });
                },
                'reason' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('reason', 'LIKE', "%{$query['reason']}%");
                    });
                },
                'from_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['from_created_at']);
                },
            ]);
        parent::__construct();
    }
}
