<?php

namespace App\Actions;

use App\Http\Resources\SeoResource;
use App\Http\Resources\TransactionResource;
use App\Models\Seo;
use App\Models\Transaction;
use App\Models\User;
use Closure;
use Genocide\Radiocrud\Services\ActionService\ActionService;
use Illuminate\Validation\Rule;

class SeoAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(Seo::class)
            ->setResource(SeoResource::class)
            ->setValidationRules([
                'get_query' => [
                    'pageable_type' => 'string|max:100',
                    'pageable_id' => '',
                    'title' => 'string|max:100',
                    'description' => 'string|max:1000',
                    'language' => 'string|max:100',
                    'country' => '',
                    'city' => '',
                ],
                'store' => [
                    'pageable_type' => ['required', 'string', 'max:100', function (string $attribute, mixed $value, Closure $fail) {
                        if (!in_array($value, ['CATEGORY', 'SHOP', 'PRODUCT']) && Seo::where('pageable_type', $value)->where('language', $this->request->get('language'))
                                ->where(fn($builder) => $builder->where('country', $this->request->country)->where('city', $this->request->city))->exists()) {
                            $fail(trans('validation.unique'));
                        }
                    }],
                    'pageable_id' => ['string', 'max:100', 'nullable',
                        function (string $attribute, mixed $value, Closure $fail) {
                            $pageableType = $this->request->get('pageable_type');
                            if (in_array($pageableType, ['CATEGORY', 'SHOP', 'PRODUCT']) && (empty($value) || in_array($value, ['null', 'undefined', '[object Object]']))) {
                                $fail(trans('validation.filled'));
                                return false;
                            }

                            if (in_array($pageableType, ['CATEGORY', 'SHOP', 'PRODUCT']) &&
                                Seo::where('pageable_type', $pageableType)->where('pageable_id', $value)->where('language', $this->request->get('language'))
                                    ->where(fn($builder) => $builder->where('country', $this->request->country)->where('city', $this->request->city))->exists()) {
                                $fail(trans('validation.unique'));
                                return false;
                            }

                        },
                    ],
                    'title' => 'string|max:100|required',
                    'description' => 'string|max:1000|required',
                    'language' => 'string|max:100',
                    'country' => ['max:100',
                        Rule::requiredIf(fn() => !$this->request->get('city'))
                    ],
                    'city' => ['max:100', Rule::requiredIf(fn() => !$this->request->get('country'))],
                    'h1' => 'string|max:100|nullable',
                    'content' => 'string|nullable',
                ],
                'update' => [
                    'pageable_type' => 'string|max:100',
                    'pageable_id' => 'string|max:100|nullable',
                    'title' => 'string|max:100',
                    'description' => 'string|max:100',
                    'language' => 'string|max:100',
                    'country' => 'string|max:100|nullable',
                    'city' => 'string|max:100|nullable',
                    'h1' => 'string|max:100|nullable',
                    'content' => 'string|nullable',
                ],
            ])
            ->setQueryToEloquentClosures([
                'pageable_type' => function (&$eloquent, $query) {
                    if (is_array($query['pageable_type'])) {
                        $eloquent = $eloquent->where(function ($q) use ($query) {
                            $q
                                ->whereIn('pageable_type', $query['pageable_type']);
                        });
                    } else {
                        $eloquent = $eloquent->where(function ($q) use ($query) {
                            $q
                                ->where('pageable_type', $query['pageable_type']);
                        });
                    }
                },
                'pageable_id' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('pageable_id', $query['pageable_id']);
                    });
                },
                'title' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('title', $query['title']);
                    });
                },
                'description' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('description', $query['description']);
                    });
                },
                'language' => function (&$eloquent, $query) {
                    $eloquent = $eloquent->where(function ($q) use ($query) {
                        $q
                            ->where('language', $query['language']);
                    });
                },
                'country' => function (&$eloquent, $query) {
                    if (is_array($query['country'])) {
                        $eloquent = $eloquent->where(function ($q) use ($query) {
                            $q
                                ->whereIn('country', $query['country']);
                        });
                    } else {
                        $eloquent = $eloquent->where(function ($q) use ($query) {
                            $q
                                ->where('country', $query['country']);
                        });
                    }
                },
                'city' => function (&$eloquent, $query) {
                    if (is_array($query['city'])) {
                        $eloquent = $eloquent->where(function ($q) use ($query) {

                            $q
                                ->whereIn('city', $query['city']);
                        });
                    } else {
                        $eloquent = $eloquent->where(function ($q) use ($query) {
                            $q
                                ->where('city', $query['city']);
                        });
                    }
                },
            ]);
        parent::__construct();
    }
}
