<?php

namespace App\Actions;

use App\Http\Resources\TransactionResource;
use App\Models\Transaction;
use App\Models\User;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class TransactionAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(Transaction::class)
            ->setResource(TransactionResource::class)
            ->setValidationRules([
                'get_query' => [
                    'first_name' => 'string|max:100',
                    'last_name' => 'string|max:100',
                    'status' => 'boolean',
                    'price' => 'int',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ]
            ])
            ->setQueryToEloquentClosures([
                'first_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('user', function ($q) use ($query){
                        $q->where('users.name', $query['first_name']);
                    });
                },
                'last_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereHas('user', function ($q) use ($query){
                        $q->where('users.lastname', $query['last_name']);
                    });
                },
                'price' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('paid', $query['price']);
                    });
                },
                'status' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('status', $query['status']);
                    });
                },
                'from_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['from_created_at']);
                },
            ]);
        parent::__construct();
    }

    public function tetherPendingList()
    {
        return $this->applyResourceToCollection(Transaction::query()
            ->whereIn('promote_type', ['fori-tether','nardeban-tether','renrewal-tether'])
            ->where('status', '1')
            ->get());
    }
}
