<?php

namespace App\Actions;

use App\Models\Token;
use App\Models\User;
use Genocide\Radiocrud\Services\ActionService\ActionService;

class UserAction extends ActionService
{
    public function __construct()
    {
        $this
            ->setModel(User::class)
            ->setValidationRules([
                'get_query' => [
                    'first_name' => 'string|max:100',
                    'last_name' => 'string|max:100',
                    'city' => 'string|max:100',
                    'mobile' => 'string|max:100',
                    'email' => 'string|max:100',
                    'from_created_at' => ['date_format:Y-m-d'],
                    'to_created_at' => ['date_format:Y-m-d'],
                ],
                'cc_update' => [
                    'cc_number' => 'required|string|max:100',
                    'shaba_number' => 'required|string|max:100',
                    'cc_account_name' => 'required|string|max:100',
                    'cvv2' => 'string|max:100',
                    'expiration_date' => 'string|max:100',
                ],
                'status_update' => [
                    'status' => ['boolean', 'required'],
                    'user_id' => ['int', 'required']
                ]
            ])
            ->setQueryToEloquentClosures([
                'first_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('name', 'LIKE', "%{$query['first_name']}%");
                    });
                },
                'last_name' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('lastname', 'LIKE', "%{$query['last_name']}%");
                    });
                },
                'city' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('city', 'LIKE', "%{$query['city']}%");
                    });
                },
                'mobile' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('phone', 'LIKE', "%{$query['mobile']}%");
                    });
                },
                'email' => function(&$eloquent, $query)
                {
                    $eloquent = $eloquent->where(function ($q) use ($query){
                        $q
                            ->where('email', 'LIKE', "%{$query['email']}%");
                    });
                },
                'from_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '>=', $query['from_created_at']);
                },
                'to_created_at' => function (&$eloquent, $query)
                {
                    $eloquent = $eloquent->whereDate('created_at', '<=', $query['to_created_at']);
                },
            ]);
        parent::__construct();
    }

    public function ccUpdate($token)
    {
        $user_id = Token::query()->where('token', $token)->first()->user_id;
        $data = $this->getDataFromRequest();

        $user = User::query()->where('id', $user_id)->update([
            'cc_number' => $data['cc_number'] ?? null,
            'shaba_number' => $data['shaba_number'] ?? null,
            'cc_account_name' => $data['cc_account_name'] ?? null,
            'cvv2' => $data['cvv2'] ?? null,
            'expiration_date' => $data['expiration_date'] ?? null
        ]);
         return $user;
    }

    public function getCC($token)
    {
        $user_id = Token::query()->where('token', $token)->first()->user_id;
        return User::query()->where('id', $user_id)->get(['cc_number', 'shaba_number', 'cc_account_name', 'cvv2', 'expiration_date', 'wallet_amount']);
    }

    public function changeUserStatus()
    {
        $data = $this->getDataFromRequest();
        $user = User::query()->where('id', $data['user_id'])->first();
        $user->status = $data['status'];
        $user->save();

        return $user;
    }
}
