<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\RoleRequest;
use App\Models\Role;
use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class RoleController extends Controller
{
    public function __construct()
    {
        $this->middleware('check_role:ROLES_LIST')->only(['index']);
        $this->middleware('check_role:ROLES_CREATE')->only(['store']);
        $this->middleware('check_role:ROLES_SHOW')->only(['show']);
        $this->middleware('check_role:ROLES_UPDATE')->only(['update']);
        $this->middleware('check_role:ROLES_DELETE')->only(['destroy']);
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Database\Eloquent\Builder[]|\Illuminate\Database\Eloquent\Collection
     */
    public function index()
    {
        return Role::query()->get();
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param RoleRequest $request
     * @return \Illuminate\Database\Eloquent\Builder|\Illuminate\Database\Eloquent\Model
     */
    public function store(RoleRequest $request)
    {
        $data = $request->validated();

        $role = Role::query()->create([
            'title' => $data['name']
        ]);

        $role?->permissions()->sync($data['permissions']);

        $role?->load(['permissions']);

//        return $role
//            ? $role
//            : throw new CustomException('عمليات با خطا مواجه شد',400,400);
    }

    /**
     * Display the specified resource.
     *
     * @param Role $role
     * @return \Illuminate\Database\Eloquent\Builder|\Illuminate\Database\Eloquent\Model|object
     */
//    public function show(Role $role)
//    {
//        $role->load(['permissions']);
//        return $role;
//    }

    public function show($id)
    {
        $role = Role::query()->where('id', $id)->with('permissions')->first();
        return $role;
    }

    /**
     * Update the specified resource in storage.
     *
     * @param RoleRequest $request
     * @param Role $role
     * @return \Illuminate\Database\Eloquent\Builder|\Illuminate\Database\Eloquent\Model|object
     */
//    public function update(RoleRequest $request, Role $role)
//    {
//        $data = $request->validated();
//
//        $status = $role->update([
//            'title' => $data['name']
//        ]);
//
//        $role->permissions()->sync($data['permissions']);
//
//        $role->load(['permissions']);
//
//        return $status
//            ? $role
//            : throw new CustomException('عمليات با خطا مواجه شد',400,400);
//    }

    public function update(RoleRequest $request, $id)
    {
        $data = $request->validated();
        $role = Role::query()->where('id', $id)->first();
        $status = $role->update([
            'title' => $data['name']
        ]);

        $role->permissions()->sync($data['permissions']);

        $role->load(['permissions']);

        return $status
            ? $role
            : throw new CustomException('عمليات با خطا مواجه شد',400,400);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Role $role
     * @return JsonResponse
     */
//    public function destroy(Role $role)
//    {
//        return $role->delete()
//            ? null
//            : throw new CustomException('عمليات با خطا مواجه شد',400,400);
//    }

    public function destroy($id)
    {
        $role = Role::query()->where('id',$id)->first();

        return $role->delete()
            ? null
            : throw new CustomException('عمليات با خطا مواجه شد',400,400);
    }
}
