<?php

namespace App\Http\Controllers;

use App\Actions\ChatAction;
use App\Actions\ProductChatAction;
use App\Actions\ReportAction;
use App\Models\BlockChat;
use App\Models\Chat;
use App\Models\Message;
use App\Models\Report;
use App\Models\Setting;
use App\Models\User;
use Carbon\Carbon;
use Elastic\Elasticsearch\ClientBuilder;
use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Traits\ElasticFunctions;


class ChatController extends Controller
{

    use ElasticFunctions;



    public function show(Request $request)

    {
                date_default_timezone_set('Asia/Tehran');

        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $chat_setting = Setting::where('user_id', $request->id)->first();
        $chats = Chat::where('user_id',$request->id)->orwhere('advertiser_id', $request->id)->get();
        $valid_chats = [];
        $total_chats = [];
        foreach ($chats as $chat){
            $params = [
                'index' => 'products',
                'body' => [
                    'query' => [
                        "match" => [
                            '_id' => $chat->product_id
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);

            if($response['hits']['hits']) {
                $endDate = Carbon::now();
                $startDateLeft = Carbon::parse($response['hits']['hits'][0]['_source']['object']['renewal_at']);
                $diffInDays = $startDateLeft->diffInDays($endDate);
                $time_left = 29 - $diffInDays;

                if ($time_left > 0) {
                    array_push($valid_chats,$chat);
                }
            }
        }

        foreach ($valid_chats as $valid_chat) {
            if ($valid_chat->status == 1) {
                array_push($total_chats, $valid_chat);
            }
        }
        //show blocked chats only
        if ($chat_setting->blocked_chat == 1) {
            $b_chats = collect($valid_chats)->where('status' , 0)->all();
            foreach ($b_chats as $b_chat){
                array_push($total_chats,$b_chat);
            }
            //show expired ads chats only
        } if($chat_setting->deactive_chat == 1) {
        $ch = Chat::where('user_id',$request->id)->orwhere('advertiser_id', $request->id)->get();
        $chats = [];
        foreach ($ch as $chat){
            $params = [
                'index' => 'products',
                'body' => [
                    'query' => [
                        "match" => [
                            '_id' => $chat->product_id
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);

            if($response['hits']['hits']) {
                $endDate = Carbon::now();
                $startDateLeft = Carbon::parse($response['hits']['hits'][0]['_source']['object']['renewal_at']);
                $diffInDays = $startDateLeft->diffInDays($endDate);
                $time_left = 29 - $diffInDays;

                if ($time_left <= 0) {
                    array_push($total_chats,$chat);
                }
            }
        }
    }
        $total_chats = array_unique($total_chats);
        $products = [];
        if ($total_chats) {
            foreach ($total_chats as $chat) {

                $params = [
                    'index' => 'products',
                    'body' => [
                        'query' => [
                            "match" => [
                                '_id' => $chat->product_id
                            ]
                        ]
                    ]
                ];
                $response = $this->client->search($params);

                if ($response['hits']['hits']) {
                    $chat->product = $response['hits']['hits'][0]['_source'];
                    array_push($products, $chat);
                    $last_msg = Message::where('chat_id' , $chat->id)->orderBy('created_at' , 'desc')->first();
                    $chat->last_message = $last_msg;
                }
            }


            //get blockers id
            foreach ($products as $product){
                $blocker_id = [];
                if($product->status == 0){
                    $blockchats = BlockChat::where('chat_id' , $product->id)->where('blocked' , 1)->get();
                    foreach ($blockchats as $blockchat){
                        array_push($blocker_id,$blockchat->user_id);
                    }
                    $product->blockers_id = $blocker_id;
                }
            }

            return [
                'total' => [
                    'value' => count([$products][0]),
                    'relation' => $response['hits']['total']['relation']
                ],
                'meta' => [
                    'perpage' => $size,
                    'currentpage' => $page,
                    'totalpage' => ceil(count([$products][0]) / $size),
                    'totalrows' => count([$products][0]),
                ],
                'max_score' => $response['hits']['max_score'],
                'hits' => [$products]
            ];
        } else {
            return [
                'total' => [
                    'value' => count([$products][0]),
                ],
                'meta' => [
                    'perpage' => $size,
                    'currentpage' => $page,
                    'totalpage' => ceil(count([$products][0]) / $size),
                    'totalrows' => count([$products][0]),
                ],
                'hits' => []
            ];
        }


    }


    public function store(Request $request)
        {
                            date_default_timezone_set('Asia/Tehran');

        $chat = Chat::where('user_id' ,$request->id)
            ->where('advertiser_id' , $this->getProductUserId($request))
            ->where('product_id' , $request->product_id)->first();
        if(!$chat){

        Chat::query()
            ->create([
                'user_id' => $request->id,
                'product_id' => $request->product_id,
                'advertiser_id' => $this->getProductUserId($request),
            ]);

        }else{
            abort(403,'this chat already exist');
        }
    }


    public function chatBlock(Request $request)
        {
        $chat = Chat::where('id' , $request->chat_id)->first();
        $blockchat = BlockChat::where('user_id' , $request->id)->where('chat_id' , $request->chat_id)->first();
        if ($request->block_chat == 1) {
            if ($chat){
                if(($chat->user_id ==$request->id) or ($chat->advertiser_id ==$request->id)){
                    if ($blockchat){
                        $blockchat->update(['blocked'=>1]);
                        $chat->update(['status' => 0]);
                    }else{
                        BlockChat::create([
                            'blocked' => 1,
                            'user_id' => $request->id,
                            'chat_id' => $request->chat_id
                        ]);
                        $chat->update(['status' => 0]);
                    }
                }else{
                    abort(403,'access denied');
                }
            }else{
            abort(403,'this chat doesnt exist');
            }

        }elseif ($request->block_chat == 0){
            if ($chat){
                if($blockchat && $blockchat->blocked == 1){
                    $blockchat->update(['blocked'=> 0]);
                    $b_chats = BlockChat::where('chat_id' , $request->chat_id)->get();
                    $count = 0;
                    foreach ($b_chats as $b_chat){
                        if($b_chat->blocked ==1){
                            break;
                        }else{
                            $count++;
                        }
                    }
                    if ($count == 2){
                        $chat->update(['status' => 1]);
                    }
                }else{
                    abort(403 , 'this chat already unblocked');
                }
            }else{
                abort(403,'this chat doesnt exist');
            }
        }

    }


    public function showMessages(Request $request)
    {
        $messages = Message::where('chat_id', $request->chat_id)->orderBy('id', 'ASC')->get();
        $arr = [];
        foreach ($messages as $message) {
            //the date of creation
            $startDate = Carbon::createFromFormat('Y-m-d H:i:s', $message['created_at'], 'UTC')->diffForHumans();
            $message['creation_date'] = $startDate;
            array_push($arr, $message);
        }


        return $messages;
    }


    public function storeMessage(Request $request)


    {
        date_default_timezone_set('Asia/Tehran');


        if (! empty($request->chat_id) && $request->chat_id != null && $request->chat_id != 'null') {
            $s = Chat::where('user_id', $request->id)->orwhere('advertiser_id',$request->id)->first();
            $params = [
                'index' => 'products',
                'body' => [
                    'query' => [
                        'match' => [
                            '_id' => $request->product_id
                        ]
                    ]
                ]
            ];
            $response = $this->client->search($params);
            $chat_time = $response['hits']['hits'][0]['_source']['object']['renewal_at'];

            //days left
            $endDate = Carbon::now();
            $startDateLeft = Carbon::parse($response['hits']['hits'][0]['_source']['object']['renewal_at']);
            $diffInDays = $startDateLeft->diffInDays($endDate);
            $time_left = 29 - $diffInDays;

            if ($s->status == 1) {
                if ($time_left>=0) {
                    if ($s->user_id = $request->id) {
                        $send = 'user';
                    } else {
                        $send = 'advertiser';
                    }

                    $validate = Validator::make($request->all(), [
                        'chat_id' => 'nullable',
                        'text' => 'nullable|text',
                        'store_file' => 'nullable'

                    ]);
                    if (count($validate->errors()) > 0) {
                        foreach ($validate->errors()->messages() AS $key => $value)
                        {
                            if(str_contains($value[0],'required'))
                            {
                                throw new CustomException(trans('messages.'.$key),422,422);
                            }
                            else
                            {
                                throw new CustomException($key." ".$value[0], 422,422);

                            }
                        }
                    } else {
                        $msg = new Message();
                        $msg->fill([
                            'chat_id' => $request->chat_id,
                            'user_id' => $request->id
                        ]);

                        if ($request->message) {
                            $msg->fill([
                                'text' => $request->message,
                                'store_file' => null
                            ]);
                        } elseif ($request->store_file) {
                            $msg->store_file = env('PUBLIC_URL').'/storage/app/' . $request->file('store_file')->store('store_file');
                            $msg->fill([
                                'store_file' => $msg->store_file,
                                'text' => null
                            ]);
                        }
                        $msg->sender = $send;
                        $msg->save();
                    }
                }else{
                    abort(403,'this ad is expired');
                }
            } else {
                abort(403, 'this chat is blocked');
            }

        } else {
            $chat = Chat::where('user_id', $request->id)->where('product_id', $request->product_id)->first();
            if (!$chat) {
                if ($request->id === $this->getProductUserId($request)) {
                    abort(403, 'access denied');
                } else {
                    $this->store($request);
                    $chat = Chat::where('user_id', $request->id)->where('product_id', $request->product_id)->first();
                }

            }

            $s = Chat::where('user_id', $request->id)->where('product_id', $request->product_id)->first();
            if ($s->status == 1) {
                if ($s->user_id = $request->id) {
                    $send = 'user';
                } else {
                    $send = 'advertiser';
                }

                $validate = Validator::make($request->all(), [
                    'chat_id' => 'nullable',
                    'text' => 'nullable|text',
                    'store_file' => 'nullable'

                ]);

                if (count($validate->errors()) > 0) {
                    foreach ($validate->errors()->messages() AS $key => $value)
                    {
                        if(str_contains($value[0],'required'))
                        {
                            throw new CustomException(trans('messages.'.$key),422,422);
                        }
                        else
                        {
                            throw new CustomException($key." ".$value[0], 422,422);

                        }
                    }
                } else {
                    $msg = new Message();
                    $msg->fill([
                        'chat_id' => $s->id,
                        'user_id' => $request->id

                    ]);

                    if ($request->message) {
                        $msg->fill([
                            'text' => $request->message,
                            'store_file' => null
                        ]);

                    } elseif ($request->store_file) {
                        $msg->store_file = env('PUBLIC_URL').'/storage/app/' . $request->file('store_file')->store('store_file');
                        $msg->fill([
                            'store_file' => $msg->store_file,
                            'text' => null
                        ]);
                    }


                }
            } else {
                abort(403, 'this chat is not active');
            }


        }
        $msg->sender = $send;
        $msg->save();

        $startDate = Carbon::createFromFormat('Y-m-d H:i:s', $msg['created_at'], 'UTC')->diffForHumans();
        $msg['creation_date'] = $startDate;

        $advertiser_id = $this->getProductUserId($request);
        return $msg;

    }


    public function deleteSingleChat(Request $request)
    {
        $chats = Chat::where('user_id', $request->id)->orwhere('advertiser_id', $request->id)->get();
        foreach ($chats as $chat) {
            if ($request->input('chat_id') == $chat->id) {
                $chat->delete();
            }
        }
    }


    public function deleteChats(Request $request)
    {
        $chats = Chat::where('user_id', $request->id)->orwhere('advertiser_id', $request->id)->get();
        if ($chats !== null) {
            foreach ($chats as $chat) {
                if (is_array($request->chat_id)) {
                    foreach ($request->chat_id as $key) {
                        if ($key == $chat->id) {
                            $chat->delete();
                        }
                    }
                } else {
                    if ($request->chat_id == $chat->id) {
                        $chat->delete();
                    }
                }

            }
        } else {
            abort(403, 'this user doesnt have any chat');
        }

    }


    public function showSetting(Request $request)
    {
        $user_setting = Setting::where('user_id', $request->id)->get();
        return $user_setting;
    }



    public function updateSetting(Request $request)
    {
        $user = User::where('id', $request->id)->first();
        $validate = Validator::make($request->all(), [
            'deactive_chat' => 'required',
            'blocked_chat' => 'required'
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $setting = Setting::where('user_id', $request->id)->first();
            $setting->fill([
                'name' => $setting->name,
                'deactive_chat' => $request->deactive_chat,
                'blocked_chat' => $request->blocked_chat
            ]);
            if ($request->name) {
                $setting->name = $request->name;
            }
            $setting->save();
        }
    }


    public function showChatReports(Request $request)
    {
        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $reports = Report::skip(($page - 1) * $size)->limit($size)->orderByDesc("created_at")->get();

        return [
            'meta' => [
                'perpage' =>$size,
                'currentpage' =>$page,
                'totalpage' =>ceil(count($reports)/$size),
                'totalrows' =>count($reports),
            ],
            'hits' => $reports,
        ];
    }


    public function storeReport(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'reason' => 'nullable',
            'message' => 'nullable|min:4',
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() AS $key => $value)
            {
                if(str_contains($value[0],'required'))
                {
                    throw new CustomException(trans('messages.'.$key),422,422);
                }
                else
                {
                    throw new CustomException($key." ".$value[0], 422,422);

                }
            }
        } else {
            $report = new Report();
            $report->reason = $request->reason;
            $report->message = $request->message;
            $report->chat_id = $request->chat_id;
            $chat = Chat::where('id', $request->chat_id)->first();
            if ($chat->user_id == $request->id) {
                $report->reporter_id = $request->id;
                $report->reported_id = $chat->advertiser_id;
            } elseif ($chat->advertiser_id == $request->id) {
                $report->reporter_id = $chat->advertiser_id;
                $report->reported_id = $request->id;
            } else {
                abort(403, 'chat not found');
            }
            $report->save();
        }
    }



    public function productChatReports(Request $request)
    {
        $chats = Chat::where('product_id',$request->product_id)->get();
        $reports =[];
        foreach ($chats as $chat){
            $report = Report::where('chat_id',$chat->id)->first();
            if($report){
                array_push($reports,$report);
            }
        }
        return $reports;

    }


    public function getProductUserId(Request $request)
    {
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'match' => [
                        '_id' => $request->input('product_id')
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        if ($response['hits']['hits']) {
            return $response['hits']['hits'][0]['_source']['id']; // documents
        } else {
            abort(404, 'ad not found');
        }
    }

    public function get(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Messages List : ',
            'data' => (new ChatAction())
                ->setRequest($request)
                ->setValidationRule('user_get_query')
                ->setOrderBy(['id' => 'ASC'])
                ->makeEloquentViaRequest()
                ->getByUserRequest()
        ]);
    }
    public function adminGet(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Messages List : ',
            'data' => (new ChatAction())
                ->setRequest($request)
                ->setValidationRule('get_query')
                ->setOrderBy(['id' => 'DESC'])
                ->makeEloquentViaRequest()
                ->getByRequestAndEloquent()
        ]);
    }

    public function getReports(Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Reports List : ',
            'data' => (new ReportAction())
                ->setRequest($request)
                ->setValidationRule('get_query')
                ->makeEloquentViaRequest()
                ->getByRequestAndEloquent()
        ]);
    }

    public function allChats(Request $request)
    {
        return response()->json([
            'success' => true,
            'message' => 'Chats List : ',
            'data' => (new ProductChatAction())
                ->setRequest($request)
                ->setValidationRule('get_query')
                ->makeEloquentViaRequest()
                ->getByRequestAndEloquent()
        ]);
    }
}
