<?php

namespace App\Http\Controllers\Dashboard;

use App\Http\Controllers\Controller;
use App\Models\DiscountCode;
use App\Models\DiscountCodeUser;
use App\Models\Favorite;
use App\Models\LastVisit;
use App\Models\Note;
use App\Models\Order;
use App\Models\Referral;
use App\Models\Token;
use App\Models\Transaction;
use App\Models\User;
use App\Traits\ElasticFunctions;
use Carbon\Carbon;
use Closure;
use Genocide\Radiocrud\Exceptions\CustomException;
use Genocide\Radiocrud\Models\KeyValueConfig;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;
use Shetabit\Multipay\Exceptions\InvalidPaymentException;
use Shetabit\Multipay\Exceptions\PurchaseFailedException;
use Shetabit\Multipay\Invoice;
use Shetabit\Payment\Facade\Payment;
use Exception;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;


//use Shetabit\Multipay\Payment;

class ProfileController extends Controller
{
    use ElasticFunctions {
        ElasticFunctions::__construct as acceptElasticProvider;
    }


    public function __construct(
        public \App\Services\Transaction $transactionService
    )
    {
        $this->acceptElasticProvider();


    }

    public function show(Request $request)
    {

        $user = User::query()->where('id', $request->id)->with(['business'])->first();
        $refStatus = Referral::query()->where('referred_id', $user->id)->first();
        if (!empty($refStatus) && $refStatus->is_valid == false) {
            $refStatus->is_valid = true;
            $refUser = User::query()->where('id', $refStatus->referrer_id)->first();
            if (!empty($refUser)) {
                $refUser->referred_count++;
                if ($refUser->referred_count == 15) {
                    $refUser->wallet_amount += 150000;
                }
                $refUser->save();
            }
            $refStatus->save();
        }
        if (!empty($user->business)) {
            $user = $user->toArray();
            $user['has_bussiness'] = true;
        } else {
            $user = $user->toArray();
            $user['has_bussiness'] = false;
        }
        return $user;
    }


    public function showUserById(Request $request)
    {
        $user = User::query()->where('id', $request->user_id)->with(['business'])->first();
        if (!empty($user->business)) {
            $user = $user->toArray();
            $user['has_bussiness'] = true;
        } else {
            $user = $user->toArray();
            $user['has_bussiness'] = false;
        }
        return $user;
    }


    public function update(Request $request)
    {

        $validate = Validator::make($request->all(), [
            'name' => 'nullable|string',
            'lastname' => 'nullable|string',
            'email' => ['nullable', 'email', Rule::unique('users')->ignore($request->id, 'id')],
            'address' => 'nullable',
            'location' => 'nullable',
            'city' => 'nullable',
            'notification' => 'nullable|boolean'
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() as $key => $value) {
                if (str_contains($value[0], 'required')) {
                    throw new CustomException(trans('messages.' . $key), 422, 422);
                } else {
                    throw new CustomException($key . " " . $value[0], 422, 422);
                }
            }
        }
        $user = User::find($request->id);
        $data = $request->all();
        //TODO::what is business
        unset($data['business']);
        unset($data['has_bussiness']);
        $user->update($data);
        return $user;
    }

    public function delete(Request $request)
    {
        $user = User::where('id', $request->user_id)->delete();
    }

    public function myAds(Request $request)
    {
        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            "size" => $size,
            "from" => (($page < 1 ? 1 : $page) - 1) * $size,
            'body' => [
                'query' => [
                    'bool' => [
                        'must' => [
                            [
                                'match' => [
                                    'id' => $request->id
                                ],
                            ]
                        ],
                        'must_not' =>
                            [
                                'range' => [
                                    'object.shop' => [
                                        'gt' => 0
                                    ]
                                ]
                            ]
                    ]
                ],

            ]
        ];
        $response = $this->client->search($params);

        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];

    }

    public function myFavorites(Request $request)
    {
        $fav = Favorite::where('user_id', $request->id)->get();
        $p_id = [];
        foreach ($fav as $p) {
            array_push($p_id, $p->product_id);
        }


        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "filter" => [
                            [
                                "terms" => [
                                    "_id" => $p_id
                                ],
                            ],
                            [
                                "range" => [
                                    "object.renewal_at" => [
                                        "gte" => Carbon::now()->subDay(30)
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ];

        $response = $this->client->search($params);

        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];


    }


    public function deleteFavorites(Request $request)
    {
        $fav = Favorite::where('user_id', $request->id)->delete();
    }


    public function myNotes(Request $request)
    {
        $note = Note::where('user_id', $request->id)->get();
        $p_id = [];
        foreach ($note as $n) {
            array_push($p_id, $n->product_id);
        }


        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "filter" => [
                            [
                                "terms" => [
                                    "_id" => $p_id
                                ],
                            ]
                            , [
                                "range" => [
                                    "object.renewal_at" => [
                                        "gte" => Carbon::now()->subDay(30)
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ];

        $response = $this->client->search($params);

        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];


    }


    public function deleteNotes(Request $request)
    {
        $notes = Note::where('user_id', $request->id)->delete();
    }


    public function lastVisited(Request $request)
    {
        $lvs = LastVisit::where('user_id', $request->id)->orderByDesc('updated_at')->get();
        $p_id = [];
        foreach ($lvs as $lv) {
            array_push($p_id, $lv->product_id);
        }


        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $params = [
            'index' => 'products',
            'body' => [
                "size" => $size,
                "from" => (($page < 1 ? 1 : $page) - 1) * $size,
                "query" => [
                    "bool" => [
                        "filter" => [
                            [
                                "terms" => [
                                    "_id" => $p_id
                                ],
                            ],
                            [
                                "range" => [
                                    "object.renewal_at" => [
                                        "gte" => Carbon::now()->subDay(30)
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ]
        ];

        $response = $this->client->search($params);

        return [
            'total' => $response['hits']['total'],
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil($response['hits']['total']['value'] / $size),
                'totalrows' => $response['hits']['total']['value'],
            ],
            'max_score' => $response['hits']['max_score'],
            'hits' => $response['hits']['hits'],
        ];


    }


    public function deleteLastVisited(Request $request)
    {
        $lvs = LastVisit::where('user_id', $request->id)->delete();
    }


    public function fori(Request $request)
    {
        $p = $this->purchase($request);

    }

    public function purchase(Request $request)
    {
        $userId = $request->id;
        $discountCode = null;

        $attributes = $request->validate(
            [
                'product_id' => ['required'],
                'promote_type' => ['required', Rule::in(['fori', 'nardeban', 'renewal'])],
                'discount_code' => [
                    'nullable',
                    function (string $attribute, mixed $value, Closure $fail) use ($userId, &$discountCode) {
                        $discountCode = DiscountCode::query()->where('code', $value)
                            ->first();

                        if (!isset($discountCode)) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }
                        if (Str::upper($discountCode->type) !== Str::upper(\request('promote_type'))) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }

                        $discountCodeUser = $discountCode->discountCodeUsers()
                            ->where('user_id', $userId)
                            ->first();

                        if (!$discountCodeUser) {
                            $fail(trans('messages.wrong_user_code'));
                            return false;
                        }

                        if ($discountCodeUser?->is_used) {
                            $fail(trans('messages.used_code'));
                            return false;
                        }

                        if (Carbon::make($discountCode->expires_at)->timestamp < now()->timestamp) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }
                    },
                ],
                'gateway' => ['nullable']
            ]
        );

        $price = 0;
        $footer = KeyValueConfig::query()->where('key', 'footer')->first();


        $price = match ($attributes['promote_type']) {
            "fori" => $footer->value->fori_price,
            "renewal" => $footer->value->tamdid_price,
            "nardeban" => $footer->value->nardeban_price,
        };


        if (isset($attributes['discount_code']) && $discountCode) {
            $reducedAmount = $price * $discountCode->percent / 100;
            if ($reducedAmount > $discountCode->max_price) {
                $reducedAmount = $discountCode->max_price;
            }
            $price = $price - $reducedAmount;
        }
        $paymentId = md5(uniqid());


        $transaction = Transaction::create([
            'user_id' => $userId,
            'product_id' => $attributes['product_id'],
            'promote_type' => $attributes['promote_type'],
            'paid' => $price,
            'invoice_details' => null,
            'payment_id' => $paymentId,
            'discount_id' => $discountCode->id ?? null
        ]);

        try {

            $callbackUrl = route('purchase.result', ['payment_id' => $paymentId]);


            if (empty($price)) {
                return [
                    'action' => $callbackUrl
                ];
            }


            if ($request->has('gateway') && $request->gateway === 'STRIPE') {
                $user = User::find($userId);
                $payment = new \Stripe\StripeClient(env('STRIPE_SECRET'));
                $payment->checkout->sessions->create([
                    'success_url' => $callbackUrl,
                    'line_items' => [
                        [
                            'price_data' => [
                                'currency' => 'usd',
                                'unit_amount' => 100,
                                'product_data' => [
                                    'name' => $user->name,
                                    'description' => ''
                                ]
                            ],
                            'quantity' => $price,
                        ],
                    ],
                    'mode' => 'payment',
                ]);

                $transaction->transaction_id = $payment->id;
                $transaction->save();


                return [
                    'action' => $payment->success_url
                ];
            }

            $invoice = new Invoice();
            $invoice->amount($price);
            $payment = Payment::callbackUrl($callbackUrl);
            $payment->config('description', 'ارتقای آگهی');
            $payment->purchase($invoice, function ($driver, $transactionId) use ($transaction) {
                $transaction['transaction_id'] = $transactionId;
                $transaction->save();
            });
            return $payment->pay();

        } catch (Exception|PurchaseFailedException|SoapFault $e) {
            $transaction['transaction_result'] = $e;
            $transaction->status = Transaction::STATUS_FAILED;
            $transaction->save();

            throw new CustomException($e->getMessage(), 400, 400);
        }
    }

    public function nardeban(Request $request)
    {
        $p = $this->purchase($request);

    }

    public function createDiscountCode(Request $request)
    {
        $dis_code = DiscountCode::create([
            'code' => $this->newCode(),
            'user_id' => $request->user_id,
            'validity_time' => $request->validity_time,
            'percent' => $request->percent,
            'used' => false
        ]);

    }

    public function newCode()
    {
        $code = Str::random(8);
        return $code;
    }

    public function checkDiscountCode($code)
    {
        $discount = DiscountCode::where('code', $code)->first();
        //days left
        $now = date('Y-m-d H:i:s');
        $expireDate = $discount->expires_at;
        if (Carbon::parse($expireDate)->lt($now)) {
            abort(403, 'code is expired');
        } else {
            return $discount->percent;
        }
    }

    public function discountCodes()
    {
        $discountcodes = DiscountCode::all();
        return $discountcodes;
    }

    public function transactions(Request $request)
    {
        $size = $request->size ?? 10;
        $page = $request->page ?? 1;
        $transactions = Transaction::skip(($page - 1) * $size)->limit($size)->orderByDesc("created_at")->get();
        $arr = [];
        foreach ($transactions as $transaction) {
            $user = User::where('id', $transaction->user_id)->first();
            $collection = collect($user);
            $per_user = $collection->merge($transaction);
            array_push($arr, $per_user);
        }
        return [
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil(count($arr) / $size),
                'totalrows' => count(Transaction::all()),
            ],
            'hits' => $arr,
        ];

    }

    public function result(Request $request)
    {

        $transaction = Transaction::where('payment_id', $request?->payment_id)->firstOrFail();

        $status = null;
        $reference_id = null;
        $receipt = null;

        abort_if($transaction->status !== Transaction::STATUS_PENDING, 404);

        if (!empty($transaction->price)) {
            try {
                if ($transaction->gateway === 'ZARRINPAL') {
                    $receipt = Payment::amount($transaction->paid)
                        ->transactionId($transaction->transaction_id)
                        ->verify();
                }

                // todo change for stripe
                if ($transaction->gateway === 'STRIPE') {
                    $payment = new \Stripe\StripeClient(env('STRIPE_SECRET'));
                    $receipt = $payment->checkout
                        ->sessions
                        ->retrieve($transaction->transction_id);


                }


            } catch (InvalidPaymentException $e) {
                $transaction->update([
                        'status' => Transaction::STATUS_FAILED,
                        'transaction_result' => [
                            'message' => $e->getMessage(),
                            'code' => $e->getCode(),
                        ]
                    ]
                );

                return view('purchase', [
                    'status' => 0,
                    'message' => $e->getMessage()
                ]);
            }
        }


        DB::transaction(function () use ($transaction, $receipt) {
            if (!empty($transaction->discount_id)) {
                DiscountCodeUser::query()
                    ->where('user_id', $transaction->user_id)
                    ->where('discount_id', $transaction->discount_id)
                    ->update([
                        'is_used' => true
                    ]);
            }

            $transaction->transaction_result = serialize($receipt);
            $transaction->status = Transaction::STATUS_SUCCESS;
            $transaction->save();

            $this->{'set' . $transaction->promote_type}($transaction->product_id);
        });

        return view('purchase', [
            'status' => 1,
            'reference_id' => isset($receipt) ? $receipt->getReferenceId() : $transaction->payment_id
        ]);


        if ($transaction->promote_type === 'buy_product') {
            try {
                $receipt = Payment::amount($transaction->paid)
                    ->transactionId($transaction->transaction_id)
                    ->verify();

                $transaction->transaction_result = serialize($receipt);
                $transaction->status = Transaction::STATUS_SUCCESS;
                $transaction->save();
                $this->setPending($transaction->product_id);
                $this->addPriceToWallet($transaction->product_id);

                $params = [
                    'index' => 'products',
                    'body' => [
                        'query' => [
                            'bool' => [
                                "must" => [
                                    [
                                        "term" => ['_id' => "$transaction->product_id"]
                                    ],
                                ],
                            ]
                        ]
                    ]
                ];
                $response = $this->client->search($params);
                $price = $response['hits']['hits'][0]['_source']['object']['mains']['price'];
                $adv_id = $response['hits']['hits'][0]['_source']['id'];

                Order::query()->create([
                    'user_id' => $transaction->user_id,
                    'advertiser_id' => $adv_id,
                    'product_id' => $transaction->product_id,
                    'transaction_id' => $transaction->id,
                    'price' => $price,
                ]);

                $status = 1;
                $reference_id = $receipt->getReferenceId();
                return view('purchase', compact('status', 'reference_id'));
//                return ['status' => 1,'reference_id' =>$receipt->getReferenceId()];

//                return view('phonelogin')->with([
//                    'status' => 1,
//                    'reference_id' => $receipt->getReferenceId(),
////                'product_id' => $product_id
//                ]);

            } catch (Exception|InvalidPaymentException $e) {
                if ($e->getCode() < 0) {
                    $transaction['status'] = Transaction::STATUS_FAILED;
                    $transaction['transaction_result'] = [
                        'message' => $e->getMessage(),
                        'code' => $e->getCode(),
                    ];

                    $transaction->save();
                }
                $status = $e->getCode();
                $message = $e->getMessage();
                return view('purchase', compact('status', 'message'));
//                return view('purchase')->with([
//                    'status' => $e->getCode(),
//                    'message' => $e->getMessage()
//                ]);
            }
        }


    }

    public function setPending($product_id)
    {
        $params = [
            'index' => 'products',
            'id' => $product_id,
            'body' => [
                'doc' => [
                    'object' => [
                        'status' => "pending"
                    ]
                ],
            ]
        ];
        $response = $this->client->update($params);
    }

    public function addPriceToWallet($p_id)
    {
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'bool' => [
                        "must" => [
                            [
                                "term" => ['_id' => "$p_id"]
                            ],
                        ],
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        $price = $response['hits']['hits'][0]['_source']['object']['mains']['price'];
        $adv_id = $response['hits']['hits'][0]['_source']['id'];
        $advertiser = User::query()->where('id', $adv_id)->first();
        $advertiser->wallet_amount += $price;
        $advertiser->save();
    }

    public function payViaTether(Request $request)
    {
        $userId = $request->id;
        $attributes = $request->validate([
            'promote_type' => ['required', 'in:fori,nardeban,renewal'],
            'transaction_id' => ['required', 'string', 'max:100', 'unique:transactions,transaction_id'],
            'product_id' => ['required', 'string', 'max:100'],
            'network' => ['required', 'string', 'max:100'],
            'from' => ['required', 'string'],
            'discount_code' => [
                'nullable',
                function (string $attribute, mixed $value, Closure $fail) use ($userId, &$discountCode) {
                    $discountCode = DiscountCode::query()->where('code', $value)
                        ->first();

                    if (!isset($discountCode)) {
                        $fail(trans('messages.discount_fail'));
                        return false;
                    }
                    if (Str::upper($discountCode->type) !== Str::upper(\request('promote_type'))) {
                        $fail(trans('messages.discount_fail'));
                        return false;
                    }

                    $discountCodeUser = $discountCode->discountCodeUsers()
                        ->where('user_id', $userId)
                        ->first();

                    if (!$discountCodeUser) {
                        $fail(trans('messages.wrong_user_code'));
                        return false;
                    }

                    if ($discountCodeUser?->is_used) {
                        $fail(trans('messages.used_code'));
                        return false;
                    }

                    if (Carbon::make($discountCode->expires_at)->timestamp < now()->timestamp) {
                        $fail(trans('messages.discount_fail'));
                        return false;
                    }
                },
            ]
        ]);


        $price = 0;
        $footer = KeyValueConfig::query()->where('key', 'footer')->first();


        $price = match ($attributes['promote_type']) {
            "fori" => $footer->value->fori_price,
            "renewal" => $footer->value->tamdid_price,
            "nardeban" => $footer->value->nardeban_price,
        };

        if (isset($attributes['discount_code']) && $discountCode) {
            $reducedAmount = $price * $discountCode->percent / 100;
            if ($reducedAmount > $discountCode->max_price) {
                $reducedAmount = $discountCode->max_price;
            }
            $price = $price - $reducedAmount;
        }

        $promoteType = $attributes['promote_type'];
        $promoteType = "$promoteType" . "-tether";
        $txId = $attributes['transaction_id'];
        $productId = $attributes['product_id'];
        $address = $footer->toArray()['value']->wallet_address;

        $this->transactionService
            ->isValidCryptoTransaction(
                $attributes['network'],
                $attributes['transaction_id'],
                $attributes['from'],
                $address,
                $price
            );


        $paymentId = md5(uniqid());
        $transaction = Transaction::create([
            'user_id' => $request->id,
            'product_id' => $productId,
            'promote_type' => $promoteType,
            'paid' => $price,
            'payment_id' => $paymentId,
            'transaction_id' => $txId,
            'network' => $request->network,
            'status' => 1,
        ]);

        $callbackUrl = route('purchase.result', ['payment_id' => $paymentId]);
        redirect($callbackUrl);
    }

    public function checkUser($product_id)
    {
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'match' => [
                        '_id' => $product_id
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        return $response['hits']['hits']['0']['_source']['id']; // documents

    }

    public function buyProduct(Request $request)
    {
        $productId = $request->product_id;
        $user_id = $user_id = Token::query()->where('token', $request->header('token'))->first()->user_id;
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'bool' => [
                        "must" => [
                            [
                                "term" => ['_id' => "$productId"]
                            ],
                        ],
                    ]
                ]
            ]
        ];
        $response = $this->client->search($params);
        $price = $response['hits']['hits'][0]['_source']['object']['mains']['price'];
        $adv_id = $response['hits']['hits'][0]['_source']['id'];

//        if($onlinePurchase != true)
//        {
//            throw new CustomException('اين محصول قابليت سفارش آنلاين ندارد.',400,400);
//        }

        try {
            $invoice = new Invoice();
            $invoice->amount($price);
            $paymentId = md5(uniqid());
            $transaction = Transaction::create([
                'user_id' => $user_id,
                'product_id' => $productId,
                'promote_type' => 'buy_product',
                'paid' => $invoice->getAmount(),
                'invoice_details' => null,
                'payment_id' => $paymentId
            ]);
            $callbackUrl = route('purchase.result', ['payment_id' => $paymentId]);

            $payment = Payment::callbackUrl($callbackUrl);
            $payment->config('description', 'خريد محصول');
            $payment->purchase($invoice, function ($driver, $transactionId) use ($transaction) {
                $transaction['transaction_id'] = $transactionId;
                $transaction->save();
            });
            return $payment->pay();
        } catch (Exception|PurchaseFailedException|SoapFault $e) {

            $transaction['transaction_result'] = $e;
            $transaction->status = Transaction::STATUS_FAILED;
            $transaction->save();
        }
    }

    public function changeTransactionStatus(Request $request, $txid)
    {
        $validated = $request->validate([
            'status' => 'required|in:accepted,rejected,pending',
        ]);
        $transaction = Transaction::query()->where('transaction_id', $txid)->where('status', 1)->first();
        if ($request->status == 'accepted') {
            if ($transaction->promote_type == 'fori-tether') {
                $this->setFori($transaction->product_id);
            }
            if ($transaction->promote_type == 'nardeban-tether') {
                $this->setNardeban($transaction->product_id);
            }
            if ($transaction->promote_type == 'renewal-tether') {
                $this->setRenewal($transaction->product_id);
            }
            $transaction->status = 2;
            $transaction->save();
        }
        if ($request->status == 'rejected') {
            $transaction->status = 0;
        }
    }

    public function setFori($product_id)
    {
        $footer = KeyValueConfig::query()->where('key', 'footer')->first();

        $product = $this->client
            ->search(
                [
                    'index' => 'products',
                    'body' => [
                        'query' => [
                            "term" => ['_id' => $product_id]

                        ]
                    ]
                ]
            );

        $product = $product['hits']['hits'];
        $product = array_pop($product);

        throw_if(empty($product), NotFoundHttpException::class);


        if (
            isset($product['_source']['object']['fori_expires_at']) &&
            Carbon::make(
                $product['_source']['object']['fori_expires_at']
            )->greaterThanOrEqualTo(now())
        ) {
            throw new CustomException(
                trans('messages.plan_activated_before'), 400, 400
            );
        }


        $params = [
            'index' => 'products',
            'id' => $product_id,
            'body' => [
                'doc' => [
                    'object' => [
                        'fori_expires_at' => Carbon::now()->addDays($footer->value->fori_ad_period ?? 0),
                        'fori' => true
                    ]
                ],
            ]
        ];

        $this->client->update($params);

        return $product;
    }

    public function setNardeban($product_id)
    {
        $product = $this->client
            ->search(
                [
                    'index' => 'products',
                    'body' => [
                        'query' => [
                            "term" => ['_id' => $product_id]

                        ]
                    ]
                ]
            );

        $product = $product['hits']['hits'];
        $product = array_pop($product);

        throw_if(empty($product), NotFoundHttpException::class);


        if (
            isset($product['_source']['object']['renewal_at']) &&
            !Carbon::make(
                $product['_source']['object']['renewal_at']
            )->greaterThanOrEqualTo(now())
        ) {
            throw new CustomException(
                trans('messages.add_expired'), 400, 400
            );
        }


        $params = [
            'index' => 'products',
            'id' => $product_id,
            'body' => [
                'doc' => [
                    'object' => [
                        'updated_at' => Carbon::now(),
                    ]
                ],
            ]
        ];

        $this->client->update($params);


        return $product;
    }

    public function setRenewal($product_id)
    {
        $footer = KeyValueConfig::query()->where('key', 'footer')->first();
        $product = $this->client
            ->search(
                [
                    'index' => 'products',
                    'body' => [
                        'query' => [
                            "term" => ['_id' => $product_id]

                        ]
                    ]
                ]
            );

        $product = $product['hits']['hits'];
        $product = array_pop($product);

        throw_if(empty($product), NotFoundHttpException::class);


        if (
            isset($product['_source']['object']['renewal_at']) &&
            Carbon::make(
                $product['_source']['object']['renewal_at']
            )->greaterThanOrEqualTo(now())
        ) {
            throw new CustomException(
                trans('messages.add_expired'), 400, 400
            );
        }

        $params = [
            'index' => 'products',
            'id' => $product_id,
            'body' => [
                'doc' => [
                    'object' => [
                        'renewal_at' => Carbon::now()->addDays($footer->value->fori_ad_period ?? 0),
                    ]
                ],
            ]
        ];

        $this->client->update($params);


        return $product;


    }

    public function usdPayment(Request $request)
    {

        $userId = $request->id;
        $user = User::find($userId);
        $discountCode = null;

        $attributes = $request->validate(
            [
                'product_id' => ['required'],
                'promote_type' => ['required', Rule::in(['fori', 'nardeban', 'renewal'])],
                'discount_code' => [
                    'nullable',
                    function (string $attribute, mixed $value, Closure $fail) use ($userId, &$discountCode) {
                        $discountCode = DiscountCode::query()->where('code', $value)
                            ->first();

                        if (!isset($discountCode)) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }
                        if (Str::upper($discountCode->type) !== Str::upper(\request('promote_type'))) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }

                        $discountCodeUser = $discountCode->discountCodeUsers()
                            ->where('user_id', $userId)
                            ->first();

                        if (!$discountCodeUser) {
                            $fail(trans('messages.wrong_user_code'));
                            return false;
                        }

                        if ($discountCodeUser?->is_used) {
                            $fail(trans('messages.used_code'));
                            return false;
                        }

                        if (Carbon::make($discountCode->expires_at)->timestamp < now()->timestamp) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }
                    },
                ]
            ]
        );

        $price = 0;
        $footer = KeyValueConfig::query()->where('key', 'footer')->first();


        $price = match ($attributes['promote_type']) {
            "fori" => $footer->value->fori_price,
            "renewal" => $footer->value->tamdid_price,
            "nardeban" => $footer->value->nardeban_price,
        };

        $price = $this->calculatePriceAfterDiscount(
            $discountCode,
            $price
        );


//        Stripe\Stripe::setApiKey(env('STRIPE_SECRET'));
        $payment = new \Stripe\StripeClient(env('STRIPE_SECRET'));
        return $payment->checkout->sessions->create([
            'success_url' => 'https://example.com/success',
            'line_items' => [
                [
                    'price_data' => [
                        'currency' => 'usd',
                        'unit_amount' => 100,
                        'product_data' => [
                            'name' => 'mohammad',
                            'description' => 'test'
                        ]
                    ],
                    'quantity' => 1,
                ],
            ],
            'mode' => 'payment',
        ]);
    }

    protected function calculatePriceAfterDiscount($price, $discountCode)
    {
        $reducedAmount = $price * $discountCode->percent / 100;
        if ($reducedAmount > $discountCode->max_price) {
            $reducedAmount = $discountCode->max_price;
        }
        return $price - $reducedAmount;
    }

    public function payViaWallet(Request $request)
    {
        $userId = $request->id;
        $discountCode = null;

        $attributes = $request->validate(
            [
                'product_id' => ['required'],
                'promote_type' => ['required', Rule::in(['fori', 'nardeban', 'renewal'])],
                'discount_code' => [
                    'nullable',
                    function (string $attribute, mixed $value, Closure $fail) use ($userId, &$discountCode) {
                        $discountCode = DiscountCode::query()->where('code', $value)
                            ->first();

                        if (!isset($discountCode)) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }
                        if (Str::upper($discountCode->type) !== Str::upper(\request('promote_type'))) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }

                        $discountCodeUser = $discountCode->discountCodeUsers()
                            ->where('user_id', $userId)
                            ->first();

                        if (!$discountCodeUser) {
                            $fail(trans('messages.wrong_user_code'));
                            return false;
                        }

                        if ($discountCodeUser?->is_used) {
                            $fail(trans('messages.used_code'));
                            return false;
                        }

                        if (Carbon::make($discountCode->expires_at)->timestamp < now()->timestamp) {
                            $fail(trans('messages.discount_fail'));
                            return false;
                        }
                    },
                ]
            ]
        );


        $price = 0;
        $footer = KeyValueConfig::query()->where('key', 'footer')->first();


        $price = match ($attributes['promote_type']) {
            "fori" => $footer->value->fori_price,
            "renewal" => $footer->value->tamdid_price,
            "nardeban" => $footer->value->nardeban_price,
        };

        if (isset($attributes['discount_code']) && $discountCode) {
            $reducedAmount = $price * $discountCode->percent / 100;
            if ($reducedAmount > $discountCode->max_price) {
                $reducedAmount = $discountCode->max_price;
            }
            $price = $price - $reducedAmount;
        }

        $user = User::find($userId);

        if ($user->wallet_amount < $price) {
//                throw new CustomException(trans('messages.discount_fail'), 1011, 400);
            throw new CustomException("wallet not enough", 1041, 400);
        }

        $transaction = DB::transaction(
            function () use ($user, $price, $attributes, $discountCode) {
                $user->wallet_amount = $user->wallet_amount - $price;
                $user->save();

                $paymentId = md5(uniqid());
                return Transaction::create([
                    'user_id' => $user->id,
                    'product_id' => $attributes['product_id'],
                    'promote_type' => $attributes['promote_type'],
                    'paid' => $price,
                    'invoice_details' => null,
                    'payment_id' => $paymentId,
                    'discount_id' => $discountCode->id ?? null,
                    'status' => 1
                ]);

            }
        );

        $callbackUrl = route('purchase.result', ['payment_id' => $transaction->payment_id]);
        redirect($callbackUrl);
    }

    public function adminUpgradeAds(Request $request)
    {
        $attributes = $request->validate(
            [
                'product_id' => ['required'],
                'promote_type' => ['required', Rule::in(['fori', 'nardeban', 'renewal'])],

            ]
        );

        $this->{'set' . $attributes['promote_type']}($attributes['product_id']);

        $product = $this->client
            ->search(
                [
                    'index' => 'products',
                    'body' => [
                        'query' => [
                            "term" => ['_id' => $attributes['product_id']]

                        ]
                    ]
                ]
            );

        $product = $product['hits']['hits'];
        $product = array_pop($product);


        return response()->json(
            [
                'success' => true,
                'message' => trans('messages.success'),
                'data' => $product
            ]
        );


    }

    private function TRC20($txId, $address, $price)
    {
        $httpClient = new Client();

        $checkUrl = 'https://apilist.tronscanapi.com/api/transaction-info';
        $response = $httpClient->get($checkUrl, [
            'query' => [
                'hash' => $txId,
            ],
        ]);

        $trx = json_decode($response->getBody(), true);
        $isValid = true;
//        $isContractCorrect = strtolower($trx['tokenTransferInfo']['contract_address']) === strtolower($transaction->gateway->identity);
        $isAddressCorrect = strtolower($trx['tokenTransferInfo']['to_address']) === strtolower($address);
        $isAmountCorrect = (floatval($trx['tokenTransferInfo']['amount_str']) >= floatval($price));

        return [
            'isValid' => $isValid,
//            'isContractCorrect' => $isContractCorrect,
            'isAmountCorrect' => $isAmountCorrect,
            'isAddressCorrect' => $isAddressCorrect,
            'trx' => $trx,
        ];
    }
}
