<?php

namespace App\Http\Controllers;

use App\Models\Page;
use Closure;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class PageController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return JsonResponse
     */
    public function index()
    {
        return response()->json([
            'success' => true,
            'message' => 'Pages : ',
            'data' => Page::filter(['name', 'language'])
                ->get(),
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param Request $request
     * @return JsonResponse
     */
    public function store(Request $request)
    {
        $attribute = $request->validate([
            'name' => ['required'],
            'language' => ['required'],
            'title' => ['sometimes', 'max:255'],
            'content' => 'required'
        ]);

        $page = Page::query()->updateOrCreate(
            [
                'name' => $attribute['name'],
                'language' => $attribute['language']
            ],
            $attribute
        );

        return response()->json([
            'success' => true,
            'message' => 'Page : ',
            'data' => $page
        ]);
    }

    /**
     * Display the specified resource.
     *
     * @param Page $page
     * @return JsonResponse
     */
    public function show($id)
    {
        $page = (is_numeric($id)) ?
            Page::findOrFail($id) :
            Page::where('name', $id)
                ->where('language', request('language', 'fa'))
                ->firstOrFail();


        return response()->json([
            'success' => true,
            'message' => 'Page : ',
            'data' => $page
        ]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param Request $request
     * @param Page $page
     * @return JsonResponse
     */
    public function update(Page $page, Request $request)
    {
        $attribute = $request->validate([
            'name' => ['required', function (string $attribute, mixed $value, Closure $fail) {
                if (Page::query()
                    ->where('name', $value)
                    ->where('language', \request('language', 'fa'))
                    ->exists()) {
                    $fail(trans('validation.unique', ['attribute' => $attribute]));
                }
            }],
            'title' => ['sometimes', 'max:255'],
            'content' => 'required',
            'language' => 'sometimes'
        ]);
        $page->where('language', request('language', 'fa'))
            ->firstOrFail()->update($attribute);
        return response()->json([
            'success' => true,
            'message' => 'Page : ',
            'data' => $page
        ]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param Page $page
     * @return JsonResponse
     */
    public function destroy(Page $page)
    {
        return response()->json([
            'success' => true,
            'message' => 'Page : ',
            'data' => $page->delete(),
            \Symfony\Component\HttpFoundation\Response::HTTP_NO_CONTENT
        ]);
    }
}
