<?php

namespace App\Http\Controllers;

use App\Actions\SeoAction;
use App\Http\Requests\SeoRequest;
use App\Models\Seo;
use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;

class SeoController extends Controller
{
    public function store (Request $request): JsonResponse
    {
        $exists = Seo::query();

        foreach (['city', 'country', 'language', 'pageable_type', 'pageable_id'] as $item ) {
            $exists->when(
                isset($request->$item),
            fn($query) => $query->where($item, $request->$item),
            fn($query) => $query->whereNull($item),
            );
        }

        if ($exists->exists()) {
            throw new CustomException("exist before");
        }

        return response()->json([
            'success' => true,
            'message' => 'Store : ',
            'data' => (new SeoAction())
                ->setRequest($request)
                ->setValidationRule('store')
                ->storeByRequest()
        ]);
    }

    public function updateById (string $id, Request $request): JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Update : ',
            'data' => (new SeoAction())
                ->setRequest($request)
                ->setValidationRule('update')
                ->updateByIdAndRequest($id)
        ]);
    }

    public function get (Request $request) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'List : ',
            'data' => (new SeoAction())
                ->setRequest($request)
                ->setValidationRule('get_query')
                ->makeEloquentViaRequest()
                ->getByRequestAndEloquent()
        ]);
    }

    public function getById (string $id) : JsonResponse
    {
        return response()->json([
            'success' => true,
            'message' => 'Single Seo : ',
            'data' => (new SeoAction())
                ->makeEloquent()
                ->getById($id)
        ]);
    }

    public function deleteById (string $id): JsonResponse
    {
        (new SeoAction())->deleteById($id);

        return response()->json([
            'message' => 'deleted'
        ]);
    }

    public function single(Request $request)
    {
        $attributes = $request->validate(
            [
                'pageable_type' => ['required'],
                'pageable_id' => [Rule::requiredIf(fn()=> in_array($request->pageable_type, ['CATEGORY', 'SHOP', 'PRODUCT']) )],
                'country' => [Rule::requiredIf(fn() => !$request->has('city'))],
                'city' => [Rule::requiredIf(fn() => !$request->has('country'))],
                'language' => ['required']
            ]
        );

        var_dump($attributes);

        return Seo::query()
            ->where('pageable_type', $attributes['pageable_type'])
            ->get();
    }
}
