<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Models\User;
use App\Models\Postchi;
use App\Models\Support;
use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Kavenegar;

class SupportController extends Controller
{
    public function show(Request $request)
    {
        $arr = [];
        $size = $request->limit ?? 10;
        $page = (int)($request->skip && $request->limit ? $request->skip / $request->limit : $request->page ?? 0) + 1;
        if ($request->subject) {
            $total = Support::where('subject', $request->subject)->get();
            $tickets = Support::query()->orderBy('updated_at', 'desc')->where('subject', $request->subject);
            if ($request->email) {
                $tickets = $tickets->where('email', $request->email);
            }
            if ($request->ads_phone) {
                $tickets = $tickets->where('ads_phone', $request->ads_phone);
            }
            if ($request->type) {
                $tickets = $tickets->where('type', $request->type);
            }
            $tickets = $tickets->paginate($size, ['*'], 'page', $page);
            foreach ($tickets as $ticket) {
                $user = User::where('id', $ticket->user_id)->first();
                $collection = collect($user);
                $per_user = $collection->merge($ticket);
                array_push($arr, $per_user);
            }
        } else {
            $total = Support::count();
            $tickets = Support::query()->orderBy('updated_at', 'desc');
            if ($request->email) {
                $tickets = $tickets->where('email', $request->email);
            }
            if ($request->ads_phone) {
                $tickets = $tickets->where('ads_phone', $request->ads_phone);
            }
            if ($request->type) {
                $tickets = $tickets->where('type', $request->type);
            }
            $tickets = $tickets->paginate($size, ['*'], 'page', $page);
            foreach ($tickets as $ticket) {
                $user = User::where('id', $ticket->user_id)->first();
                $collection = collect($user);
                $per_user = $collection->merge($ticket);
                array_push($arr, $per_user);
            }
        }

        return [
            'meta' => [
                'perpage' => $size,
                'currentpage' => $page,
                'totalpage' => ceil(count($tickets) / $size),
                'totalrows' => $total,
            ],
            'hits' => $arr,
        ];
    }


    public function store(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'subject' => 'required',
            'email' => 'required|email',
            'title' => 'required',
            'message' => 'required',
            'ads_phone' => 'nullable|regex:/[0-9]{9}/|size:11',
            'ads_title' => 'nullable',
            'photos' => 'nullable',
            'type' => 'string|required',
            'rate' => 'nullable'
        ]);
        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() as $key => $value) {
                if (str_contains($value[0], 'required')) {
                    throw new CustomException(trans('messages.' . $key), 422, 422);
                } else {
                    throw new CustomException($key . " " . $value[0], 422, 422);

                }
            }
        } else {
            $support = new Support();
            $support->fill($request->all());
            $support['user_id'] = $request->user_id ?? null;
            $support->id = null;
            $support['photos'] = json_encode($support['photos']);
            $support->save();
            $subject = $support['subject'];
            $title = $support['ads_title'];
            if ($support['type'] === 'report') {
                $message = "کاربری با موضوع $subject برای آگهی $title تخلف ثبت کرد";
                Kavenegar::Send(env('SMS_FROM_NUMBER'), env('ADMIN_NUMBER'), $message);
            } elseif ($support['type'] === 'suggestion') {
                //TODO::if needed
            }
            return $support;
        }
    }


    public function Postchi(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'admin_id' => 'nullable',
            'user_id' => 'required',
            'text' => 'nullable',
            'store_file' => 'nullable'
        ]);

        if (count($validate->errors()) > 0) {
            foreach ($validate->errors()->messages() as $key => $value) {
                if (str_contains($value[0], 'required')) {
                    throw new CustomException(trans('messages.' . $key), 422, 422);
                } else {
                    throw new CustomException($key . " " . $value[0], 422, 422);

                }
            }
        } else {
            $postchi = new Postchi();
            $postchi->fill([
                'admin_id' => $request->id,
                'user_id' => $request->user_id,
            ]);

            if ($request->text) {
                $postchi->fill([
                    'text' => $request->text,
                    'store_file' => null,
                ]);
            } elseif ($request->store_file) {
                $postchi->store_file = 'https://8.vistadev.ir/divar/core/storage/app/' . $request->file('store_file')->store('store_file');
                $postchi->fill([
                    'store_file' => $postchi->store_file,
                    'text' => "null"
                ]);
            }
        }
        $postchi->save();

        $startDate = Carbon::parse($postchi['created_at']);
        $endDate = Carbon::now();
        $diffInDays = $startDate->diffInDays($endDate);
        $postchi['creation_date'] = $diffInDays;


        return $postchi;
    }


    public function showPostchi(Request $request)
    {
        $posts = Postchi::where('user_id', $request->id)->get();
        $full_posts = [];
        foreach ($posts as $post) {
            //the date of creation
            $startDate = Carbon::parse($post['created_at']);
            $endDate = Carbon::now();
            $diffInDays = $startDate->diffInDays($endDate);
            $post['creation_date'] = $diffInDays;
            array_push($full_posts, $post);
        }

        return $full_posts;
    }

    public function showUserPostchis(Request $request)
    {
        $posts = Postchi::where('user_id', $request->user_id)->get();
        return $posts;
    }

    public function showLastPostchi(Request $request)
    {
        $posts = Postchi::where('user_id', $request->id)->orderBy('created_at', 'desc')->first();
        if ($posts) {
            if ($posts->text != " ") {
                return [
                    "message" => $posts->text,
                    "created_at" => $posts->created_at
                ];
            } elseif ($posts->store_file) {
                return [
                    "store_file" => $posts->store_file,
                    "created_at" => $posts->created_at
                ];
            } else {
                return null;
            }
        } else {
            return null;
        }

    }

}
