<?php

namespace App\Http\Resources;

use App\Models\Chat;
use App\Models\User;
use Elastic\Elasticsearch\ClientBuilder;
use Illuminate\Http\Request;
use Illuminate\Http\Resources\Json\JsonResource;

class ChatResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array|\Illuminate\Contracts\Support\Arrayable|\JsonSerializable
     */
    public function toArray($request)
    {
        $chat = Chat::query()->where('id', $this->chat_id)->first();

        $client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'),env('ELASTIC_PASSWORD'))
            ->build();
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'match' => [
                        '_id' => $chat?->product_id
                    ]
                ]
            ]
        ];
        $response = $client->search($params);
        if ($response['hits']['hits']) {
            $id = $response['hits']['hits'][0]['_source']['id']; // documents
            $title = $response['hits']['hits'][0]['_source']['object']['mains']['title']; // documents
        } else {
            abort(404, 'ad not found');
        }
        $user = User::query()->where('id', $this->user_id)->first();
        $advertiser = User::query()->where('id', $id)->first();

        return [
            'id' => $this->id,
            'user' => "$user?->name" . " " . "$user?->lastname",
            'user_id' => "$user?->id",
            'advertiser' => "$advertiser?->name" . " " . "$advertiser?->lastname",
            'sender' => $this->sender,
            'text' => $this->text,
            'product_title' => $title,
            'store_file' => $this->store_file,
            'created_at' => $this->created_at
        ];
    }
}
