<?php

namespace App\Http\Resources;

use App\Models\Transaction;
use App\Models\User;
use Elastic\Elasticsearch\ClientBuilder;
use Illuminate\Http\Resources\Json\JsonResource;

class OrderResource extends JsonResource
{
    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array|\Illuminate\Contracts\Support\Arrayable|\JsonSerializable
     */
    public function toArray($request)
    {
        $client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'),env('ELASTIC_PASSWORD'))
            ->build();
        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'bool' => [
                        "must" => [
                            [
                                "term" => ['_id' => "$this->product_id"]
                            ],
                        ],
                    ]
                ]
            ]
        ];
        $response = $client->search($params);
        $price = $response['hits']['hits'][0]['_source']['object']['mains']['price'] ?? null;
        $p_title = $response['hits']['hits'][0]['_source']['object']['mains']['title']?? null;
        $p_city_id = $response['hits']['hits'][0]['_source']['object']['mains']['city_id']?? null;
        $p_images = $response['hits']['hits'][0]['_source']['object']['mains']['images']?? null;
        $p_description = $response['hits']['hits'][0]['_source']['object']['mains']['description']?? null;
        $p_category = $response['hits']['hits'][0]['_source']['object']['mains']['category']?? null;
        $p_subcategory = $response['hits']['hits'][0]['_source']['object']['mains']['subCategory']?? null;
        $p_innercategory = $response['hits']['hits'][0]['_source']['object']['mains']['innerCategory']?? null;
        $p_created_at = $response['hits']['hits'][0]['_source']['object']['created_at']?? null;
        $p_slug = $response['hits']['hits'][0]['_source']['object']['slug']?? null;
        $adv_id = $response['hits']['hits'][0]['_source']['id']?? null;
        $advertiser = User::query()->where('id', $adv_id)->first();
        $transaction = Transaction::query()->where('id', $this->transaction_id)->first();
        return [
            'id' => $this->id,
            'user_id' => $this->user_id,
            'advertiser_name' => "$advertiser?->name"." "."$advertiser?->lastname" ?? null,
            'city_id' => $p_city_id,
            'images' => $p_images,
            'description' => $p_description,
            'product_title' => $p_title,
            'category' => $p_category,
            'subCategory' => $p_subcategory,
            'innerCategory' => $p_innercategory,
            'slug' => $p_slug,
            'price' => $this->price,
            'has_reached_destination' => $this->has_reached_destination,
            'product_created_at' => $p_created_at,
            'transaction' => $transaction,
            'created_at' => $this->created_at
        ];
    }
}
