<?php

namespace App\Http\Resources;

use App\Models\User;
use Elastic\Elasticsearch\ClientBuilder;
use Illuminate\Http\Resources\Json\JsonResource;
use App\Traits\ElasticFunctions;


class TransactionResource extends JsonResource
{

    /**
     * Transform the resource into an array.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return array
     */
    public function toArray($request)
    {
        $client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'),env('ELASTIC_PASSWORD'))
            ->build();
        $user = User::query()->where('id', $this->user_id)->first();

        $params = [
            'index' => 'products',
            'body' => [
                'query' => [
                    'bool' => [
                        "must" => [
                            [
                                "term" => ['_id' => "$this->product_id"]
                            ],
                        ],
                    ]
                ]
            ]
        ];
        $response = $client->search($params);
        if ($response['hits']['hits']) {
            $res = $response['hits']['hits'][0]['_source']['object']['mains']['title'];
        }
        if($this->promote_type == 'fori-tether' || $this->promote_type == 'nardeban-tether' || $this->promote_type == 'renewal-tether')
        {
            switch ($this->network)
            {
                case 'ERC20' : $scanLink = "https://etherscan.io/tx/";break;
                case 'ERC20s' : $scanLink = "https://etherscan.io/tx/";break;
                case 'TRC20' : $scanLink = "https://tronscan.org/#/transaction/";break;
                case 'TRC20s' : $scanLink = "https://tronscan.org/#/transaction/";break;
            }
            return [
                'id' => $this->id,
                'first_name' => $user?->name ?? null,
                'last_name' => $user?->lastname ?? null,
                'price' => $this->paid,
                'status' => $this->status,
                'network' => $this->network,
                'promote_type' => $this->promote_type,
                'payment_id' => $this->payment_id,
                'transaction_id' => $scanLink.$this->transaction_id ?? null,
                'created_at' => $this->created_at
            ];
        }
        return [
            'id' => $this->id,
            'first_name' => $user?->name ?? null,
            'last_name' => $user?->lastname ?? null,
            'product' => $res ?? null,
            'price' => $this->paid,
            'status' => $this->status,
            'promote_type' => $this->promote_type,
            'invoice_details' => $this->invoice_details,
            'payment_id' => $this->payment_id,
            'transaction_id' => $this->transaction_id,
            'transaction_result' => $this->transaction_result,
            'created_at' => $this->created_at
        ];
    }
}
