<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Foundation\Auth\User as Authenticatable;

class Admin extends Authenticatable
{
    use HasFactory;
    protected $guarded = [];

    public function role(): BelongsTo
    {
        return $this->belongsTo(Role::class, 'role_id', 'id');
    }

    public function hasPermission(Permission $permission)
    {
        $role = $this->role()->first();
        if (!$role) {
            return false;
        }
        $role->load(['permissions']);
        return $role->permissions->contains('key', $permission->key);
    }

    public function revokeAllTokens()
    {
        $this->load(['tokens' => fn($q) => $q->where('name', 'admin auth token')->where('revoked', false)]);

        foreach ($this->tokens as $token) {
            $token->revoke();
        }

        $this->unsetRelation('tokens');
    }
}
