<?php
namespace App\Services;

use Genocide\Radiocrud\Exceptions\CustomException;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Str;
use Web3\RequestManagers\HttpRequestManager;
use Web3\Web3;
use Web3\Utils;
use Web3\Providers\HttpProvider;


class CheckCryptoTransactions
{
    public static function TRC20($txId,$address,$price)
    {
        $httpClient = new Client();

        $checkUrl = 'https://apilist.tronscanapi.com/api/transaction-info';
        $response = $httpClient->get($checkUrl, [
            'query' => [
                'hash' => $txId,
            ],
        ]);

        $trx = json_decode($response->getBody(), true);
        $isValid = true;

        try {
            $isContractCorrect = strtolower($trx['tokenTransferInfo']['contract_address']) === strtolower('TR7NHqjeKQxGTCi8q8ZY4pL8otSzgjLj6t');
            $isAddressCorrect = strtolower($trx['tokenTransferInfo']['to_address']) === strtolower($address);
            $isAmountCorrect = (floatval($trx['tokenTransferInfo']['amount_str']) >= floatval($price*1000000));

            return [
                'isValid' => $isValid,
                'isContractCorrect' => $isContractCorrect,
                'isAmountCorrect' => $isAmountCorrect,
                'isAddressCorrect' => $isAddressCorrect,
                'trx' => $trx,
            ];
        }
        catch (\Exception $exception)
        {
            throw new CustomException('transaction not found',400,400);
        }
    }

    public static function ERC20($txId, $address, $price)
    {
        $apiKey = 'API_KEY';
        $etherscanUrl = 'https://api.etherscan.io/api';

        $httpProvider = new HttpProvider(new HttpRequestManager($etherscanUrl));
        $web3 = new Web3($httpProvider);

        $trx = $web3->eth->getTransactionByHash($txId);
        $status = $web3->eth->getTransactionReceiptStatus($txId);

        try {
            $details = $web3->abi->decodeMethod($trx['input']);
            $isValid = $status === '1' && $details['name'] === 'transfer';
            $isContractCorrect = strtolower($trx['to']) === strtolower('');
            $isAddressCorrect = strtolower($details['params'][0]['value']) === strtolower($address);
            $isAmountCorrect = (floatval($details['params'][0]['value']) >= floatval($price*1000000));

            return [
                'isValid' => $isValid,
                'isContractCorrect' => $isContractCorrect,
                'isAddressCorrect' => $isAddressCorrect,
                'isAmountCorrect' => $isAmountCorrect,
                'trx' => $trx,
            ];
        }
        catch (\Exception $exception)
        {
            throw new CustomException('transaction not found',400,400);
        }

    }


}
