<?php


namespace App\Services\CryptoNetworks;


use Genocide\Radiocrud\Exceptions\CustomException;
use Illuminate\Support\Facades\Http;

class ERC20 extends CryptoNetwork
{

    const URL = "https://api.etherscan.io/api";
    const TOKEN_ADDRESS = "0xdAC17F958D2ee523a2206206994597C13D831ec7";
    const DECIMALS = 6;

    protected function getTokenInfo(string $txId): array
    {

        $response = Http::get(static::URL,
            [
                "module" => "proxy",
                "action" => "eth_getTransactionByHash",
                "txhash" => $txId,
                "apikey" => "YourApiKeyToken"
            ]
        );

        if (!$response->successful()) {
            throw new CustomException('wrong request');
        }

        $data = $response->json('result');


        if (!$data) {
            throw new CustomException('wrong request');
        }

        return $this->tokenInfoResponseMapper($data);

    }

    protected function tokenInfoResponseMapper($info): array
    {

        $contractABI = '[{"constant":true,"inputs":[],"name":"name","outputs":[{"name":"","type":"string"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"name":"_spender","type":"address"},{"name":"_value","type":"uint256"}],"name":"approve","outputs":[{"name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[],"name":"totalSupply","outputs":[{"name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"name":"_from","type":"address"},{"name":"_to","type":"address"},{"name":"_value","type":"uint256"}],"name":"transferFrom","outputs":[{"name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[],"name":"decimals","outputs":[{"name":"","type":"uint8"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[{"name":"_owner","type":"address"}],"name":"balanceOf","outputs":[{"name":"balance","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":true,"inputs":[],"name":"symbol","outputs":[{"name":"","type":"string"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"name":"_to","type":"address"},{"name":"_value","type":"uint256"}],"name":"transfer","outputs":[{"name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[{"name":"_owner","type":"address"},{"name":"_spender","type":"address"}],"name":"allowance","outputs":[{"name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"payable":true,"stateMutability":"payable","type":"fallback"},{"anonymous":false,"inputs":[{"indexed":true,"name":"owner","type":"address"},{"indexed":true,"name":"spender","type":"address"},{"indexed":false,"name":"value","type":"uint256"}],"name":"Approval","type":"event"},{"anonymous":false,"inputs":[{"indexed":true,"name":"from","type":"address"},{"indexed":true,"name":"to","type":"address"},{"indexed":false,"name":"value","type":"uint256"}],"name":"Transfer","type":"event"}]';

        $decodedData = $this->decodeMethodInput(
            $contractABI,
            $info['input']
        );


        return [
            'method' => $decodedData['name'],
            'contract_address' => $info['to'] ?? '',
            'to_address' => '0x' . ltrim($decodedData['params']['_to'], '\0') ?? '',
            'amount_str' => (string)$decodedData['params']['_value'] ?? '',
            'from_address' => $info['from'] ?? '',
        ];
    }

    private function decodeMethodInput(string $contractABI, string $inputData): array
    {
        $method = substr($inputData, 0, 10);

        if ($method !== '0xa9059cbb') {
            throw new \Exception('wrong tx id.');
        }

        $methodData = substr($inputData, 10);

        // Find the method in the ABI
        $abi = json_decode($contractABI, true);
        $method = null;
        foreach ($abi as $item) {
            if (isset($item['type']) && $item['type'] === 'function') {
                if ($item['name'] === 'transfer') {
                    $method = $item;
                    break;
                }
            }
        }

        if (!$method) {
            throw new CustomException(
                "Method not found in ABI.",
                400,
                400
            );
        }

        $params = [];

        foreach ($method['inputs'] as $index => $param) {
            $paramData = substr($methodData, 64 * $index, 64);
            $paramValue = hexdec($paramData);
            if ($param['type'] === 'uint256') {
                $params[$param['name']] = $paramValue;
            } else {
                // Add handling for other types if necessary
                $params[$param['name']] = $paramData;
            }
        }

        return [
            'name' => $method['name'],
            'params' => $params,
        ];
    }


}
