<?php


namespace App\Services\Sitemap;


use Carbon\Carbon;
use Illuminate\Support\Str;

use Spatie\Sitemap\Sitemap;
use Spatie\Sitemap\SitemapIndex;
use Spatie\Sitemap\Tags\Url;

class ProductSitemap extends SitemapFactory
{

    public array $categories;

    public function __construct(SitemapIndex $sitemapIndex)
    {
        parent::__construct($sitemapIndex);
    }

    public function generate()
    {
        $size = 10000;
        do {
            $this->initiateSitemap();

            $products = $this->client
                ->search(
                    [
                        'index' => 'products',
                        'body' => [
                            'size' => $size,
                            'from' => ($this->page - 1) * $size,
                            'query' => [
                                "bool" =>
                                    [
                                        'must_not' => [
                                            'match' => ['object.status' => 'rejected'],
                                        ],

                                    ]
                            ]
                        ]
                    ]
                );

            $products = $products['hits']['hits'];


            foreach ($products as $product) {

                $id = $product['_id'];
                $object = $product['_source']['object'];

                $this->addCategoriesBasedOnCity($object['mains']);
                if (isset($object['mains']['city_id'])) {
                }
                $this->sitemap
                    ->add(
                        Url::create(
                            'products/' . $object['slug'] . '/' . $id)
                            ->setPriority(
                                Carbon::make($object['renewal_at'])->timestamp > now()->addDays(1)->timestamp ? 0.5 : 0.4
                            )->setChangeFrequency(Url::CHANGE_FREQUENCY_MONTHLY)
                    );
            }


            $this->saveIn('product-sitemap');

        } while (!empty($products));

        $this->sanitizeCategories();
    }

    private function addCategoriesBasedOnCity($mains): void
    {
        $cityCategory = $this->categories[$mains['city_id']] ?? [];

        if (isset($mains['innerCategory'])) {
            $cityCategory[] = $mains['innerCategory'];
        }
        $cityCategory[] = $mains['subCategory'];
        $cityCategory[] = $mains['category'];

        $cityCategory = array_unique($cityCategory);

        $this->categories[(int)$mains['city_id']] = $cityCategory;
    }

    private function sanitizeCategories()
    {

    }

}
