<?php


namespace App\Services\Sitemap;


use Elastic\Elasticsearch\ClientBuilder;
use Illuminate\Support\Str;
use Spatie\Sitemap\Sitemap;
use Spatie\Sitemap\SitemapIndex;

class SitemapFactory
{

    public static string $directory = "sitemap";
    protected \Elastic\Elasticsearch\Client $client;
    protected int $page = 1;

    /**
     * @var Sitemap
     */
    protected Sitemap $sitemap;

    public function __construct(
        protected SitemapIndex $sitemapIndex
    )
    {

        $this->client = ClientBuilder::create()
            ->setHosts([env('ELASTIC_HOST')])
            ->setBasicAuthentication(env('ELASTIC_USERNAME'), env('ELASTIC_PASSWORD'))
            ->build();

    }

    protected function initiateSitemap()
    {
        $this->sitemap = Sitemap::create();
    }

    protected function saveIn($filename, $makeUnique = true) : void
    {
        $path = static::$directory;
        if (Str::endsWith(static::$directory, '/')) {
            $path .= '/';
        }

        if ($makeUnique) {
            $filename = Str::uuid() . '-' . $filename;
        }

        $path .= $filename;

        $this->sitemap
            ->writeToFile(
                $path
            );

        $this->sitemapIndex
            ->add(
                $path
            );
    }

}
