<?php

use App\Http\Controllers\AuthController;
use App\Http\Controllers\Auth\UserController;
use App\Http\Controllers\ContactUsContentController;
use App\Http\Controllers\Dashboard\BusinessController;
use App\Http\Controllers\Ads\CategoryController;
use App\Http\Controllers\Dashboard\ProfileController;
use App\Http\Controllers\DiscountCodeController;
use App\Http\Controllers\MainController;
use App\Http\Controllers\Ads\ProductController;
use App\Http\Controllers\PageController;
use App\Http\Controllers\SeoController;
use App\Http\Controllers\CountryController;
use App\Http\Controllers\TransactionController;
use App\Http\Controllers\CityAndProvinceController;
use App\Http\Controllers\HomeController;
use App\Http\Controllers\ChatController;
use App\Http\Controllers\OrderController;
use App\Http\Controllers\Admin\RoleController;
use App\Http\Controllers\Admin\PermissionController;
use App\Http\Controllers\WithdrawRequestController;
use App\Http\Controllers\Ads\FormController;
use App\Http\Controllers\SupportController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;

Route::any('payment_check', function (Request $request) {
    return (new App\Http\Controllers\Dashboard\ProfileController)->verifyUSDpayment($request);
});

Route::group([
    'middleware' => ['localization']
], function () {
    Route::get('/admin/user', [UserController::class, 'get'])->middleware(['localization', 'adminauth']);
    Route::get('/admin/user/{id}', [UserController::class, 'getById'])->middleware('adminauth');
    Route::delete('/admin/user/{id}', [UserController::class, 'deleteById'])->middleware('adminauth');

    Route::get('/admin/business', [BusinessController::class, 'get'])->middleware('adminauth');
    Route::put('/admin/business', [BusinessController::class, 'changeStatus'])->middleware('adminauth');
    Route::put('/admin/business/special', [BusinessController::class, 'setSpecial'])->middleware('adminauth');
    Route::get('/admin/business/{id}', [BusinessController::class, 'getById'])->middleware('adminauth');
    Route::delete('/admin/business/{id}', [BusinessController::class, 'deleteById'])->middleware('adminauth');

    Route::get('/admin/transaction/excel', [TransactionController::class, 'transactionToExcel']);
    Route::get('/admin/transaction', [TransactionController::class, 'get']);
    Route::get('/admin/transaction/tether', [TransactionController::class, 'getTethetPending']);
    Route::get('/admin/transaction/{id}', [TransactionController::class, 'getById'])->middleware('adminauth');
    Route::patch('/admin/transaction/{txid}/status', [ProfileController::class, 'changeTransactionStatus'])->middleware('adminauth');

    Route::get('/admin/contact_us', [ContactUsContentController::class, 'readMessage'])->middleware('adminauth');
    Route::get('/admin/contact_us/count', [ContactUsContentController::class, 'count'])->middleware('adminauth');

    Route::post('/admin/discount_code', [DiscountCodeController::class, 'store'])->middleware('adminauth');
    Route::get('/admin/discount_code', [DiscountCodeController::class, 'get'])->middleware('adminauth');
    Route::put('/admin/discount_code/{id}', [DiscountCodeController::class, 'updateById'])->middleware('adminauth');
    Route::get('/admin/discount_code/{id}', [DiscountCodeController::class, 'getById'])->middleware('adminauth');
    Route::get('/admin/discount_code/code/{code}', [DiscountCodeController::class, 'getByCode']);
    Route::delete('/admin/discount_code/{id}', [DiscountCodeController::class, 'delete'])->middleware('adminauth');

    Route::get('/admin/reports', [ChatController::class, 'getReports'])->middleware('adminauth');
    Route::get('/admin/chats', [ChatController::class, 'allChats'])->middleware('adminauth');

    Route::get('/admin/withdraw-requests', [WithdrawRequestController::class, 'get'])->middleware('adminauth');
    Route::post('/admin/withdraw-requests/{id}', [WithdrawRequestController::class, 'changeStatus'])->middleware('adminauth');


    Route::get('/user/credit_card', [UserController::class, 'getCreditCard'])->middleware('auth');
    Route::post('/user/credit_card', [UserController::class, 'creditCardUpdate'])->middleware('auth');

    Route::post('/user/withdraw-request', [WithdrawRequestController::class, 'newRequest'])->middleware('auth');
    Route::get('/user/withdraw-request', [WithdrawRequestController::class, 'userWithdrawHistory'])->middleware('auth');

    Route::post('/user/buy', [ProfileController::class, 'buyProduct'])->middleware('auth');
    Route::post('/user/purchase-confirmation', [OrderController::class, 'confirm'])->middleware('auth');
    Route::get('/user/order', [OrderController::class, 'get'])->middleware('auth');

    Route::get('/admin/permission', [PermissionController::class, 'index'])->middleware('adminauth');
    Route::get('/admin/roles', [RoleController::class, 'index'])->middleware('adminauth');
    Route::post('/admin/role', [RoleController::class, 'store'])->middleware('adminauth');
    Route::get('/admin/role/{role}', [RoleController::class, 'show'])->middleware('adminauth');
    Route::post('/admin/role/{role}', [RoleController::class, 'update'])->middleware('adminauth');
    Route::delete('/admin/role/{role}', [RoleController::class, 'destroy'])->middleware('adminauth');

    Route::post('/admin/change-pass', [AuthController::class, 'adminChangePassword'])->middleware('adminauth');
    Route::post('/admin/change-role/{id}', [AuthController::class, 'updateAdminRole'])->middleware('adminauth');
    Route::get('/admins', [AuthController::class, 'allAdmins'])->middleware('adminauth');
    Route::delete('/admin/{id}', [AuthController::class, 'deleteAdmin'])->middleware('adminauth');
    Route::get('/admin/{id}', [AuthController::class, 'getById'])->middleware('adminauth');

    Route::post('/admin/user/status', [UserController::class, 'changeStatus'])->middleware('adminauth');
    Route::post('/admin/ads/upgrade', [ProfileController::class, 'adminUpgradeAds'])->middleware('adminauth');


    Route::get('/cities', [CityAndProvinceController::class, 'getCities']);
    Route::get('/cities/{id}', [CityAndProvinceController::class, 'getCityById']);

    Route::get('/provinces', [CityAndProvinceController::class, 'getProvinces']);
    Route::get('/provinces/{id}', [CityAndProvinceController::class, 'getProvinceById']);

    Route::get('/countries', [CountryController::class, 'get']);
    Route::get('/countries/{id}', [CountryController::class, 'getById']);

    Route::apiResource('/new-pages', PageController::class)
        ->only(['store']);

    Route::apiResource('/new-pages', PageController::class)
        ->only(['index', 'show']);

    Route::get('/new-pages/{id}', [PageController::class, 'show']);
    Route::post('/new-pages/{page}', [PageController::class, 'update']);
    Route::delete('/new-pages/{page}', [PageController::class, 'destroy']);

    Route::post('/doVerify', [AuthController::class, 'doVerify'])->name('doVerify');
    Route::post('/register', [AuthController::class, 'doRegister'])->name('doRegister');
    Route::post('/convertImageToUrl', [BusinessController::class, 'convertImageToUrl'])->name('convertImageToUrl');
    Route::post('/convertVideoToUrl', [BusinessController::class, 'convertVideoToUrl'])->name('convertVideoToUrl');
    Route::get('static_links', [MainController::class, 'static_links']);
//    Route::post('/pages', [HomeController::class, 'footer'])->middleware('adminauth');
    Route::post('/footer', [HomeController::class, 'footer'])->middleware('adminauth');
    Route::get('/pages', [HomeController::class, 'showPages']);
    Route::get('/pages/{key}', [HomeController::class, 'showPagesByKey']);
    Route::post('/pages/{key}', [HomeController::class, 'updatePagesByKey']);
    Route::get('show_footer', [HomeController::class, 'showFooter']);
    Route::post('/about', [HomeController::class, 'about'])->middleware('adminauth');
    Route::get('show_about', [HomeController::class, 'showAbout']);
    Route::get('/admin_panel', [HomeController::class, 'admin_panel'])->middleware('adminauth');
    Route::post('/info_count', [HomeController::class, 'count']);
    Route::post('/banner', [HomeController::class, 'banner']);
    Route::delete('/banner/{id}', [HomeController::class, 'deleteBanner']);
    Route::get('/get_banner', [HomeController::class, 'getBanner']);
    Route::get('/business/{id}/banner', [HomeController::class, 'getBannerById']);
    Route::post('/filter', [HomeController::class, 'fieldFilter']);


    Route::post('users', [UserController::class, 'authenticate'])->middleware('auth');
    Route::post('adminlogin', [AuthController::class, 'adminLogin']);
    Route::post('admin_Change_password', [AuthController::class, 'adminChangePassword'])->middleware('adminauth');
    Route::post('create_admin', [AuthController::class, 'addAdmin'])->middleware('adminauth');
    Route::get('users_archive', [UserController::class, 'users'])->middleware('auth');
    Route::get('user', [AuthController::class, 'user']);
    Route::get('/logout', [AuthController::class, 'logout'])->name('logout');

    Route::post('user/contact_us', [ContactUsContentController::class, 'store'])->middleware('auth');

    Route::prefix('profile')->group(function () {
        Route::get('/', [ProfileController::class, 'show'])->middleware('auth');
        Route::post('/showuserbyid', [ProfileController::class, 'showUserById'])->middleware('adminauth');
        Route::post('/delete_user', [ProfileController::class, 'delete'])->middleware('adminauth');
        Route::post('/my-account', [ProfileController::class, 'update'])->middleware('auth');
        Route::get('/business', [BusinessController::class, 'show'])->middleware('auth');
        Route::get('/business_archive', [BusinessController::class, 'showAllBusinesses'])->middleware('adminauth');
        Route::get('/business/special', [BusinessController::class, 'findSpecialBusiness']);
        Route::get('/business/{id}', [BusinessController::class, 'showByID']);
        Route::post('/business', [BusinessController::class, 'store'])->middleware('auth');
        Route::post('/business/update', [BusinessController::class, 'update'])->middleware('auth');
        Route::post('/business/changeBusinessStatus', [BusinessController::class, 'changeBusinessStatus'])->middleware('auth');
        Route::get('/my-ads', [ProfileController::class, 'myAds'])->middleware('auth');
        Route::get('/my-favorites', [ProfileController::class, 'myFavorites'])->middleware('auth');
        Route::delete('/delete_favorites', [ProfileController::class, 'deleteFavorites'])->middleware('auth');
        Route::get('/my-notes', [ProfileController::class, 'myNotes'])->middleware('auth');
        Route::delete('/delete_notes', [ProfileController::class, 'deleteNotes'])->middleware('auth');
        Route::get('/last-visited', [ProfileController::class, 'lastVisited'])->middleware('auth');
        Route::delete('/delete_last_visits', [ProfileController::class, 'deleteLastVisited'])->middleware('auth');
        Route::post('/my-payments', [ProfileController::class, 'myPayments'])->middleware('auth');
        Route::post('/fori', [ProfileController::class, 'fori'])->middleware('auth');
        Route::post('/nardeban', [ProfileController::class, 'nardeban'])->middleware('auth');
        Route::post('/renewal', [ProfileController::class, 'renewal'])->middleware('auth');
        Route::post('/create_discount_code', [ProfileController::class, 'createDiscountCode'])->middleware('adminauth');
        Route::post('/check_discount_codes', [ProfileController::class, 'checkDiscountCode']);
        Route::post('/product/purchase', [ProfileController::class, 'purchase'])->middleware('auth');
        Route::post('/product/purchase/tether', [ProfileController::class, 'payViaTether'])->middleware('auth');
        Route::post('/product/purchase/wallet', [ProfileController::class, 'payViaWallet'])->middleware('auth');
        Route::post('/product/purchase/usd', [ProfileController::class, 'usdPayment'])->middleware('auth');
        Route::post('/product/purchase/result', [ProfileController::class, 'result'])->middleware('auth');
        Route::get('/transactions', [ProfileController::class, 'transactions'])->middleware('auth');
    });

    Route::prefix('Ads')->group(function () {
        Route::get('/categories', [CategoryController::class, 'show']);

        Route::get('/cat-archive', [CategoryController::class, 'showCatArchive']);
        Route::post('/categories', [CategoryController::class, 'store'])->middleware('adminauth');
        Route::post('/show_by_id', [CategoryController::class, 'showCatById'])->middleware('adminauth');
        Route::post('/categories/update', [CategoryController::class,'update']);
        Route::delete('/categories/delete', [CategoryController::class,'destroy']);

        Route::get('/forms', [FormController::class,'show']);
        Route::get('/all-forms', [FormController::class,'showAll']);
        Route::get('/forms/category', [FormController::class,'showByCategory']);
        Route::post('/forms', [FormController::class,'store']);
        Route::post('/forms/update', [FormController::class,'update']);
        Route::delete('/forms', [FormController::class,'destroy']);

        Route::post('/product/add-favorite', [ProductController::class,'addToFavorite'])->middleware('auth');
        Route::post('/product/add-note', [ProductController::class,'addNote'])->middleware('auth');
        Route::post('/product', [ProductController::class,'addToFavorite'])->middleware('auth');
        Route::get('/product/{slug}/{product_id}', [ProductController::class,'showSingleProduct']);
        Route::get('/products', [ProductController::class,'show']);
        Route::get('/pending-products', [ProductController::class,'showPending'])->middleware('adminauth');
        Route::get('/expired-products', [ProductController::class,'showExpired'])->middleware('adminauth');
        Route::get('/rejected-products', [ProductController::class,'showRejected'])->middleware('adminauth');
        Route::get('/business/products', [ProductController::class,'showProductbyBusiness'])->middleware('auth');
        Route::get('/business/{business}/products', [ProductController::class, 'showProductByBusinessId']);
        Route::get('/business/product-count', [ProductController::class,'showBusinessProductCount'])->middleware('auth');
        Route::get('/products/product-feedback', [ProductController::class,'showProductFeedbacks'])->middleware('auth');
        Route::post('/products/product-feedback', [ProductController::class,'storeFeedback']);
        Route::post('/products', [ProductController::class, 'store'])->middleware('auth');
        Route::post('/products/update', [ProductController::class,'update']);
        Route::post('/products/search', [ProductController::class, 'search']);
        Route::post('/products/changeProductStatus', [ProductController::class,'changeProductStatus'])->middleware('adminauth');
        Route::post('/products/delete', [ProductController::class,'destroy'])->middleware('useradminauth');;
    });

    Route::prefix('support')->group(function () {

        Route::get('/contact', [SupportController::class,'show'])->middleware('auth');
        Route::post('/contact', [SupportController::class,'store']);
        Route::post('/postchi', [SupportController::class,'postchi'])->middleware('adminauth');
        Route::post('/show_user_postchi', [SupportController::class,'showUserPostchis'])->middleware('adminauth');
        Route::get('/postchi', [SupportController::class,'showPostchi'])->middleware('useradminauth');
        Route::get('/show_last_postchi', [SupportController::class,'showLastPostchi'])->middleware('auth');

    });

    Route::prefix('chat')->group(function () {

        Route::get('/', [ChatController::class,'show'])->middleware('auth');
        Route::post('/showChatReports', [ChatController::class,'showChatReports']);
        Route::post('/productchatreports', [ChatController::class,'productChatReports'])->middleware('adminauth');;
        Route::post('/', [ChatController::class,'store'])->middleware('auth');
        Route::post('/storeReport', [ChatController::class,'storeReport'])->middleware('auth');
        Route::post('/block', [ChatController::class,'chatBlock'])->middleware('auth');

        Route::post('/message', [ChatController::class,'storeMessage'])->middleware('auth');
        Route::get('/admin-messages', [ChatController::class, 'adminGet'])->middleware('adminauth');
        Route::get('/messages', [ChatController::class, 'get'])->middleware('auth');
        Route::delete('/', [ChatController::class,'deleteChats'])->middleware('auth');

        Route::get('/showSetting', [ChatController::class,'showSetting'])->middleware('auth');
        Route::post('/Setting', [ChatController::class,'Setting'])->middleware('auth');
        Route::post('/updateSetting', [ChatController::class,'updateSetting'])->middleware('auth');
        Route::delete('/', [ChatController::class,'deleteChats'])->middleware('auth');

    });

    Route::get('seos', [SeoController::class, 'get']);
    Route::get('/seos/single', [SeoController::class, 'single']);
    Route::post('seos', [SeoController::class, 'store'])->middleware(['adminauth', \App\Http\Middleware\TrimEmptyJsElement::class]);
    Route::post('seos/{id}', [SeoController::class, 'updateById'])->middleware('adminauth');
    Route::get('seos/{id}', [SeoController::class, 'getById']);
    Route::delete('seos/{id}', [SeoController::class, 'deleteById']);

});

